Package: BayesERtools
Type: Package
Title: Bayesian Exposure-Response Analysis Tools
Version: 0.2.6
Authors@R: 
    c(person("Kenta", "Yoshida", , "yoshida.kenta.6@gmail.com", role = c("aut", "cre"),
             comment = c(ORCID = "0000-0003-4967-3831")),
      person("François", "Mercier", , "francois.mercier@roche.com", role = c("aut"),
             comment = c(ORCID = "0000-0002-5685-1408")),
      person("Danielle", "Navarro", , "djnavarro@protonmail.com", c("aut"),
             comment = c(ORCID = "0000-0001-7648-6578")),
      person("Genentech, Inc.", role = "cph")
      )
Maintainer: Kenta Yoshida <yoshida.kenta.6@gmail.com>
Description: Suite of tools that facilitate
    exposure-response analysis using Bayesian methods. The package
    provides a streamlined workflow for fitting types of models that are
    commonly used in exposure-response analysis - linear and Emax for continuous
    endpoints, logistic linear and logistic Emax for binary endpoints, as well
    as performing simulation and visualization. Learn more about the workflow
    at <https://genentech.github.io/BayesERbook/>.
License: Apache License 2.0
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.1)
URL: https://genentech.github.io/BayesERtools/,
        https://genentech.github.io/BayesERbook/
Config/testthat/edition: 3
RoxygenNote: 7.3.3
Imports: dplyr, tidyr, purrr, ggplot2, gt, cli, rlang, rstanarm,
        rstanemax (>= 0.1.9), rstantools, loo, ggdist, posterior
Suggests: testthat (>= 3.0.0), covr, knitr, rmarkdown, rstan,
        htmltools, digest, ggforce, xgxr, scales, readr, bayestestR,
        patchwork, projpred, rsample, yardstick
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-06-13 05:28:34 UTC; yoshidk6
Author: Kenta Yoshida [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4967-3831>),
  François Mercier [aut] (ORCID: <https://orcid.org/0000-0002-5685-1408>),
  Danielle Navarro [aut] (ORCID: <https://orcid.org/0000-0001-7648-6578>),
  Genentech, Inc. [cph]
Repository: CRAN
Date/Publication: 2026-06-13 06:10:08 UTC
Built: R 4.5.2; ; 2026-06-13 10:16:06 UTC; unix

Package: BayesERtools
Type: Package
Title: Bayesian Exposure-Response Analysis Tools
Version: 0.2.6
Authors@R: 
    c(person("Kenta", "Yoshida", , "yoshida.kenta.6@gmail.com", role = c("aut", "cre"),
             comment = c(ORCID = "0000-0003-4967-3831")),
      person("François", "Mercier", , "francois.mercier@roche.com", role = c("aut"),
             comment = c(ORCID = "0000-0002-5685-1408")),
      person("Danielle", "Navarro", , "djnavarro@protonmail.com", c("aut"),
             comment = c(ORCID = "0000-0001-7648-6578")),
      person("Genentech, Inc.", role = "cph")
      )
Maintainer: Kenta Yoshida <yoshida.kenta.6@gmail.com>
Description: Suite of tools that facilitate
    exposure-response analysis using Bayesian methods. The package
    provides a streamlined workflow for fitting types of models that are
    commonly used in exposure-response analysis - linear and Emax for continuous
    endpoints, logistic linear and logistic Emax for binary endpoints, as well
    as performing simulation and visualization. Learn more about the workflow
    at <https://genentech.github.io/BayesERbook/>.
License: Apache License 2.0
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.1)
URL: https://genentech.github.io/BayesERtools/,
        https://genentech.github.io/BayesERbook/
Config/testthat/edition: 3
RoxygenNote: 7.3.3
Imports: dplyr, tidyr, purrr, ggplot2, gt, cli, rlang, rstanarm,
        rstanemax (>= 0.1.9), rstantools, loo, ggdist, posterior
Suggests: testthat (>= 3.0.0), covr, knitr, rmarkdown, rstan,
        htmltools, digest, ggforce, xgxr, scales, readr, bayestestR,
        patchwork, projpred, rsample, yardstick
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-06-13 05:28:34 UTC; yoshidk6
Author: Kenta Yoshida [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4967-3831>),
  François Mercier [aut] (ORCID: <https://orcid.org/0000-0002-5685-1408>),
  Danielle Navarro [aut] (ORCID: <https://orcid.org/0000-0001-7648-6578>),
  Genentech, Inc. [cph]
Repository: CRAN
Date/Publication: 2026-06-13 06:10:08 UTC

# Generated by roxygen2: do not edit by hand

S3method(as_draws,ermod)
S3method(as_draws_array,ermod)
S3method(as_draws_df,ermod)
S3method(as_draws_list,ermod)
S3method(as_draws_matrix,ermod)
S3method(as_draws_rvars,ermod)
S3method(bayestestR::p_direction,ermod_bin)
S3method(coef,ermod)
S3method(extract_data,ermod)
S3method(extract_data,ersim)
S3method(extract_data,ersim_med_qi)
S3method(extract_exp_sel_comp,ermod_exp_sel)
S3method(extract_exp_sel_list_model,ermod_exp_sel)
S3method(extract_mod,ermod)
S3method(extract_var_cov,ermod)
S3method(extract_var_cov,ersim)
S3method(extract_var_cov,ersim_med_qi)
S3method(extract_var_exposure,ermod)
S3method(extract_var_exposure,ersim)
S3method(extract_var_exposure,ersim_med_qi)
S3method(extract_var_resp,ermod)
S3method(extract_var_resp,ersim)
S3method(extract_var_resp,ersim_med_qi)
S3method(extract_var_selected,ermod_cov_sel)
S3method(kfold,ermod)
S3method(loo,ermod)
S3method(loo,ermod_bin_emax)
S3method(loo,ermod_emax)
S3method(plot,ermod_bin)
S3method(plot,ermod_cov_sel)
S3method(plot,ermod_exp_sel)
S3method(plot,ersim)
S3method(plot,ersim_med_qi)
S3method(plot_coveff,coveffsim)
S3method(plot_coveff,ermod)
S3method(plot_er,ermod)
S3method(plot_er,ersim)
S3method(plot_er,ersim_med_qi)
S3method(print,er_plot_components)
S3method(print,ermod)
S3method(print,ermod_cov_sel)
S3method(print,ermod_exp_sel)
S3method(print,kfold_ermod)
S3method(prior_summary,ermod)
S3method(summary,ermod)
export(.dev_ermod_refmodel)
export(.select_cov_projpred)
export(as_draws)
export(as_draws_array)
export(as_draws_df)
export(as_draws_list)
export(as_draws_matrix)
export(as_draws_rvars)
export(build_spec_coveff)
export(build_spec_coveff_one_variable)
export(calc_ersim_med_qi)
export(combine_er_components)
export(dev_ermod_bin)
export(dev_ermod_bin_cov_sel)
export(dev_ermod_bin_emax)
export(dev_ermod_bin_emax_exp_sel)
export(dev_ermod_bin_exp_sel)
export(dev_ermod_emax)
export(dev_ermod_emax_exp_sel)
export(dev_ermod_lin)
export(dev_ermod_lin_cov_sel)
export(dev_ermod_lin_exp_sel)
export(eval_ermod)
export(extract_coef_exp_ci)
export(extract_data)
export(extract_exp_sel_comp)
export(extract_exp_sel_list_model)
export(extract_kfold_loo)
export(extract_mod)
export(extract_var_cov)
export(extract_var_exposure)
export(extract_var_resp)
export(extract_var_selected)
export(kfold)
export(loo)
export(plot_coveff)
export(plot_er)
export(plot_er_exp_sel)
export(plot_er_gof)
export(plot_submod_performance)
export(plot_var_ranking)
export(print_coveff)
export(prior_summary)
export(replace_spec_coveff)
export(sim_coveff)
export(sim_er)
export(sim_er_curve)
export(sim_er_curve_marg)
export(sim_er_new_exp)
export(sim_er_new_exp_marg)
importFrom(loo,kfold)
importFrom(loo,loo)
importFrom(posterior,as_draws)
importFrom(posterior,as_draws_array)
importFrom(posterior,as_draws_df)
importFrom(posterior,as_draws_list)
importFrom(posterior,as_draws_matrix)
importFrom(posterior,as_draws_rvars)
importFrom(rstanarm,prior_summary)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# BayesERtools <a href="https://genentech.github.io/BayesERtools/"><img src="man/figures/logo.png" align="right" height="138" alt="BayesERtools website" /></a>

<!-- badges: start -->

[![R-CMD-check](https://github.com/Genentech/BayesERtools/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/Genentech/BayesERtools/actions/workflows/R-CMD-check.yaml)
[![CRAN
status](https://www.r-pkg.org/badges/version/BayesERtools)](https://CRAN.R-project.org/package=BayesERtools)
[![downloads](https://cranlogs.r-pkg.org/badges/grand-total/BayesERtools)](https://CRAN.R-project.org/package=BayesERtools)
[![Codecov test
coverage](https://codecov.io/gh/Genentech/BayesERtools/branch/main/graph/badge.svg)](https://app.codecov.io/gh/Genentech/BayesERtools?branch=main)
<!-- badges: end -->

`BayesERtools` provides a suite of tools that facilitate
exposure-response analysis using Bayesian methods.

- Tutorial (`BayesERbook`): <https://genentech.github.io/BayesERbook/>
- Package documentation: <https://genentech.github.io/BayesERtools/>
- GitHub repo of the package:
  <https://github.com/genentech/BayesERtools/>

## Installation

You can install the `BayesERtools` with:

``` r
install.packages('BayesERtools')
# devtools::install_github("genentech/BayesERtools") # development version
```

## Supported model types

<div id="pzveloiefp" style="padding-left:0px;padding-right:0px;padding-top:10px;padding-bottom:10px;overflow-x:auto;overflow-y:auto;width:auto;height:auto;">
  &#10;  <table class="gt_table" data-quarto-disable-processing="false" data-quarto-bootstrap="false">
  <thead>
    <tr class="gt_col_headings gt_spanner_row">
      <th class="gt_col_heading gt_columns_bottom_border gt_left" rowspan="2" colspan="1" style="text-align: center; vertical-align: top;" scope="col" id="feature_name"></th>
      <th class="gt_center gt_columns_top_border gt_column_spanner_outer" rowspan="1" colspan="2" scope="colgroup" id="Binary endpoint">
        <div class="gt_column_spanner">Binary endpoint</div>
      </th>
      <th class="gt_center gt_columns_top_border gt_column_spanner_outer" rowspan="1" colspan="2" scope="colgroup" id="Continuous endpoint">
        <div class="gt_column_spanner">Continuous endpoint</div>
      </th>
    </tr>
    <tr class="gt_col_headings">
      <th class="gt_col_heading gt_columns_bottom_border gt_left" rowspan="1" colspan="1" style="text-align: center; vertical-align: top;" scope="col" id="lin_logit">Linear (logit)</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_left" rowspan="1" colspan="1" style="text-align: center; vertical-align: top;" scope="col" id="emax_logit"><span class='gt_from_md'>E<sub/>max</sub> (logit)</span></th>
      <th class="gt_col_heading gt_columns_bottom_border gt_left" rowspan="1" colspan="1" style="text-align: center; vertical-align: top;" scope="col" id="linear">Linear</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_left" rowspan="1" colspan="1" style="text-align: center; vertical-align: top;" scope="col" id="emax"><span class='gt_from_md'>E<sub/>max</sub></span></th>
    </tr>
  </thead>
  <tbody class="gt_table_body">
    <tr><td headers="feature_name" class="gt_row gt_left" style="text-align: right; vertical-align: middle;"><span class='gt_from_md'>backend</span></td>
<td headers="lin_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'><code>rstanarm</code></span></td>
<td headers="emax_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'><code>rstanemax</code></span></td>
<td headers="linear" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'><code>rstanarm</code></span></td>
<td headers="emax" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'><code>rstanemax</code></span></td></tr>
    <tr><td headers="feature_name" class="gt_row gt_left" style="text-align: right; vertical-align: middle;"><span class='gt_from_md'>reference</span></td>
<td headers="lin_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span style="white-space: pre;"><a href="https://mc-stan.org/rstanarm/reference/stan_glm.html" target="_blank" style="color:#008B8B;text-decoration:none;display: inline-block;">🔗</a></span></td>
<td headers="emax_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span style="white-space: pre;"><a href="https://yoshidk6.github.io/rstanemax/reference/stan_emax.html" target="_blank" style="color:#008B8B;text-decoration:none;display: inline-block;">🔗</a></span></td>
<td headers="linear" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span style="white-space: pre;"><a href="https://mc-stan.org/rstanarm/reference/stan_glm.html" target="_blank" style="color:#008B8B;text-decoration:none;display: inline-block;">🔗</a></span></td>
<td headers="emax" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span style="white-space: pre;"><a href="https://yoshidk6.github.io/rstanemax/reference/stan_emax_binary.html" target="_blank" style="color:#008B8B;text-decoration:none;display: inline-block;">🔗</a></span></td></tr>
    <tr><td headers="feature_name" class="gt_row gt_left" style="text-align: right; vertical-align: middle;"><span class='gt_from_md'>develop model</span></td>
<td headers="lin_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="emax_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="linear" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="emax" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td></tr>
    <tr><td headers="feature_name" class="gt_row gt_left" style="text-align: right; vertical-align: middle;"><span class='gt_from_md'>simulate &amp; plot ER</span></td>
<td headers="lin_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="emax_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="linear" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="emax" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td></tr>
    <tr><td headers="feature_name" class="gt_row gt_left" style="text-align: right; vertical-align: middle;"><span class='gt_from_md'>exposure metrics selection</span></td>
<td headers="lin_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="emax_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="linear" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="emax" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td></tr>
    <tr><td headers="feature_name" class="gt_row gt_left" style="text-align: right; vertical-align: middle;"><span class='gt_from_md'>covariate selection</span></td>
<td headers="lin_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="emax_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>❌</span></td>
<td headers="linear" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="emax" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>❌</span></td></tr>
    <tr><td headers="feature_name" class="gt_row gt_left" style="text-align: right; vertical-align: middle;"><span class='gt_from_md'>covariate forest plot</span></td>
<td headers="lin_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="emax_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>❌</span></td>
<td headers="linear" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="emax" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>❌</span></td></tr>
  </tbody>
  &#10;  <tfoot class="gt_footnotes">
    <tr>
      <td class="gt_footnote" colspan="5"> ✅ Available, 🟡 In plan/under development, ❌ Not in a current plan</td>
    </tr>
  </tfoot>
</table>
</div>

## Quick guide

Here is a quick demo on how to use this package for E-R analysis. See
[Basic
workflow](https://genentech.github.io/BayesERbook/notebook/binary/basic_workflow.html)
for more thorough walk through.

``` r
# Load package and data
library(dplyr)
library(BayesERtools)
ggplot2::theme_set(ggplot2::theme_bw(base_size = 12))

data(d_sim_binom_cov)

# Hyperglycemia Grade 2+ (hgly2) data
df_er_ae_hgly2 <-
  d_sim_binom_cov |>
  filter(AETYPE == "hgly2") |>
  # Re-scale AUCss, baseline age
  mutate(
    AUCss_1000 = AUCss / 1000, BAGE_10 = BAGE / 10,
    Dose = paste(Dose_mg, "mg")
  )

var_resp <- "AEFLAG"
```

### Simple univariable model for binary endpoint

``` r
set.seed(1234)
ermod_bin <- dev_ermod_bin(
  data = df_er_ae_hgly2,
  var_resp = var_resp,
  var_exposure = "AUCss_1000"
)
ermod_bin
#> 
#> ── Binary ER model ─────────────────────────────────────────────────────────────
#> ℹ Use `plot_er()` to visualize ER curve
#> 
#> ── Developed model ──
#> 
#> stan_glm
#>  family:       binomial [logit]
#>  formula:      AEFLAG ~ AUCss_1000
#>  observations: 500
#>  predictors:   2
#> ------
#>             Median MAD_SD
#> (Intercept) -2.04   0.23 
#> AUCss_1000   0.41   0.08 
#> ------
#> * For help interpreting the printed output see ?print.stanreg
#> * For info on the priors used see ?prior_summary.stanreg

# Using `*` instead of `+` so that scale can be
# applied for both panels (main plot and boxplot)
plot_er_gof(ermod_bin, var_group = "Dose", show_coef_exp = TRUE) *
  xgxr::xgx_scale_x_log10(guide = ggplot2::guide_axis(minor.ticks = TRUE))
#> Warning in ggplot2::annotate("label", x = pos_ci_annot[1], y = pos_ci_annot[2],
#> : Ignoring unknown parameters: `label.size`
#> Warning: annotation$theme is not a valid theme.
#> Please use `theme()` to construct themes.
```

<img src="man/figures/README-ermod_bin-1.png" width="75%" />

### Covariate selection

BGLUC (baseline glucose) is selected while other two covariates are not.

``` r
set.seed(1234)
ermod_bin_cov_sel <-
  dev_ermod_bin_cov_sel(
    data = df_er_ae_hgly2,
    var_resp = var_resp,
    var_exposure = "AUCss_1000",
    var_cov_candidate = c("BAGE_10", "RACE", "BGLUC")
  )
#> 
#> ── Step 1: Full reference model fit ──
#> 
#> ── Step 2: Variable selection ──
#> 
#> ℹ The variables selected were: AUCss_1000, BGLUC
#> 
#> ── Step 3: Final model fit ──
#> 
#> ── Cov mod dev complete ──
#> 
ermod_bin_cov_sel
#> ── Binary ER model & covariate selection ───────────────────────────────────────
#> ℹ Use `plot_submod_performance()` to see variable selection performance
#> ℹ Use `plot_er()` with `marginal = TRUE` to visualize marginal ER curve
#> 
#> ── Selected model ──
#> 
#> stan_glm
#>  family:       binomial [logit]
#>  formula:      AEFLAG ~ AUCss_1000 + BGLUC
#>  observations: 500
#>  predictors:   3
#> ------
#>             Median MAD_SD
#> (Intercept) -7.59   0.90 
#> AUCss_1000   0.46   0.08 
#> BGLUC        0.87   0.13 
#> ------
#> * For help interpreting the printed output see ?print.stanreg
#> * For info on the priors used see ?prior_summary.stanreg
plot_submod_performance(ermod_bin_cov_sel)
```

<img src="man/figures/README-ermod_bin_cov_sel-1.png" width="75%" />

``` r
coveffsim <- sim_coveff(ermod_bin_cov_sel)
plot_coveff(coveffsim)
#> Warning in geom_errorbar(..., orientation = orientation): Ignoring unknown
#> parameters: `height`
```

<img src="man/figures/README-plot_coveff-1.png" width="75%" />

<!-- README.md is generated from README.Rmd. Please edit that file -->

# BayesERtools <a href="https://genentech.github.io/BayesERtools/"><img src="man/figures/logo.png" align="right" height="138" alt="BayesERtools website" /></a>

<!-- badges: start -->

[![R-CMD-check](https://github.com/Genentech/BayesERtools/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/Genentech/BayesERtools/actions/workflows/R-CMD-check.yaml)
[![CRAN
status](https://www.r-pkg.org/badges/version/BayesERtools)](https://CRAN.R-project.org/package=BayesERtools)
[![downloads](https://cranlogs.r-pkg.org/badges/grand-total/BayesERtools)](https://CRAN.R-project.org/package=BayesERtools)
[![Codecov test
coverage](https://codecov.io/gh/Genentech/BayesERtools/branch/main/graph/badge.svg)](https://app.codecov.io/gh/Genentech/BayesERtools?branch=main)
<!-- badges: end -->

`BayesERtools` provides a suite of tools that facilitate
exposure-response analysis using Bayesian methods.

- Tutorial (`BayesERbook`): <https://genentech.github.io/BayesERbook/>
- Package documentation: <https://genentech.github.io/BayesERtools/>
- GitHub repo of the package:
  <https://github.com/genentech/BayesERtools/>

## Installation

You can install the `BayesERtools` with:

``` r
install.packages('BayesERtools')
# devtools::install_github("genentech/BayesERtools") # development version
```

## Supported model types

<div id="pzveloiefp" style="padding-left:0px;padding-right:0px;padding-top:10px;padding-bottom:10px;overflow-x:auto;overflow-y:auto;width:auto;height:auto;">
  &#10;  <table class="gt_table" data-quarto-disable-processing="false" data-quarto-bootstrap="false">
  <thead>
    <tr class="gt_col_headings gt_spanner_row">
      <th class="gt_col_heading gt_columns_bottom_border gt_left" rowspan="2" colspan="1" style="text-align: center; vertical-align: top;" scope="col" id="feature_name"></th>
      <th class="gt_center gt_columns_top_border gt_column_spanner_outer" rowspan="1" colspan="2" scope="colgroup" id="Binary endpoint">
        <div class="gt_column_spanner">Binary endpoint</div>
      </th>
      <th class="gt_center gt_columns_top_border gt_column_spanner_outer" rowspan="1" colspan="2" scope="colgroup" id="Continuous endpoint">
        <div class="gt_column_spanner">Continuous endpoint</div>
      </th>
    </tr>
    <tr class="gt_col_headings">
      <th class="gt_col_heading gt_columns_bottom_border gt_left" rowspan="1" colspan="1" style="text-align: center; vertical-align: top;" scope="col" id="lin_logit">Linear (logit)</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_left" rowspan="1" colspan="1" style="text-align: center; vertical-align: top;" scope="col" id="emax_logit"><span class='gt_from_md'>E<sub/>max</sub> (logit)</span></th>
      <th class="gt_col_heading gt_columns_bottom_border gt_left" rowspan="1" colspan="1" style="text-align: center; vertical-align: top;" scope="col" id="linear">Linear</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_left" rowspan="1" colspan="1" style="text-align: center; vertical-align: top;" scope="col" id="emax"><span class='gt_from_md'>E<sub/>max</sub></span></th>
    </tr>
  </thead>
  <tbody class="gt_table_body">
    <tr><td headers="feature_name" class="gt_row gt_left" style="text-align: right; vertical-align: middle;"><span class='gt_from_md'>backend</span></td>
<td headers="lin_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'><code>rstanarm</code></span></td>
<td headers="emax_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'><code>rstanemax</code></span></td>
<td headers="linear" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'><code>rstanarm</code></span></td>
<td headers="emax" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'><code>rstanemax</code></span></td></tr>
    <tr><td headers="feature_name" class="gt_row gt_left" style="text-align: right; vertical-align: middle;"><span class='gt_from_md'>reference</span></td>
<td headers="lin_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span style="white-space: pre;"><a href="https://mc-stan.org/rstanarm/reference/stan_glm.html" target="_blank" style="color:#008B8B;text-decoration:none;display: inline-block;">🔗</a></span></td>
<td headers="emax_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span style="white-space: pre;"><a href="https://yoshidk6.github.io/rstanemax/reference/stan_emax.html" target="_blank" style="color:#008B8B;text-decoration:none;display: inline-block;">🔗</a></span></td>
<td headers="linear" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span style="white-space: pre;"><a href="https://mc-stan.org/rstanarm/reference/stan_glm.html" target="_blank" style="color:#008B8B;text-decoration:none;display: inline-block;">🔗</a></span></td>
<td headers="emax" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span style="white-space: pre;"><a href="https://yoshidk6.github.io/rstanemax/reference/stan_emax_binary.html" target="_blank" style="color:#008B8B;text-decoration:none;display: inline-block;">🔗</a></span></td></tr>
    <tr><td headers="feature_name" class="gt_row gt_left" style="text-align: right; vertical-align: middle;"><span class='gt_from_md'>develop model</span></td>
<td headers="lin_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="emax_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="linear" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="emax" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td></tr>
    <tr><td headers="feature_name" class="gt_row gt_left" style="text-align: right; vertical-align: middle;"><span class='gt_from_md'>simulate &amp; plot ER</span></td>
<td headers="lin_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="emax_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="linear" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="emax" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td></tr>
    <tr><td headers="feature_name" class="gt_row gt_left" style="text-align: right; vertical-align: middle;"><span class='gt_from_md'>exposure metrics selection</span></td>
<td headers="lin_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="emax_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="linear" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="emax" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td></tr>
    <tr><td headers="feature_name" class="gt_row gt_left" style="text-align: right; vertical-align: middle;"><span class='gt_from_md'>covariate selection</span></td>
<td headers="lin_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="emax_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>❌</span></td>
<td headers="linear" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="emax" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>❌</span></td></tr>
    <tr><td headers="feature_name" class="gt_row gt_left" style="text-align: right; vertical-align: middle;"><span class='gt_from_md'>covariate forest plot</span></td>
<td headers="lin_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="emax_logit" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>❌</span></td>
<td headers="linear" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>✅</span></td>
<td headers="emax" class="gt_row gt_left" style="text-align: center; vertical-align: middle;"><span class='gt_from_md'>❌</span></td></tr>
  </tbody>
  <tfoot>
    <tr class="gt_footnotes">
      <td class="gt_footnote" colspan="5"> ✅ Available, 🟡 In plan/under development, ❌ Not in a current plan</td>
    </tr>
  </tfoot>
</table>
</div>

## Quick guide

Here is a quick demo on how to use this package for E-R analysis. See
[Basic
workflow](https://genentech.github.io/BayesERbook/notebook/binary/basic_workflow.html)
for more thorough walk through.

``` r
# Load package and data
library(dplyr)
library(BayesERtools)
theme_set(theme_bw(base_size = 12))

data(d_sim_binom_cov)

# Hyperglycemia Grade 2+ (hgly2) data
df_er_ae_hgly2 <-
  d_sim_binom_cov |>
  filter(AETYPE == "hgly2") |>
  # Re-scale AUCss, baseline age
  mutate(
    AUCss_1000 = AUCss / 1000, BAGE_10 = BAGE / 10,
    Dose = paste(Dose_mg, "mg")
  )

var_resp <- "AEFLAG"
```

### Simple univariable model for binary endpoint

``` r
set.seed(1234)
ermod_bin <- dev_ermod_bin(
  data = df_er_ae_hgly2,
  var_resp = var_resp,
  var_exposure = "AUCss_1000"
)
ermod_bin
#> 
#> ── Binary ER model ─────────────────────────────────────────────────────────────
#> ℹ Use `plot_er()` to visualize ER curve
#> 
#> ── Developed model ──
#> 
#> stan_glm
#>  family:       binomial [logit]
#>  formula:      AEFLAG ~ AUCss_1000
#>  observations: 500
#>  predictors:   2
#> ------
#>             Median MAD_SD
#> (Intercept) -2.04   0.23 
#> AUCss_1000   0.41   0.08 
#> ------
#> * For help interpreting the printed output see ?print.stanreg
#> * For info on the priors used see ?prior_summary.stanreg

# Using `*` instead of `+` so that scale can be
# applied for both panels (main plot and boxplot)
plot_er_gof(ermod_bin, var_group = "Dose", show_coef_exp = TRUE) *
  coord_transform(x = "log10") *
  scale_x_continuous(
    breaks = xgxr::xgx_breaks_log10,
    minor_breaks = xgxr::xgx_minor_breaks_log10
  )
```

<img src="man/figures/README-ermod_bin-1.png" alt="" width="75%" />

### Covariate selection

BGLUC (baseline glucose) is selected while other two covariates are not.

``` r
set.seed(1234)
ermod_bin_cov_sel <-
  dev_ermod_bin_cov_sel(
    data = df_er_ae_hgly2,
    var_resp = var_resp,
    var_exposure = "AUCss_1000",
    var_cov_candidate = c("BAGE_10", "RACE", "BGLUC")
  )
#> 
#> ── Step 1: Full reference model fit ──
#> 
#> ── Step 2: Variable selection ──
#> 
#> ℹ The variables selected were: AUCss_1000, BGLUC
#> 
#> ── Step 3: Final model fit ──
#> 
#> ── Cov mod dev complete ──
#> 
ermod_bin_cov_sel
#> ── Binary ER model & covariate selection ───────────────────────────────────────
#> ℹ Use `plot_submod_performance()` to see variable selection performance
#> ℹ Use `plot_er()` with `marginal = TRUE` to visualize marginal ER curve
#> 
#> ── Selected model ──
#> 
#> stan_glm
#>  family:       binomial [logit]
#>  formula:      AEFLAG ~ AUCss_1000 + BGLUC
#>  observations: 500
#>  predictors:   3
#> ------
#>             Median MAD_SD
#> (Intercept) -7.59   0.90 
#> AUCss_1000   0.46   0.08 
#> BGLUC        0.87   0.13 
#> ------
#> * For help interpreting the printed output see ?print.stanreg
#> * For info on the priors used see ?prior_summary.stanreg
plot_submod_performance(ermod_bin_cov_sel)
```

<img src="man/figures/README-ermod_bin_cov_sel-1.png" alt="" width="75%" />

``` r
coveffsim <- sim_coveff(ermod_bin_cov_sel)
plot_coveff(coveffsim)
```

<img src="man/figures/README-plot_coveff-1.png" alt="" width="75%" />

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----echo = FALSE, warning=FALSE, message=FALSE-------------------------------
library(gt)
library(dplyr)
set.seed(1234) # Needed to stablize div id

# Saved csv from README.Rmd
tab_mod <- read.csv("data/supported_models.csv")

tab_mod |>
  select(-.row_id) |>
  gt() |>
  fmt_markdown() |>
  fmt_url(
    columns = !1,
    rows = 2,
    label = "🔗",
    show_underline = FALSE
  ) |>
  tab_spanner(
    label = "Binary endpoint",
    columns = c(lin_logit, emax_logit)
  ) |>
  tab_spanner(
    label = "Continuous endpoint",
    columns = c(linear, emax)
  ) |>
  cols_label(
    feature_name = "",
    lin_logit = "Linear  \n(logit)",
    emax_logit = md("E<sub/>max</sub> (logit)"),
    linear = "Linear",
    emax = md("E<sub/>max</sub>"),
  ) |>
  tab_style(
    style = cell_text(v_align = "top", align = "center"),
    locations = cells_column_labels()
  ) |>
  tab_style(
    style = cell_text(v_align = "middle", align = "center"),
    locations = cells_body()
  ) |>
  tab_style(
    style = cell_text(v_align = "middle", align = "right"),
    locations = cells_body(columns = feature_name)
  ) |>
  tab_footnote(
    footnote = paste(
      "✅ Available",
      "🟡 In plan/under development",
      "❌ Not in a current plan",
      sep = ", "
    )
  )

# Variable selection ----------------------------------------------------------

#' Plot variable selection performance
#' @export
#' @name plot_cov_sel
#' @param x An object of class \code{ermod_bin_cov_sel}
#' @details
#' [plot_submod_performance()] plots the performance of submodels
#' evaluated during variable selection.
#'
#' @return No return value, called for plotting side effect.
#' @examplesIf BayesERtools:::.if_run_ex_covsel()
#' \donttest{
#' data(d_sim_binom_cov_hgly2)
#'
#' er_binary_cov_model_kfold <- dev_ermod_bin_cov_sel(
#'   data = d_sim_binom_cov_hgly2,
#'   var_resp = "AEFLAG",
#'   var_exposure = "AUCss_1000",
#'   var_cov_candidate = c(
#'     "BAGE_10", "BWT_10", "BGLUC",
#'     "BHBA1C_5", "RACE", "VISC"
#'   ),
#'   cv_method = "kfold",
#'   k = 3, # Choose 3 to make the example go fast
#'   validate_search = TRUE,
#' )
#'
#' plot_submod_performance(er_binary_cov_model_kfold)
#' plot_var_ranking(er_binary_cov_model_kfold)
#' }
plot_submod_performance <- function(x) {
  stopifnot(inherits(x, "ermod_cov_sel"))
  plot(x$cvvs, text_angle = 45, show_cv_proportions = FALSE, deltas = TRUE)
}


#' @export
#' @rdname plot_cov_sel
#' @details
#' [plot_var_ranking()] plots the variable ranking
#' evaluated during variable selection.
plot_var_ranking <- function(x) {
  if (is.null(x$rk$foldwise)) {
    stop(
      'ranking only supported for "kfold" with ',
      'validate_search = TRUE"'
    )
  }
  plot(x$rk)
}


# Exposure metric selection ---------------------------------------------------

#' Plot exposure metric selection comparison
#'
#' Plot ER curve for each exposure metric and compare them.
#'
#' @export
#' @param x An object of class `ermod_bin_exp_sel`
#' @param n_draws_sim Number of draws to simulate response for each exposure
#' value. Default is NULL (use all draws in the model object)
#'
#' @return No return value, called for plotting side effect.
#' @examplesIf BayesERtools:::.if_run_ex_plot_er()
#' \donttest{
#' data(d_sim_binom_cov_hgly2)
#'
#' ermod_bin_exp_sel <-
#'   dev_ermod_bin_exp_sel(
#'     data = d_sim_binom_cov_hgly2,
#'     var_resp = "AEFLAG",
#'     var_exp_candidates = c("AUCss_1000", "Cmaxss", "Cminss")
#'   )
#'
#' plot_er_exp_sel(ermod_bin_exp_sel) +
#'   xgxr::xgx_scale_x_log10(guide = ggplot2::guide_axis(minor.ticks = TRUE))
#' }
#'
plot_er_exp_sel <- function(x, n_draws_sim = NULL) {
  stopifnot(inherits(x, "ermod_exp_sel"))
  is_binary_mod <- inherits(x, c("ermod_bin", "ermod_bin_emax"))

  var_resp <- extract_var_resp(x)
  var_exp_order <-
    rownames(x$loo_comp_exposures)

  df_map_var_exp <-
    dplyr::tibble(.exp_metric = var_exp_order) |>
    dplyr::mutate(
      .exp_met_fct = factor(.exp_metric, levels = unique(.exp_metric))
    )

  origdata <-
    extract_data(x) |>
    tidyr::pivot_longer(dplyr::all_of(var_exp_order),
      names_to = ".exp_metric", values_to = ".exposure"
    ) |>
    dplyr::left_join(df_map_var_exp, by = ".exp_metric")

  data_sim <-
    purrr::map2(
      x$l_mod_exposures,
      names(x$l_mod_exposures),
      function(.x, .y) sim_er_each_exp_met(.x, .y, n_draws_sim = n_draws_sim)
    ) |>
    purrr::list_rbind() |>
    dplyr::left_join(df_map_var_exp, by = ".exp_metric")

  gg <-
    ggplot2::ggplot(
      data = data_sim,
      ggplot2::aes(x = .exposure, y = .epred)
    ) +
    ggplot2::geom_ribbon(
      ggplot2::aes(ymin = .epred.lower, ymax = .epred.upper),
      alpha = 0.3
    ) +
    ggplot2::geom_line() +
    ggplot2::facet_wrap(~.exp_met_fct, scales = "free_x") +
    ggplot2::labs(x = "Exposure")

  if (is_binary_mod) {
    gg <-
      gg +
      ggplot2::scale_y_continuous(
        breaks = c(0, .5, 1),
        labels = scales::percent
      ) +
      ggplot2::geom_jitter(
        data = origdata,
        ggplot2::aes(y = .data[[var_resp]]),
        width = 0, height = 0.05, alpha = 0.5
      ) +
      xgxr::xgx_stat_ci(
        data = origdata,
        ggplot2::aes(y = .data[[var_resp]]),
        bins = 4, conf_level = 0.95, distribution = "binomial",
        geom = c("point"), shape = 0, size = 4
      ) +
      xgxr::xgx_stat_ci(
        data = origdata,
        ggplot2::aes(y = .data[[var_resp]]),
        bins = 4, conf_level = 0.95, distribution = "binomial",
        geom = c("errorbar"), linewidth = 0.5
      ) +
      ggplot2::labs(y = "Probability of event") +
      ggplot2::coord_cartesian(ylim = c(-0.05, 1.05))
  } else {
    gg <- gg +
      ggplot2::geom_ribbon(
        ggplot2::aes(ymin = .prediction.lower, ymax = .prediction.upper),
        alpha = 0.15
      ) +
      ggplot2::geom_point(
        data = origdata,
        ggplot2::aes(y = .data[[var_resp]])
      ) +
      ggplot2::labs(y = "Response")
  }

  return(gg)
}


sim_er_each_exp_met <- function(.x, .y, n_draws_sim) {
  sim <- sim_er_curve(.x, output_type = "median_qi", n_draws_sim = n_draws_sim)
  sim$.exposure <- sim[[.y]]
  sim[[.y]] <- NULL
  sim$.exp_metric <- .y
  sim
}

# Set globalVariables to minimize R CMD check notes


if (getRversion() >= "2.15.1") {
  # General
  utils::globalVariables(c(
    ".", "n", ":=", ".epred", ".linpred", ".prediction", ".draw", ".response",
    ".id_exposure", ".chain", ".iteration", ".row"
  ))

  # ggplot
  utils::globalVariables(c(
    ".data", ".lower", ".upper", ".exposure", ".exp_metric",
    ".epred.lower", ".epred.upper", ".linpred.lower", ".linpred.upper",
    ".prediction.lower", ".prediction.upper"
  ))

  # coveff
  utils::globalVariables(c(
    "var_name", "var_order", "value_label", "value_cont", "value_cat",
    "value_order", "value_annot", "value_order", "var_label",
    "var_value_index_num", ".response_diff",
    "is_covariate", "is_ref_value", "show_ref_value",
    "is_ref", ".linpred_ref", ".delta_linpred", ".is_ref_value",
    ".row_id", "df_one_row", "df_one_row_new", ".odds_ratio",
    "Odds ratio", "95% CI"
  ))

  # rstanemax
  utils::globalVariables(c(
    "mcmcid", "respHat", "response", ".upper", ".exposure", "exp_metric"
  ))

  # eval_ermod and cv
  utils::globalVariables(c(
    "truth", "pred", "fold_id", ".row_orig"
  ))

  # sim data gen
  utils::globalVariables(c(
    "exposure", "cnt_a", "cnt_b", "cnt_c", "bin_d", "bin_pred", "bin_prob"
  ))
}


#' Sample simulated data for exposure-response with binary endpoint.
#'
#' @name d_sim_binom_cov
#' @format A data frame with columns:
#' \describe{
#' \item{ID}{Subject ID}
#' \item{AETYPE}{Adverse event type: hgly2 (Gr2+ hyperglycemia),
#' dr2 (Gr2+ Diarrhea), ae_covsel_test (hypothetical AE for covariate
#' selection function test)}
#' \item{AEFLAG}{Adverse event flag: 0 - no event, 1 - event}
#' \item{Dose_mg}{Dose in mg: 200, 400}
#' \item{AUCss}{Steady-state area under the curve}
#' \item{Cmaxss}{Steady-state maximum (peak) concentration}
#' \item{Cminss}{Steady-state minimum (trough) concentration}
#' \item{BAGE}{Baseline age in years}
#' \item{BWT}{Baseline weight in kg}
#' \item{BGLUC}{Baseline glucose in mmol/L}
#' \item{BHBA1C}{Baseline HbA1c in percentage}
#' \item{RACE}{Race: White, Black, Asian}
#' \item{VISC}{Visceral disease: No, Yes}
#' \item{AUCss_1000}{AUCss/1000}
#' \item{BAGE_10}{BAGE/10}
#' \item{BWT_10}{BWT/10}
#' \item{BHBA1C_5}{BHBA1C/5}
#' }
#' @details
#'
#' This simulated dataset is very loosely inspired by ER analysis of
#' ipatasertib by Kotani (2022) at:
#'
#' https://doi.org/10.1007/s00280-022-04488-2
#'
#' You can find the data generating code in the package source code,
#' under `data-raw/d_sim_binom_cov.R`.
#'
#' d_sim_binom_cov_hgly2 is a subset of this dataset with only hgly2 AE type
#' and some columns added for testing.
#'
#' @examples
#' d_sim_binom_cov
#' d_sim_binom_cov_hgly2
"d_sim_binom_cov"

#' @rdname d_sim_binom_cov
"d_sim_binom_cov_hgly2"


#' Sample simulated data for exposure-response with continuous endpoint
#' using linear model.
#'
#' @format A data frame with columns:
#' \describe{
#' \item{ID}{Subject ID}
#' \item{AUCss}{Steady-state area under the curve}
#' \item{Cmaxss}{Steady-state maximum (peak) concentration}
#' \item{BAGE}{Baseline age in years}
#' \item{SEX}{M or F}
#' \item{response}{Response}
#' }
#' @details
#'
#' True model is defined as `0.5 * AUCss + 0.5 * BAGE + 5 * SEX`, with
#' variability added with standard deviation of 10.
#' You can find the data generating code in the package source code,
#' under `data-raw/d_sim_lin.R`.
#'
#' @examples
#' d_sim_lin
"d_sim_lin"



#' Sample simulated data for Emax exposure-response models with covariates.
#'
#' @name d_sim_emax
#' @format A data frame with columns:
#' \describe{
#' \item{dose}{Nominal dose, units not specified}
#' \item{exposure}{Exposure value, units and metric not specified}
#' \item{response_1}{Continuous response value (units not specified)}
#' \item{response_2}{Binary response value (group labels not specified)}
#' \item{cnt_a}{Continuous valued covariate}
#' \item{cnt_b}{Continuous valued covariate}
#' \item{cnt_c}{Continuous valued covariate}
#' \item{bin_d}{Binary valued covariate}
#' \item{bin_e}{Binary valued covariate}
#' }
#' @details
#'
#' This simulated dataset is entirely synthetic. It is a generic data set that can be used
#' to illustrate Emax modeling. It contains variables corresponding to dose and exposure,
#' and includes both a continuous response variable and a binary response variable. Three
#' continuous valued covariates are included, along with two binary covariates.
#'
#' You can find the data generating code in the package source code,
#' under `data-raw/d_sim_emax.R`.
#'
#' @examples
#' d_sim_emax
"d_sim_emax"


#' Sample simulated data for Emax exposure-response models with covariates and placebo
#'
#' @name d_sim_placebo
#' @format A data frame with columns:
#' \describe{
#' \item{id}{Subject identifier}
#' \item{dose}{Nominal dose, units not specified}
#' \item{exp_1}{Exposure value on metric 1, units and metric not specified}
#' \item{exp_2}{Exposure value on metric 2, units and metric not specified}
#' \item{rsp_1}{Continuous response value (units not specified)}
#' \item{rsp_2}{Binary response value (group labels not specified)}
#' \item{cnt_a}{Continuous valued covariate}
#' \item{cnt_b}{Continuous valued covariate}
#' \item{cnt_c}{Continuous valued covariate}
#' \item{bin_d}{Binary valued covariate}
#' \item{bin_e}{Binary valued covariate}
#' }
#' @details
#'
#' This simulated dataset is entirely synthetic. It is a generic data set that can be used
#' to illustrate Emax modeling with placebo data. It contains variables corresponding to
#' dose and exposure, and includes both a continuous response variable and a binary response
#' variable. Three continuous valued covariates are included, along with two binary covariates.
#' Data from two exposure metrics are included.
#'
#' You can find the data generating code in the package source code,
#' under `data-raw/d_sim_placebo.R`.
#'
#' @examples
#' d_sim_placebo
"d_sim_placebo"

#' Extract elements from S3 objects
#'
#' S3 methods are defined for `ermod_*` (see [extract_ermod]) and
#' `ersim_*` (see [extract_ersim]) classes.
#'
#' @name extract_method
#' @param x An object to extract elements from
#' @return
#' - [extract_data()] extracts data used for the model fit.
#' - [extract_mod()] extracts the model fit object.
#' - [extract_var_resp()] extracts the response variable name
#' - [extract_var_exposure()] extracts the exposure metric name
#' - [extract_var_cov()] extracts the covariates name
#' - [extract_exp_sel_list_model()] extracts the list of fitted models for
#'   each exposure metrics.
#' - [extract_exp_sel_comp()] extracts the comparison results of the exposure
#'   metrics.
#' - [extract_var_selected()] extracts the selected variables (both exposure
#'   and covariates)in the final model after covariate selection.
NULL

#' @export
#' @rdname extract_method
extract_data <- function(x) UseMethod("extract_data")

#' @export
#' @rdname extract_method
extract_mod <- function(x) UseMethod("extract_mod")

#' @export
#' @rdname extract_method
extract_var_resp <- function(x) UseMethod("extract_var_resp")

#' @export
#' @rdname extract_method
extract_var_exposure <- function(x) UseMethod("extract_var_exposure")

#' @export
#' @rdname extract_method
extract_var_cov <- function(x) UseMethod("extract_var_cov")

#' @export
#' @rdname extract_method
extract_exp_sel_list_model <- function(x) {
  UseMethod("extract_exp_sel_list_model")
}

#' @export
#' @rdname extract_method
extract_exp_sel_comp <- function(x) UseMethod("extract_exp_sel_comp")

#' @export
#' @rdname extract_method
extract_var_selected <- function(x) UseMethod("extract_var_selected")

#' Simulate from ER model
#'
#' @export
#' @name sim_er
#' @param ermod An object of class \code{ermod}
#' @param newdata New data to use for simulation.
#' Default is NULL (use the data in the model object).
#' @param n_draws_sim Number of draws for simulation. If NULL (default),
#' all draws in the model object are used.
#' @param seed_sample_draws Seed for sampling draws. Default is NULL.
#' @param output_type Type of output. "draws" returns the raw draws from the
#' simulation, and "median_qi" returns the median and quantile interval.
#' @param qi_width Width of the quantile interval. Default is 0.95. Only
#' used when `output_type = "median_qi"`.
#' @param .nrow_cov_data Number of rows in the covariate data,
#' used for internal purposes. Users should not set this argument.
#'
#' @return `ersim` object, which is a tibble with the simulated responses
#' with some additional information in object attributes.
#' It has three types of predictions - `.linpred`, `.epred`, `.prediction`.
#' `.linpred` and `.epred` are similar in a way that they both represent
#' "expected response", i.e. without residual variability. They are the same
#' for models with continuous endpoits (Emax model). For models with binary
#' endpoints, `.linpred` is the linear predictor (i.e. on the logit scale) and
#' `.epred` is on the probability scale. `.prediction` is the predicted
#' response with residual variability (or in case of binary endpoint,
#' the predicted yes (1) or no (0) for event occurrence).
#' See [rstantools::posterior_epred()] for more details.
#'
#' In case of `output_type = "median_qi"`, it returns `ersim_med_qi` object.
#'
#' @seealso [calc_ersim_med_qi()] for calculating median and quantile interval
#' from `ersim` object (generated with `output_type = "draws"`).
#'
#' @examples
#' \donttest{
#' data(d_sim_binom_cov_hgly2)
#'
#' ermod_bin <- dev_ermod_bin(
#'   data = d_sim_binom_cov_hgly2,
#'   var_resp = "AEFLAG",
#'   var_exposure = "AUCss_1000",
#'   var_cov = "BHBA1C_5",
#' )
#'
#' ersim <- sim_er(
#'   ermod_bin,
#'   n_draws_sim = 500, # This is set to make the example run faster
#'   output_type = "draws"
#' )
#'
#' ersim_med_qi <- sim_er(
#'   ermod_bin,
#'   n_draws_sim = 500, # This is set to make the example run faster
#'   output_type = "median_qi"
#' )
#'
#' ersim
#' ersim_med_qi
#' }
#'
sim_er <- function(
    ermod,
    newdata = NULL,
    n_draws_sim = NULL,
    seed_sample_draws = NULL,
    output_type = c("draws", "median_qi"),
    qi_width = 0.95,
    .nrow_cov_data = NULL) {
  stopifnot(inherits(ermod, "ermod"))
  output_type <- match.arg(output_type)

  if (is.null(seed_sample_draws)) {
    seed_sample_draws <- sample.int(.Machine$integer.max, 1)
  }

  if (is.null(newdata)) newdata <- extract_data(ermod)
  check_data_columns(newdata,
    var_exposure = ermod$var_exposure,
    var_cov = ermod$var_cov
  )
  if (is.null(.nrow_cov_data)) {
    .nrow_cov_data <- nrow(newdata)
  }

  mod <- extract_mod(ermod)
  n_draws_sim <- chech_ndraws(mod, n_draws_sim)

  simdata_epred <-
    .pp_matrix_to_draws_tbl(
      .posterior_draws(rstantools::posterior_epred, mod, newdata,
        n_draws_sim, seed_sample_draws
      ),
      newdata, ".epred"
    )
  simdata_linpred <-
    .pp_matrix_to_draws_tbl(
      .posterior_draws(rstantools::posterior_linpred, mod, newdata,
        n_draws_sim, seed_sample_draws
      ),
      newdata, ".linpred"
    ) |>
    dplyr::select(.draw, .row, .linpred)
  simdata_predicted <-
    .pp_matrix_to_draws_tbl(
      .posterior_draws(rstantools::posterior_predict, mod, newdata,
        n_draws_sim, seed_sample_draws
      ),
      newdata, ".prediction"
    ) |>
    dplyr::select(.draw, .row, .prediction)

  simdata <-
    simdata_epred |>
    dplyr::left_join(simdata_linpred, by = dplyr::join_by(.draw, .row)) |>
    dplyr::left_join(simdata_predicted, by = dplyr::join_by(.draw, .row))

  if (output_type == "draws") {
    return(new_ersim(
      simdata,
      ermod,
      nrow_cov_data = .nrow_cov_data
    ))
  }

  simdata_med_qi <-
    simdata |>
    dplyr::group_by(dplyr::across(
      !c(.chain, .iteration, .draw, .epred, .linpred, .prediction)
    )) |>
    ggdist::median_qi(.width = qi_width) |>
    dplyr::as_tibble()

  if (inherits(ermod, "ermod_bin")) {
    simdata_med_qi <-
      simdata_med_qi |>
      dplyr::select(-dplyr::starts_with(".prediction"))
  }

  return(new_ersim_med_qi(simdata_med_qi, ermod,
    nrow_cov_data = .nrow_cov_data,
    qi_width = qi_width
  ))
}


#' Simulate from ER model at specified exposure values
#'
#' @export
#' @name sim_er_new_exp
#' @inheritParams sim_er
#' @inherit sim_er return
#' @param exposure_to_sim_vec Vector of exposure values to simulate.
#' @param data_cov Data frame containing covariates to use for simulation,
#' see details below.
#'
#' @details
#' Simulation dataset will be all combinations of covariates in `data_cov`
#' and exposure values in `exposure_to_sim_vec`,
#' so the run time can become very long if `data_cov` has many rows.
#'
#' `data_cov` has to be supplied if `ermod` is a model with covariates.
#' It is recommended that `data_cov` contains subject identifiers such as
#' `ID` for post-processing.
#'
#' Exposure values in `data_cov` will be ignored.
#'
#' @seealso [calc_ersim_med_qi()] for calculating median and quantile interval
#' from `ersim` object (generated with `output_type = "draws"`).
#'
#' @examples
#' \donttest{
#' data(d_sim_binom_cov_hgly2)
#'
#' ermod_bin <- dev_ermod_bin(
#'   data = d_sim_binom_cov_hgly2,
#'   var_resp = "AEFLAG",
#'   var_exposure = "AUCss_1000",
#'   var_cov = "BHBA1C_5",
#' )
#'
#' ersim_new_exp_med_qi <- sim_er_new_exp(
#'   ermod_bin,
#'   exposure_to_sim_vec = seq(2, 6, by = 0.2),
#'   data_cov = dplyr::tibble(BHBA1C_5 = 4:10),
#'   n_draws_sim = 500, # This is set to make the example run faster
#'   output_type = "median_qi"
#' )
#'
#' ersim_new_exp_med_qi
#' }
#'
sim_er_new_exp <- function(
    ermod,
    exposure_to_sim_vec = NULL,
    data_cov = NULL,
    n_draws_sim = NULL,
    seed_sample_draws = NULL,
    output_type = c("draws", "median_qi"),
    qi_width = 0.95) {
  stopifnot(inherits(ermod, "ermod"))
  output_type <- match.arg(output_type)

  var_exposure_sym <- rlang::sym(extract_var_exposure(ermod))

  newdata <- dplyr::tibble(!!var_exposure_sym := exposure_to_sim_vec)

  var_cov <- extract_var_cov(ermod)

  # Handle data_cov
  ## No need to use data_cov if there are no covariates in the model
  if (is.null(var_cov)) {
    .nrow_cov_data <- 0
  }
  if (!is.null(var_cov) && is.null(data_cov)) {
    stop("data_cov must be supplied for models with covariates.")
  }
  ## This is the only situation where data_cov is used
  if (!is.null(var_cov) && !is.null(data_cov)) {
    .nrow_cov_data <- nrow(data_cov)

    # remove exposure column from data_cov, if it exists
    if (extract_var_exposure(ermod) %in% colnames(data_cov)) {
      data_cov <- data_cov |> dplyr::select(-!!var_exposure_sym)
    }
    newdata <- tidyr::expand_grid(newdata, data_cov)
  }

  sim_er(
    ermod = ermod,
    newdata = newdata,
    n_draws_sim = n_draws_sim,
    seed_sample_draws = seed_sample_draws,
    output_type = output_type,
    qi_width = qi_width,
    .nrow_cov_data = .nrow_cov_data
  )
}


#' @export
#' @rdname sim_er_new_exp
#' @param exposure_range Range of exposure values to simulate. If NULL
#' (default), it is set to the range of the exposure variable in the original
#' data for model development.
#' @param num_exposures Number of exposure values to simulate.
#'
#' @details
#' [sim_er_curve()] is a wrapper function for [sim_er_new_exp()]
#' that use a range of exposure values to simulate the expected responses.
#' Particularly useful for plotting the exposure-response curve.
#'
sim_er_curve <- function(
    ermod,
    exposure_range = NULL,
    num_exposures = 51,
    data_cov = NULL,
    n_draws_sim = NULL,
    seed_sample_draws = NULL,
    output_type = c("draws", "median_qi"),
    qi_width = 0.95) {
  if (is.null(exposure_range)) {
    exposure_range <-
      range(extract_data(ermod)[[extract_var_exposure(ermod)]])
  } else {
    stopifnot(length(exposure_range) == 2)
  }

  exposure_to_sim_vec <-
    seq(exposure_range[1], exposure_range[2], length.out = num_exposures)

  sim_er_new_exp(
    ermod = ermod,
    exposure_to_sim_vec = exposure_to_sim_vec,
    data_cov = data_cov,
    n_draws_sim = n_draws_sim,
    seed_sample_draws = seed_sample_draws,
    output_type = output_type,
    qi_width = qi_width
  )
}


#' Calculate marginal expected response for specified exposure values
#'
#' Responses at specified exposure values are calculated for `n_subj_sim`
#' subjects with different covariates (sampled from `newdata`),
#' and the predicted responses are "marginalized" (averaged),
#' resulting in marginal expected response on the
#' population of interest.
#'
#' @export
#' @name sim_er_new_exp_marg
#' @inheritParams sim_er_new_exp
#' @param data_cov Data frame containing covariates to use for simulation.
#' Different from [sim_er_new_exp()], `data_cov` can be large as long as
#' `n_subj_sim` is set to a reasonable number. Default is set to
#' `extract_data(ermod)` which is the full data used to fit the model.
#' @param n_subj_sim Maximum number of subjects to simulate. Default of 100
#' should be sufficient in many cases, as it's only used for marginal
#' response calculation.
#' Set to NULL to use all subjects in `data_cov` without resampling;
#' in this case, be mindful of the computation time.
#' @param n_draws_sim Number of draws for simulation. Default is set to 500
#' to reduce computation time for marginal response calculation.
#'
#' @return `ersim_marg` object, which is a tibble with the simulated marginal
#' expected response with some additional information in object attributes.
#' In case of `output_type = "median_qi"`, it returns `ersim_marg_med_qi`
#'  object.
#'
#' @details
#' [sim_er_new_exp_marg()] returns a tibble with the marginal expected
#' response for each exposure value in `exposure_to_sim_vec`.
#'
#' @seealso [calc_ersim_med_qi()] for calculating median and quantile interval
#' from `ersim_marg` object (generated with `output_type = "draws"`).
#'
#' @examples
#' \donttest{
#' data(d_sim_binom_cov_hgly2)
#'
#' ermod_bin <- dev_ermod_bin(
#'   data = d_sim_binom_cov_hgly2,
#'   var_resp = "AEFLAG",
#'   var_exposure = "AUCss_1000",
#'   var_cov = "BHBA1C_5",
#' )
#'
#' ersim_new_exp_marg_med_qi <- sim_er_new_exp_marg(
#'   ermod_bin,
#'   exposure_to_sim_vec = seq(2, 6, by = 0.2),
#'   data_cov = dplyr::tibble(BHBA1C_5 = 4:10),
#'   n_subj_sim = NULL,
#'   n_draws_sim = 500, # This is set to make the example run faster
#'   output_type = "median_qi"
#' )
#'
#' ersim_new_exp_marg_med_qi
#' }
#'
sim_er_new_exp_marg <- function(
    ermod,
    exposure_to_sim_vec = NULL,
    data_cov = extract_data(ermod),
    n_subj_sim = 100,
    n_draws_sim = 500,
    seed_sample_draws = NULL,
    output_type = c("draws", "median_qi"),
    qi_width = 0.95) {
  stopifnot(inherits(ermod, "ermod"))
  stopifnot(!is.null(data_cov))
  output_type <- match.arg(output_type)

  var_exposure_sym <- rlang::sym(extract_var_exposure(ermod))

  # remove exposure column from data_cov, if it exists
  if (extract_var_exposure(ermod) %in% colnames(data_cov)) {
    data_cov <- data_cov |> dplyr::select(-!!var_exposure_sym)
  }

  if (is.null(n_subj_sim)) {
    data_to_expand <- data_cov
  } else {
    if (n_subj_sim > nrow(data_cov)) {
      warning(paste0(
        "n_subj_sim (", n_subj_sim, ") is greater than the number of ",
        "subjects in data_cov (", nrow(data_cov), "). ",
        "Consider setting `n_subj_sim = NULL` to use all subjects in data_cov."
      ))
    }
    data_to_expand <-
      data_cov |>
      dplyr::slice_sample(n = n_subj_sim, replace = TRUE)
  }

  newdata <-
    dplyr::tibble(!!var_exposure_sym := exposure_to_sim_vec) |>
    dplyr::mutate(.id_exposure = dplyr::row_number()) |>
    tidyr::expand_grid(data_to_expand)

  ersim_raw <-
    sim_er(
      ermod = ermod,
      newdata = newdata,
      n_draws_sim = n_draws_sim,
      seed_sample_draws = seed_sample_draws,
      # Need to marginalize first before taking median and qi
      output_type = "draws"
    )

  # Calculate marginal expected response for each exposure value and draw
  simdata <-
    ersim_raw |>
    dplyr::ungroup() |>
    dplyr::summarize(
      .epred = mean(.epred),
      .linpred = mean(.linpred),
      .prediction = mean(.prediction),
      .by = c(.id_exposure, !!var_exposure_sym, .draw)
    )

  # Marginalizing is done, so set .nrow_cov_data to 0
  .nrow_cov_data <- 0

  if (output_type == "draws") {
    return(new_ersim_marg(simdata, ermod,
      nrow_cov_data = .nrow_cov_data
    ))
  }

  # Group simdata with everything except .draw and response
  simdata_med_qi <-
    simdata |>
    dplyr::group_by(.id_exposure, !!var_exposure_sym) |>
    ggdist::median_qi(.width = qi_width) |>
    dplyr::as_tibble()

  return(new_ersim_marg_med_qi(simdata_med_qi, ermod,
    nrow_cov_data = .nrow_cov_data,
    qi_width = qi_width
  ))
}


#' @export
#' @rdname sim_er_new_exp_marg
#' @param exposure_range Range of exposure values to simulate. If NULL
#' (default), it is set to the range of the exposure variable in the original
#' data for model development.
#' @param num_exposures Number of exposure values to simulate.
#'
#' @details
#' [sim_er_curve_marg()] is a wrapper function for [sim_er_new_exp_marg()]
#' that use a range of exposure values to simulate the marginal expected
#' responses. Particularly useful for plotting the exposure-response curve.
#'
sim_er_curve_marg <- function(
    ermod,
    exposure_range = NULL,
    num_exposures = 51,
    data_cov = extract_data(ermod),
    n_subj_sim = 100,
    n_draws_sim = 500,
    seed_sample_draws = NULL,
    output_type = c("draws", "median_qi"),
    qi_width = 0.95) {
  if (is.null(exposure_range)) {
    exposure_range <-
      range(extract_data(ermod)[[extract_var_exposure(ermod)]])
  } else {
    stopifnot(length(exposure_range) == 2)
  }

  exposure_to_sim_vec <-
    seq(exposure_range[1], exposure_range[2], length.out = num_exposures)

  sim_er_new_exp_marg(
    ermod = ermod,
    exposure_to_sim_vec = exposure_to_sim_vec,
    data_cov = data_cov,
    n_subj_sim = n_subj_sim,
    n_draws_sim = n_draws_sim,
    seed_sample_draws = seed_sample_draws,
    output_type = output_type,
    qi_width = qi_width
  )
}


# Check and set the number of draws for simulation
chech_ndraws <- function(mod, n_draws_sim) {
  ndraws_mod <- nrow(as.matrix(mod$stanfit))
  if (is.null(n_draws_sim)) {
    n_draws_sim <- ndraws_mod
  } else {
    if (n_draws_sim > ndraws_mod) {
      warning(paste0(
        "n_draws_sim (", n_draws_sim, ") is greater than the number of ",
        "MCMC draws in the model (", ndraws_mod, "). ",
        "Setting n_draws_sim to ", ndraws_mod, "."
      ))
      n_draws_sim <- ndraws_mod
    }
  }
  return(n_draws_sim)
}

d_sim_binom_cov <- d_sim_binom_cov

id_to_sample <- seq(1, max(d_sim_binom_cov$ID), by = 5)
id_to_sample_2 <- seq(2, max(d_sim_binom_cov$ID), by = 10)

df_er_ae_covsel_test <-
  d_sim_binom_cov |>
  dplyr::filter(
    AETYPE == "ae_covsel_test",
    ID %in% id_to_sample
  ) |>
  dplyr::mutate(
    AUCss_1000 = AUCss / 1000, BAGE_10 = BAGE / 10,
    BWT_10 = BWT / 10, BHBA1C_5 = BHBA1C / 5
  )

df_er_ae_covsel_test_2 <-
  d_sim_binom_cov |>
  dplyr::filter(
    AETYPE == "ae_covsel_test",
    ID %in% id_to_sample_2
  ) |>
  dplyr::mutate(
    AUCss_1000 = AUCss / 1000, BAGE_10 = BAGE / 10,
    BWT_10 = BWT / 10, BHBA1C_5 = BHBA1C / 5
  )


# Create  ermod_bin objects for testing ---------------------------------------
set.seed(1234)
ermod_bin <- dev_ermod_bin(
  data = df_er_ae_covsel_test,
  var_resp = "AEFLAG",
  var_exposure = "AUCss_1000",
  var_cov = "BHBA1C_5",
  verbosity_level = 0,
  # Below option to make the test fast
  chains = 2, iter = 1000
)

ermod_bin_cov_sel <- new_ermod_bin_cov_sel(
  mod = extract_mod(ermod_bin),
  data = df_er_ae_covsel_test,
  var_resp = "AEFLAG",
  var_exposure = "AUCss_1000",
  var_cov_candidates = c("BAGE_10", "BWT_10", "BHBA1C_5", "RACE", "VISC"),
  var_cov = c("BHBA1C_5"),
  var_selected = c("AUCss_1000", "BHBA1C_5"),
  cv_method = c("LOO", "kfold"),
  cvvs = NULL,
  rk = NULL
)

# Simulate responses ----------------------------------------------------------
ersim <- sim_er(
  ermod_bin,
  newdata = NULL,
  n_draws_sim = 200,
  output_type = "draws"
)

ersim_med_qi <- sim_er(
  ermod_bin,
  newdata = df_er_ae_covsel_test_2,
  n_draws_sim = 200,
  output_type = c("median_qi"),
  qi_width = 0.95
)

## Specific exposures
ersim_new_exp <- sim_er_new_exp(
  ermod_bin,
  exposure_to_sim_vec = c(2, 2:6),
  data_cov = dplyr::tibble(BHBA1C_5 = 4:10, AUCss_1000 = 4:10),
  n_draws_sim = 200,
  output_type = "draws"
)

ersim_new_exp_med_qi <- sim_er_new_exp(
  ermod_bin,
  exposure_to_sim_vec = c(2, 2:6),
  data_cov = dplyr::tibble(BHBA1C_5 = 4:10, AUCss_1000 = 4:10),
  n_draws_sim = 200,
  output_type = "median_qi"
)

ersim_curve <- sim_er_curve(
  ermod_bin,
  data_cov = dplyr::tibble(BHBA1C_5 = c(4, 7)),
  n_draws_sim = 200,
  output_type = "draws"
)

## Marginal response
ersim_new_exp_marg <- sim_er_new_exp_marg(
  ermod_bin_cov_sel,
  exposure_to_sim_vec = c(2, 2:6),
  data_cov = dplyr::tibble(BHBA1C_5 = 4:10, AUCss_1000 = 4:10),
  n_subj_sim = NULL,
  n_draws_sim = 200
)

ersim_new_exp_marg_2 <- sim_er_new_exp_marg(
  ermod_bin_cov_sel,
  exposure_to_sim_vec = c(2, 2:6),
  n_subj_sim = 4,
  n_draws_sim = 200
)

ersim_new_exp_marg_med_qi <- sim_er_new_exp_marg(
  ermod_bin_cov_sel,
  exposure_to_sim_vec = c(2, 2:6),
  data_cov = dplyr::tibble(BHBA1C_5 = 4:10, AUCss_1000 = 4:10),
  n_subj_sim = NULL,
  n_draws_sim = 200,
  output_type = "median_qi"
)

ersim_curve_marg_med_qi <- sim_er_curve_marg(
  ermod_bin,
  exposure_range = c(2, 6),
  num_exposures = 11,
  data_cov = dplyr::tibble(BHBA1C_5 = seq(4, 10, by = 0.1)),
  n_subj_sim = NULL,
  n_draws_sim = 200,
  output_type = "median_qi"
)

ersim_curve_med_qi_2 <- calc_ersim_med_qi(ersim_curve)
ersim_new_exp_marg_med_qi_2 <- calc_ersim_med_qi(ersim_new_exp_marg)

# Test ------------------------------------------------------------------------

test_that("sim_ermod", {
  expect_equal(dim(ersim), c(20000, 24))
  expect_equal(nrow(ersim_med_qi), 50)
  expect_equal(min(ersim_med_qi$.linpred), -10.27, tolerance = 0.05)

  inv_logit <- \(x) exp(x) / (1 + exp(x))
  expect_equal(ersim$.epred, inv_logit(ersim$.linpred))
})

test_that("sim_er_new_exp", {
  expect_equal(nrow(ersim_new_exp), 8400)
  expect_equal(ncol(ersim_new_exp), 9)
  expect_equal(nrow(ersim_new_exp_med_qi), 42)
  expect_equal(max(ersim_new_exp_med_qi$.epred), 0.975, tolerance = 0.01)
  expect_equal(
    unique(ersim_curve$AUCss_1000),
    seq(0.398670752493232, 6.3639878,
      length.out = 51
    )
  )
  expect_equal(dim(ersim_curve_med_qi_2), c(102, 15))
})

test_that("sim_er_new_exp_marg", {
  expect_equal(nrow(ersim_new_exp_marg), 6 * 200)
  expect_equal(ncol(ersim_new_exp_marg), 6)
  expect_equal(nrow(ersim_new_exp_marg_2), 6 * 200)
  expect_equal(nrow(ersim_new_exp_marg_med_qi), 6)
  expect_equal(nrow(ersim_new_exp_marg_med_qi_2), 6)
  expect_equal(max(ersim_new_exp_marg_med_qi$.epred), 0.37, tolerance = 0.03)
  expect_equal(ersim_curve_marg_med_qi$AUCss_1000, seq(2, 6, by = 0.4))
})

test_that("test for errors and warnings", {
  sim_er_curve_marg(
    ermod_bin,
    exposure_range = c(2, 6),
    num_exposures = 11,
    data_cov = dplyr::tibble(BHBA1C_5 = seq(4, 10, by = 1)),
    n_subj_sim = 10,
    n_draws_sim = 200,
    output_type = "median_qi"
  ) |>
    expect_warning("n_subj_sim \\(10\\) is greater than")

  sim_er_new_exp(
    ermod_bin,
    exposure_to_sim_vec = c(2, 2:6),
    data_cov = dplyr::tibble(BHBA1C_5 = 4:10, AUCss_1000 = 4:10),
    n_draws_sim = 2000,
    output_type = "draws"
  ) |>
    expect_warning("n_draws_sim \\(2000\\) is greater than")
})

#' S3 methods for the classes `ersim_*` and `ersim_med_qi_*`
#' @name ersim_method
#' @param x An object of the classes `ersim_*` or `ersim_med_qi_*`
#' @param ... Additional arguments passed to functions
#'
#' @return No return value, called for print or plot side effects
NULL

#' @export
#' @rdname ersim_method
#' @inheritParams plot_er
plot.ersim <- function(x, show_orig_data = FALSE, ...) {
  plot_er(x, show_orig_data = show_orig_data)
}

#' @export
#' @rdname ersim_method
plot.ersim_med_qi <- function(x, show_orig_data = FALSE, ...) {
  plot_er(x, show_orig_data = show_orig_data)
}


# Extract elements ------------------------------------------------------------

#' Extract elements from objects of the classes `ersim_*` and `ersim_med_qi_*``
#'
#' @export
#' @name extract_ersim
#' @keywords internal
#' @inherit extract_method return
#' @param x An object of class \code{ersim_*}
extract_data.ersim <- function(x) attr(x, "origdata")

#' @export
#' @rdname extract_ersim
extract_data.ersim_med_qi <- function(x) attr(x, "origdata")

#' @export
#' @rdname extract_ersim
extract_var_resp.ersim <- function(x) attr(x, "var_resp")

#' @export
#' @rdname extract_ersim
extract_var_resp.ersim_med_qi <- function(x) attr(x, "var_resp")

#' @export
#' @rdname extract_ersim
extract_var_exposure.ersim <- function(x) attr(x, "var_exposure")

#' @export
#' @rdname extract_ersim
extract_var_exposure.ersim_med_qi <- function(x) attr(x, "var_exposure")

#' @export
#' @rdname extract_ersim
extract_var_cov.ersim <- function(x) attr(x, "var_cov")

#' @export
#' @rdname extract_ersim
extract_var_cov.ersim_med_qi <- function(x) attr(x, "var_cov")


#' Calculate median and quantile intervals from ersim object
#'
#' This is useful when you performed simulation with output_type = "draws" and
#' want to calculate median and quantile intervals without re-simulating.
#'
#' @export
#' @param x An object of class `ersim` or `ersim_marg`
#' @param qi_width Width of the quantile interval
#' @return An object of class `ersim_med_qi` or `ersim_marg_med_qi`
#'
calc_ersim_med_qi <- function(x, qi_width = 0.95) {
  var_exposure_sym <- rlang::sym(extract_var_exposure(x))
  if (inherits(x, "ersim_marg")) {
    x_grouped <- dplyr::group_by(x, .id_exposure, !!var_exposure_sym)
  } else {
    x_grouped <- dplyr::group_by(x, dplyr::across(
      !c(.chain, .iteration, .draw, .epred, .linpred, .prediction)
    ))
  }

  simdata_med_qi <-
    x_grouped |>
    ggdist::median_qi(.width = qi_width) |>
    dplyr::as_tibble()

  # Create dummy ermod to feed to ersim class constructor
  ermod <-
    structure(
      list(
        data = attr(x, "origdata"),
        var_resp = attr(x, "var_resp"),
        var_exposure = attr(x, "var_exposure"),
        var_cov = attr(x, "var_cov"),
        endpoint_type = attr(x, "endpoint_type")
      ),
      class = c("ermod")
    )
  if (inherits(x, "ersim_marg")) {
    return(new_ersim_marg_med_qi(simdata_med_qi, ermod,
      nrow_cov_data = attr(x, "nrow_cov_data"),
      qi_width = qi_width
    ))
  } else if (inherits(x, "ersim")) {
    return(new_ersim_med_qi(simdata_med_qi, ermod,
      nrow_cov_data = attr(x, "nrow_cov_data"),
      qi_width = qi_width
    ))
  }
}

#' Perform simulation of covariate effects for ER model
#'
#' @export
#' @param ermod an object of class `ermod` (supports `ermod_bin` and `ermod_lin`)
#' @param data an optional data frame to derive the covariate values for
#' forest plots. If NULL (default), the data used to fit the model is used.
#' @param spec_coveff you can supply spec_coveff to [sim_coveff()] or
#' [plot_coveff()], if you have already built it manually or with
#' [build_spec_coveff()]. See [build_spec_coveff()] for detail.
#' @param output_type Type of output. Currently only supports "median_qi"
#' which returns the median and quantile interval.
#' @param qi_width the width of the credible interval on the covariate effect.
#' This translate to the width of the error bars in the forest plot.
#' @param qi_width_cov the width of the quantile interval for continuous
#' covariates in the forest plot. Default is 0.9 (i.e. visualize effect of
#' covariate effect at their 5th and 95th percentile values).
#' @return A data frame with class `coveffsim` containing the median and
#' quantile interval of the covariate effects. For binary models (`ermod_bin`),
#' returns odds ratios. For linear models (`ermod_lin`), returns response differences.
#'
#' @examplesIf BayesERtools:::.if_run_ex_coveff()
#' \donttest{
#' data(d_sim_binom_cov_hgly2)
#'
#' ermod_bin <- dev_ermod_bin(
#'   data = d_sim_binom_cov_hgly2,
#'   var_resp = "AEFLAG",
#'   var_exposure = "AUCss_1000",
#'   var_cov = "BHBA1C_5",
#' )
#'
#' sim_coveff(ermod_bin)
#'
#' # Linear regression model example
#' data(d_sim_lin)
#'
#' ermod_lin <- dev_ermod_lin(
#'   data = d_sim_lin,
#'   var_resp = "response",
#'   var_exposure = "AUCss",
#'   var_cov = c("SEX", "BAGE"),
#' )
#'
#' sim_coveff(ermod_lin)
#' }
#'
sim_coveff <- function(
    ermod, data = NULL, spec_coveff = NULL,
    output_type = "median_qi",
    qi_width = 0.9,
    qi_width_cov = 0.9) {
  stopifnot(inherits(ermod, "ermod"))
  output_type <- match.arg(output_type)

  if (is.null(data)) {
    data <- ermod$data
  }
  if (is.null(spec_coveff)) {
    spec_coveff <- build_spec_coveff(ermod,
      data = data,
      qi_width_cov = qi_width_cov
    )
  }

  df_for_sim <- spec_coveff_to_df_sim(spec_coveff)

  linpred_draws <-
    .pp_matrix_to_draws_tbl(
      rstantools::posterior_linpred(extract_mod(ermod), newdata = df_for_sim),
      df_for_sim, ".linpred"
    )

  linpred_draws_ref <-
    linpred_draws |>
    dplyr::filter(is_ref) |>
    dplyr::ungroup() |>
    dplyr::select(.draw, .linpred_ref = .linpred)

  linpred_draws_2 <-
    linpred_draws |>
    dplyr::filter(!is_ref) |>
    dplyr::left_join(linpred_draws_ref, by = ".draw") |>
    dplyr::mutate(.delta_linpred = .linpred - .linpred_ref)

  if (inherits(ermod, "ermod_bin")) {
    linpred_draws_3 <-
      linpred_draws_2 |>
      dplyr::mutate(.odds_ratio = exp(.delta_linpred)) |>
      dplyr::select(-.linpred, -.linpred_ref, -.delta_linpred)
  } else if (inherits(ermod, "ermod_lin")) {
    linpred_draws_3 <-
      linpred_draws_2 |>
      dplyr::mutate(.response_diff = .delta_linpred) |>
      dplyr::select(-.linpred, -.linpred_ref, -.delta_linpred)
  } else {
    stop(
      "Only binary E-R model (`ermod_bin`) and linear E-R model ",
      "(`ermod_lin`) are supported for now"
    )
  }

  if (output_type == "draws") {
    return(coveffsim)
  }

  linpred_med_qi <-
    linpred_draws_3 |>
    dplyr::group_by(dplyr::across(
      !dplyr::any_of(c(".chain", ".iteration", ".draw",
                       ".odds_ratio", ".response_diff"))
    )) |>
    ggdist::median_qi(.width = qi_width) |>
    dplyr::arrange(var_order, value_order)

  # Determine which columns to select based on model type
  if (inherits(ermod, "ermod_bin")) {
    effect_col <- ".odds_ratio"
    ref_value <- 1
  } else {
    effect_col <- ".response_diff"
    ref_value <- 0
  }

  linpred_med_qi_to_join <-
    linpred_med_qi |>
    dplyr::select(var_order, value_order, !!effect_col, .lower, .upper)

  coveffsim <-
    spec_coveff |>
    dplyr::select(-value_cont, -value_cat) |>
    dplyr::left_join(linpred_med_qi_to_join,
      by = dplyr::join_by(var_order, value_order)
    ) |>
    dplyr::mutate(
      !!effect_col := ifelse(is_ref_value, ref_value, !!rlang::sym(effect_col))
    )

  # Add coveffsim class and model attribute
  class(coveffsim) <- c("coveffsim", class(coveffsim))
  attr(coveffsim, "model_type") <- if (inherits(ermod, "ermod_bin")) "binary" else "linear"

  return(coveffsim)
}

#' Visualize the covariate effects for ER model
#'
#' @export
#' @name plot_coveff
#' @inheritParams sim_coveff
#' @param x an object of class `ermod`, `coveffsim`,
#' or their subclasses
#' @param ... currently not used
#'
#' @return A ggplot object. For binary models (`ermod_bin`), plots odds ratios
#' on a log scale with reference line at 1. For linear models (`ermod_lin`),
#' plots response differences on a linear scale with reference line at 0.
#'
#' @examplesIf BayesERtools:::.if_run_ex_coveff()
#' \donttest{
#' data(d_sim_binom_cov_hgly2)
#'
#' ermod_bin <- dev_ermod_bin(
#'   data = d_sim_binom_cov_hgly2,
#'   var_resp = "AEFLAG",
#'   var_exposure = "AUCss_1000",
#'   var_cov = "BHBA1C_5",
#' )
#'
#' plot_coveff(ermod_bin)
#'
#' # Linear regression model example
#' data(d_sim_lin)
#'
#' ermod_lin <- dev_ermod_lin(
#'   data = d_sim_lin,
#'   var_resp = "response",
#'   var_exposure = "AUCss",
#'   var_cov = c("SEX", "BAGE"),
#' )
#'
#' plot_coveff(ermod_lin)
#' }
#'
plot_coveff <- function(x, ...) UseMethod("plot_coveff")


#' @export
#' @rdname plot_coveff
plot_coveff.ermod <- function(
    x, data = NULL, spec_coveff = NULL,
    qi_width = 0.9,
    qi_width_cov = 0.9, ...) {
  coveffsim <- sim_coveff(
    ermod = x, data = data, spec_coveff = spec_coveff,
    output_type = "median_qi", qi_width = qi_width,
    qi_width_cov = qi_width_cov
  )

  plot_coveff.coveffsim(coveffsim)
}


#' @export
#' @rdname plot_coveff
plot_coveff.coveffsim <- function(x, ...) {
  # Need the followings for plotting
  rlang::check_installed("ggforce")
  rlang::check_installed("xgxr")

  coveffsim <- x

  # Get model information from attributes
  is_binary_model <- attr(coveffsim, "model_type") == "binary"

  coveffsim_for_plot <-
    coveffsim |>
    # Only show ref value if show_ref_value is TRUE
    dplyr::filter(!is_ref_value | show_ref_value) |>
    dplyr::arrange(var_order, value_order) |>
    dplyr::mutate(
      var_value_index_num = paste0(var_order, "_", value_order),
      var_value_index_num = factor(var_value_index_num,
        levels = unique(var_value_index_num)
      ),
      var_value_index_num = as.numeric(var_value_index_num),
      value_label_plot = paste0(var_label, ": ", value_label)
    ) |>
    dplyr::mutate(
      .lower = ifelse(is_ref_value, ifelse(is_binary_model, 1, 0), .lower),
      .upper = ifelse(is_ref_value, ifelse(is_binary_model, 1, 0), .upper)
    )

  # Choose appropriate aesthetic and scale based on model type
  if (is_binary_model) {
    x_aes <- ggplot2::aes(x = .odds_ratio, y = var_value_index_num)
    x_scale <- xgxr::xgx_scale_x_log10(guide = ggplot2::guide_axis(minor.ticks = TRUE))
    x_lab <- "Odds ratio"
    x_intercept <- 1
  } else {
    x_aes <- ggplot2::aes(x = .response_diff, y = var_value_index_num)
    x_scale <- ggplot2::scale_x_continuous()
    x_lab <- "Response difference"
    x_intercept <- 0
  }

  gg <-
    coveffsim_for_plot |>
    ggplot2::ggplot(x_aes) +
    ggplot2::geom_point(size = 4) +
    ggplot2::geom_errorbar(
      ggplot2::aes(xmin = .lower, xmax = .upper),
      orientation = "y",
      width = 0.3
    ) +
    ggplot2::geom_vline(xintercept = x_intercept, linetype = "dashed") +
    x_scale +
    ggforce::facet_col(~var_label, scales = "free_y", space = "free") +
    ggplot2::labs(x = x_lab) +
    ggplot2::scale_y_continuous(
      breaks = coveffsim_for_plot$var_value_index_num,
      labels = coveffsim_for_plot$value_label_plot,
      transform = "reverse",
      sec.axis = ggplot2::sec_axis(
        transform = ~., name = "Second Axis",
        breaks = coveffsim_for_plot$var_value_index_num,
        labels = coveffsim_for_plot$value_annot
      )
    ) +
    ggplot2::theme(
      axis.title.y = ggplot2::element_blank(),
      strip.text.x = ggplot2::element_text(hjust = 0),
      panel.grid.minor.y = ggplot2::element_blank(),
      strip.background = ggplot2::element_blank()
    )

  return(gg)
}

#' Format the covariate effect simulation results for printing
#'
#' @export
#' @inheritParams build_spec_coveff
#' @param coveffsim an object of class `coveffsim`
#' @details
#' Note that `n_sigfig`, `use_seps`, and `drop_trailing_dec_mark` are only
#' applied to the response difference/odds ratio and 95% CI columns; value_label column was
#' already generated in an earlier step in [build_spec_coveff()] or
#' [sim_coveff()].
#' @return A data frame with the formatted covariate effect simulation results
#' with the following columns:
#' - `var_label`: the label of the covariate
#' - `value_label`: the label of the covariate value
#' - `value_annot`: the annotation of the covariate value
#' - `Odds ratio` or `Response difference`: the odds ratio (for binary models) or
#'   response difference (for linear models) of the covariate effect
#' - `95% CI`: the 95% credible interval of the covariate effect
#'
#' @examplesIf BayesERtools:::.if_run_ex_coveff()
#' \donttest{
#' data(d_sim_binom_cov_hgly2)
#'
#' ermod_bin <- dev_ermod_bin(
#'   data = d_sim_binom_cov_hgly2,
#'   var_resp = "AEFLAG",
#'   var_exposure = "AUCss_1000",
#'   var_cov = "BHBA1C_5",
#' )
#'
#' print_coveff(sim_coveff(ermod_bin))
#'
#' # Linear regression model example
#' data(d_sim_lin)
#'
#' ermod_lin <- dev_ermod_lin(
#'   data = d_sim_lin,
#'   var_resp = "response",
#'   var_exposure = "AUCss",
#'   var_cov = c("SEX", "BAGE"),
#' )
#'
#' print_coveff(sim_coveff(ermod_lin))
#' }
#'
print_coveff <- function(
    coveffsim, n_sigfig = 3, use_seps = TRUE, drop_trailing_dec_mark = TRUE) {
  rlang::check_installed("gt")

  # Get model information from attributes
  is_binary_model <- attr(coveffsim, "model_type") == "binary"

  coveffsim_non_ref <-
    coveffsim |>
    dplyr::filter(!is_ref_value)

  # Format values for printing in non-reference rows
  effect_col <- if (is_binary_model) ".odds_ratio" else ".response_diff"
  if (is_binary_model) {
    cols_to_format <- c(".odds_ratio", ".lower", ".upper")
    effect_col_name <- "Odds ratio"
  } else {
    cols_to_format <- c(".response_diff", ".lower", ".upper")
    effect_col_name <- "Response difference"
  }

  coveffsim_non_ref <-
    coveffsim_non_ref |>
    dplyr::mutate(dplyr::across(
      .cols = dplyr::all_of(cols_to_format),
      .fns = function(x) {
        gt::vec_fmt_number(x,
          n_sigfig = n_sigfig,
          use_seps = use_seps
        )
      }
    ))

  if (drop_trailing_dec_mark) {
    coveffsim_non_ref <-
      coveffsim_non_ref |>
      dplyr::mutate(dplyr::across(
        .cols = dplyr::all_of(cols_to_format),
        .fns = function(x) sub("\\.$", "", x)
      ))
  }

  # Format reference rows (set effect to reference value and blank for 95% CI)
  coveffsim_2 <-
    coveffsim |>
    # Only show ref value if show_ref_value is TRUE
    dplyr::filter(is_ref_value & show_ref_value) |>
    dplyr::mutate(
      dplyr::across(
        .cols = dplyr::all_of(cols_to_format),
        .fns = as.character
      )
    ) |>
    dplyr::bind_rows(coveffsim_non_ref) |>
    dplyr::arrange(var_order, value_order)

  coveffsim_2 |>
    dplyr::mutate(
      !!effect_col_name := !!rlang::sym(effect_col),
      `95% CI` =
        dplyr::if_else(is_ref_value, " ",
          paste0("[", .lower, ", ", .upper, "]")
        )
    ) |>
    dplyr::select(
      var_label, value_label, value_annot,
      !!effect_col_name, `95% CI`
    )
}


spec_coveff_to_df_sim <- function(spec_coveff) {
  # First build reference data frame
  df_ref_cont <- spec_coveff |>
    dplyr::filter(is_ref_value, !is.na(value_cont)) |>
    dplyr::select(var_name, value_cont) |>
    tidyr::pivot_wider(names_from = var_name, values_from = value_cont)
  df_ref_cat <- spec_coveff |>
    dplyr::filter(is_ref_value, !is.na(value_cat)) |>
    dplyr::select(var_name, value_cat) |>
    tidyr::pivot_wider(names_from = var_name, values_from = value_cat)

  df_ref <- dplyr::bind_cols(df_ref_cont, df_ref_cat)
  if (all(is.na(spec_coveff$value_cont))) df_ref <- df_ref_cat
  if (all(is.na(spec_coveff$value_cat))) df_ref <- df_ref_cont

  # Append the reference data frame to every row of the spec
  spec_coveff_2 <-
    spec_coveff |>
    dplyr::filter(!is_ref_value) |>
    dplyr::select(-is_ref_value) |>
    tidyr::expand_grid(df_ref)

  # Replace the corresponding values in the data columns with the
  # values in the spec
  df_for_sim <- spec_coveff_2 |>
    dplyr::mutate(.row_id = dplyr::row_number()) |>
    tidyr::nest(.by = .row_id, .key = "df_one_row") |>
    dplyr::mutate(
      df_one_row_new =
        purrr::map(df_one_row, replace_value_for_sim)
    ) |>
    dplyr::select(-.row_id, -df_one_row) |>
    tidyr::unnest(cols = df_one_row_new)

  df_for_sim_2 <-
    dplyr::bind_rows(
      dplyr::mutate(df_ref, is_ref = TRUE, var_order = 0),
      dplyr::mutate(df_for_sim, is_ref = FALSE)
    )

  return(df_for_sim_2)
}

replace_value_for_sim <- function(df_one_row) {
  var_name <- df_one_row$var_name
  value <- ifelse(
    is.na(df_one_row$value_cont),
    df_one_row$value_cat,
    df_one_row$value_cont
  )

  df_one_row[[var_name]] <- value

  return(df_one_row)
}

.if_run_ex_coveff <- function() {
  requireNamespace("ggforce", quietly = TRUE) &&
    requireNamespace("xgxr", quietly = TRUE) &&
    requireNamespace("gt", quietly = TRUE)
}

if (.if_run_ex_coveff()) {
  data(d_sim_binom_cov_hgly2)

  id_to_sample <- seq(1, max(d_sim_binom_cov_hgly2$ID), by = 5)

  df_er_ae_hgly2 <-
    d_sim_binom_cov_hgly2 |>
    dplyr::filter(ID %in% id_to_sample)

  var_resp <- "AEFLAG"

  # develop models ------------------------------------------------------------
  set.seed(1234)
  ermod_bin <- dev_ermod_bin(
    data = df_er_ae_hgly2,
    var_resp = var_resp,
    var_exposure = "AUCss_1000",
    var_cov = c("BHBA1C_5", "BGLUC", "RACE", "VISC"),
    verbosity_level = 0,
    # Below option to make the test fast
    chains = 2, iter = 1000
  )

  spec_coveff <- build_spec_coveff(ermod_bin)
  coveffsim <- sim_coveff(ermod_bin)
  coveffsim_2 <- sim_coveff(ermod_bin, spec_coveff = spec_coveff)

  # Linear regression model tests ---------------------------------------------
  data(d_sim_lin)

  set.seed(1234)
  ermod_lin <- dev_ermod_lin(
    data = d_sim_lin,
    var_resp = "response",
    var_exposure = "AUCss",
    var_cov = c("SEX", "BAGE"),
    verbosity_level = 0,
    # Below option to make the test fast
    chains = 2, iter = 1000
  )

  spec_coveff_lin <- build_spec_coveff(ermod_lin)
  coveffsim_lin <- sim_coveff(ermod_lin)
  coveffsim_lin_2 <- sim_coveff(ermod_lin, spec_coveff = spec_coveff_lin)

  # Tests ---------------------------------------------------------------------
  test_that("sim_coveff binary model", {
    expect_equal(coveffsim, coveffsim_2)
    expect_equal(dim(coveffsim), c(14, 12))
    expect_equal(
      coveffsim$.odds_ratio,
      c(
        0.394045222892321, 1, 4.80902749409125, 0.0814990385480039, 1,
        11.0151856281521, 0.148013228492832, 1,
        3.93067614853305, 1, 2.77732164370792, 2.05372376449102, 1, 0.46514853
      )
    )

    # Test model type attribute
    expect_equal(attr(coveffsim, "model_type"), "binary")
  })

  test_that("sim_coveff linear model", {
    expect_equal(coveffsim_lin, coveffsim_lin_2)
    expect_equal(dim(coveffsim_lin), c(8, 12))
    expect_true(".response_diff" %in% colnames(coveffsim_lin))
    expect_true(all(!is.na(coveffsim_lin$.response_diff)))
    expect_true(all(coveffsim_lin$.response_diff[coveffsim_lin$is_ref_value] == 0))

    # Test model type attribute
    expect_equal(attr(coveffsim_lin, "model_type"), "linear")
  })

  test_that("plot_coveff binary model", {
    g1 <- plot_coveff(coveffsim) |>
      expect_silent()
    g2 <- plot_coveff(ermod_bin) |>
      expect_silent()
    g3 <-
      coveffsim |>
      dplyr::mutate(show_ref_value = FALSE) |>
      plot_coveff() |>
      expect_silent()

    expect_equal(dim(g1$data), c(14, 14))
    expect_equal(dim(g2$data), c(14, 14))
    expect_equal(dim(g3$data), c(9, 14))
  })

  test_that("plot_coveff linear model", {
    g1 <- plot_coveff(coveffsim_lin) |>
      expect_silent()
    g2 <- plot_coveff(ermod_lin) |>
      expect_silent()
    g3 <-
      coveffsim_lin |>
      dplyr::mutate(show_ref_value = FALSE) |>
      plot_coveff() |>
      expect_silent()

    expect_equal(dim(g1$data), c(8, 14))
    expect_equal(dim(g2$data), c(8, 14))
    expect_equal(dim(g3$data), c(5, 14))
  })

  test_that("print_coveff binary model", {
    table_coveff <- print_coveff(coveffsim)

    expect_equal(
      table_coveff$`Odds ratio`,
      c(
        "0.394", "1", "4.81", "0.0815", "1", "11.0", "0.148", "1", "3.93",
        "1", "2.78", "2.05", "1", "0.465"
      )
    )
  })

  test_that("print_coveff linear model", {
    table_coveff_lin <- print_coveff(coveffsim_lin)

    expect_equal(
      table_coveff_lin$`Response difference`,
      c(
        "−20.5", "0", "20.5", "0", "−4.04", "−8.50", "0", "8.34"
      )
    )
  })

  test_that("build_spec_coveff binary model", {
    expect_equal(dim(spec_coveff), c(14, 11))
    expect_equal(
      spec_coveff$value_label,
      c(
        "0.824", "2.31", "4.80", "6.01", "8.19", "10.3", "4.42", "6.09",
        "7.28", "White", "Asian", "Black", "No", "Yes"
      )
    )
  })

  test_that("build_spec_coveff linear model", {
    expect_equal(dim(spec_coveff_lin), c(8, 11))
    expect_equal(
      spec_coveff_lin$value_label,
      c(
        "5.00", "50.0", "95.0", "F", "M", "33.5", "50.4", "67.0"
      )
    )
  })

  test_that("calc_summary_col", {
    spec_coveff <- build_spec_coveff(ermod_bin)

    calc_summary_col_cont(0:100) |> expect_equal(
      dplyr::tibble(
        value_cont = c(5, 50, 95),
        value_order = c(1, 2, 3),
        value_annot = c("5th", "median", "95th"),
        is_ref_value = c(FALSE, TRUE, FALSE),
        value_label = c("5.00", "50.0", "95.0")
      )
    )

    calc_summary_col_categ(c("A", "A", "B", "C", "C", "D")) |> expect_equal(
      dplyr::tibble(
        value_cat = c("A", "C", "B", "D"),
        value_order = c(1, 2, 3, 4),
        value_annot = c("1st freq", "2nd freq", "3rd freq", "4th freq"),
        is_ref_value = c(TRUE, FALSE, FALSE, FALSE),
        value_label = c("A", "C", "B", "D")
      )
    )
  })
}

# Logistic --------------------------------------------------------------------

#' Develop linear ER model for binary or continuous endpoint
#'
#' These functions are used to develop an linear ER model with binary
#' ([dev_ermod_bin()]) or continuous ([dev_ermod_lin()]) endpoint.
#' You can also specify covariates to be included in the model.
#'
#' @export
#' @param data Input data for E-R analysis
#' @param var_resp Response variable name in character
#' @param var_exposure Exposure variable names in character
#' @param var_cov Covariate variable names in character vector
#' @param prior,prior_intercept,prior_aux See [rstanarm::stan_glm()]
#' @param verbosity_level Verbosity level. 0: No output, 1: Display steps,
#' 2: Display progress in each step, 3: Display MCMC sampling.
#' @param chains Number of chains for Stan.
#' @param iter Number of iterations for Stan.
#'
#' @return An object of class `ermod_bin` or `ermod_lin`.
#'
#' @examples
#' \donttest{
#' data(d_sim_binom_cov_hgly2)
#'
#' ermod_bin <- dev_ermod_bin(
#'   data = d_sim_binom_cov_hgly2,
#'   var_resp = "AEFLAG",
#'   var_exposure = "AUCss_1000",
#'   var_cov = "BHBA1C_5",
#' )
#'
#' ermod_bin
#' }
#'
dev_ermod_bin <- function(
    data,
    var_resp,
    var_exposure,
    var_cov = NULL,
    prior = rstanarm::default_prior_coef(stats::binomial()),
    prior_intercept = rstanarm::default_prior_intercept(stats::binomial()),
    verbosity_level = 1,
    chains = 4,
    iter = 2000) {
  stopifnot(verbosity_level %in% c(0, 1, 2, 3))
  refresh <- dplyr::if_else(verbosity_level >= 3, iter %/% 4, 0)

  input_args <- capture_selected_args(
    c("chains", "iter"),
    environment()
  )

  check_data_columns(
    data = data,
    var_exposure = var_exposure,
    var_resp = var_resp,
    var_cov = var_cov,
    is_binary = TRUE
  )

  var_full <- c(var_exposure, var_cov)

  formula_final <-
    stats::formula(
      paste(var_resp, "~", paste(var_full, collapse = " + "))
    )

  # Need to construct call and then evaluate. Directly calling
  # rstanarm::stan_glm with formula_final does not work for the cross-validation
  call_stan_glm <- rlang::call2(
    rstanarm::stan_glm,
    formula = formula_final,
    family = stats::binomial(),
    data = quote(data),
    prior = prior,
    prior_intercept = prior_intercept,
    QR = dplyr::if_else(length(var_full) > 1, TRUE, FALSE),
    refresh = refresh, chains = chains, iter = iter
  )
  mod <- eval(call_stan_glm)

  new_ermod_bin(
    mod = mod,
    data = data,
    var_resp = var_resp,
    var_exposure = var_exposure,
    var_cov = var_cov,
    input_args = input_args
  )
}


#' Exposure metrics selection for linear ER models
#'
#' This functions is used to develop an linear ER model with binary
#' and continuous endpoint, using various exposure metrics and
#' selecting the best one.
#'
#' @export
#' @inheritParams dev_ermod_bin
#' @param var_exp_candidates Candidate exposure variable names
#' in character vector
#'
#' @return An object of class `ermod_bin_exp_sel`.or `ermod_lin_exp_sel`
#'
#' @examples
#' \donttest{
#' data(d_sim_binom_cov_hgly2)
#'
#' ermod_bin_exp_sel <-
#'   dev_ermod_bin_exp_sel(
#'     data = d_sim_binom_cov_hgly2,
#'     var_resp = "AEFLAG",
#'     var_exp_candidates = c("AUCss_1000", "Cmaxss", "Cminss")
#'   )
#'
#' ermod_bin_exp_sel
#' }
#'
dev_ermod_bin_exp_sel <- function(
    data,
    var_resp,
    var_exp_candidates,
    prior = rstanarm::default_prior_coef(stats::binomial()),
    prior_intercept = rstanarm::default_prior_intercept(stats::binomial()),
    verbosity_level = 1,
    chains = 4,
    iter = 2000) {
  fun_dev_ermod <-
    purrr::partial(
      dev_ermod_bin,
      prior = prior,
      prior_intercept = prior_intercept
    )

  l_out <-
    .dev_ermod_exp_sel(
      data = data,
      var_resp = var_resp,
      var_exp_candidates = var_exp_candidates,
      verbosity_level = verbosity_level,
      chains = chains,
      iter = iter,
      fun_dev_ermod = fun_dev_ermod
    )

  new_ermod_bin_exp_sel(l_out)
}


#' Perform covariate selection for linear ER model
#'
#' This functions is used to develop an ER model with covariates for binary
#' and continuous endpoints.
#' `projpred` package is used for variable selection.
#'
#' @export
#' @inheritParams dev_ermod_bin
#' @param var_cov_candidates Candidate covariate names in character vector
#' @param cv_method Cross-validation method. Default is "LOO" (recommended).
#' Use "kfold" if you see warnings on Pareto k estimates.
#' @param k Number of folds for kfold CV.
#' Only used if cv_method is "kfold".
#' @param validate_search Whether to validate the search. Default is FALSE.
#' Recommend to set to TRUE for kfold CV. Do not use for LOO (run time would
#' become too long).
#' @param nterms_max Maximum number of terms to consider in the model.
#'  Default is NULL (all terms are considered).
#' @param .reduce_obj_size Whether to reduce object size by removing some
#' elements from projpred outputs that are not necessary for the functionality
#' of this package.
#'
#' @return An object of class `ermod_bin_cov_sel` or `ermod_lin_cov_sel`.
#'
#' @examplesIf BayesERtools:::.if_run_ex_covsel()
#' \donttest{
#' data(d_sim_binom_cov_hgly2)
#'
#' er_binary_cov_model <- dev_ermod_bin_cov_sel(
#'   data = d_sim_binom_cov_hgly2,
#'   var_resp = "AEFLAG",
#'   var_exposure = "AUCss_1000",
#'   var_cov_candidates = c(
#'     "BAGE_10", "BWT_10", "BGLUC",
#'     "BHBA1C_5", "RACE", "VISC"
#'   )
#' )
#'
#' er_binary_cov_model
#' }
#'
dev_ermod_bin_cov_sel <- function(
    data,
    var_resp,
    var_exposure,
    var_cov_candidates,
    cv_method = c("LOO", "kfold"),
    k = 5,
    validate_search = FALSE,
    nterms_max = NULL,
    .reduce_obj_size = TRUE,
    prior = rstanarm::default_prior_coef(stats::binomial()),
    prior_intercept = rstanarm::default_prior_intercept(stats::binomial()),
    verbosity_level = 1,
    chains = 4,
    iter = 2000) {
  fun_dev_ermod <-
    purrr::partial(
      dev_ermod_bin,
      prior = prior,
      prior_intercept = prior_intercept
    )

  ll <- .dev_ermod_cov_sel(
    data = data,
    var_resp = var_resp,
    var_exposure = var_exposure,
    var_cov_candidates = var_cov_candidates,
    cv_method = cv_method,
    k = k,
    validate_search = validate_search,
    nterms_max = nterms_max,
    .reduce_obj_size = .reduce_obj_size,
    verbosity_level = verbosity_level,
    chains = chains,
    iter = iter,
    fun_dev_ermod = fun_dev_ermod,
    fun_family = quote(stats::binomial()),
    prior = prior,
    prior_intercept = prior_intercept
  )

  with(ll, new_ermod_bin_cov_sel(
    mod = extract_mod(mod),
    data = data,
    var_resp = var_resp,
    var_exposure = var_exposure,
    var_cov_candidates = var_cov_candidates,
    var_cov = var_cov,
    var_selected = var_selected,
    cv_method = cv_method,
    cvvs = cvvs,
    rk = rk,
    input_args = mod$input_args
  ))
}

# Linear ----------------------------------------------------------------------

#' @export
#' @rdname dev_ermod_bin
#' @examples
#' \donttest{
#' data(d_sim_lin)
#'
#' ermod_lin <- dev_ermod_lin(
#'   data = d_sim_lin,
#'   var_resp = "response",
#'   var_exposure = "AUCss",
#'   var_cov = c("SEX", "BAGE")
#' )
#'
#' ermod_lin
#' }
#'
dev_ermod_lin <- function(
    data,
    var_resp,
    var_exposure,
    var_cov = NULL,
    prior = rstanarm::default_prior_coef(stats::binomial()),
    prior_intercept = rstanarm::default_prior_intercept(stats::binomial()),
    prior_aux = rstanarm::exponential(autoscale = TRUE),
    verbosity_level = 1,
    chains = 4,
    iter = 2000) {
  stopifnot(verbosity_level %in% c(0, 1, 2, 3))
  refresh <- dplyr::if_else(verbosity_level >= 3, iter %/% 4, 0)

  input_args <- capture_selected_args(
    c("chains", "iter"),
    environment()
  )

  check_data_columns(
    data = data,
    var_exposure = var_exposure,
    var_resp = var_resp,
    var_cov = var_cov
  )

  var_full <- c(var_exposure, var_cov)

  formula_final <-
    stats::formula(
      paste(var_resp, "~", paste(var_full, collapse = " + "))
    )

  mod <- rstanarm::stan_glm(
    formula_final,
    family = stats::gaussian(),
    data = data,
    prior = prior,
    prior_intercept = prior_intercept,
    prior_aux = prior_aux,
    QR = dplyr::if_else(length(var_full) > 1, TRUE, FALSE),
    refresh = refresh,
    chains = chains,
    iter = iter
  )

  new_ermod_lin(
    mod = mod,
    data = data,
    var_resp = var_resp,
    var_exposure = var_exposure,
    var_cov = var_cov,
    input_args = input_args
  )
}


#' @export
#' @rdname dev_ermod_bin_exp_sel
#' @examples
#' \donttest{
#' data(d_sim_lin)
#'
#' ermod_lin_exp_sel <- dev_ermod_lin_exp_sel(
#'   data = d_sim_lin,
#'   var_resp = "response",
#'   var_exp_candidates = c("AUCss", "Cmaxss")
#' )
#'
#' ermod_lin_exp_sel
#' }
#'
dev_ermod_lin_exp_sel <- function(
    data,
    var_resp,
    var_exp_candidates,
    prior = rstanarm::default_prior_coef(stats::binomial()),
    prior_intercept = rstanarm::default_prior_intercept(stats::binomial()),
    prior_aux = rstanarm::exponential(autoscale = TRUE),
    verbosity_level = 1,
    chains = 4,
    iter = 2000) {
  fun_dev_ermod <-
    purrr::partial(
      dev_ermod_lin,
      prior = prior,
      prior_intercept = prior_intercept,
      prior_aux = prior_aux
    )

  l_out <-
    .dev_ermod_exp_sel(
      data = data,
      var_resp = var_resp,
      var_exp_candidates = var_exp_candidates,
      verbosity_level = verbosity_level,
      chains = chains,
      iter = iter,
      fun_dev_ermod = fun_dev_ermod
    )

  new_ermod_lin_exp_sel(l_out)
}


#' @export
#' @rdname dev_ermod_bin_cov_sel
#' @examplesIf BayesERtools:::.if_run_ex_covsel()
#' \donttest{
#' data(d_sim_lin)
#'
#' ermod_lin_cov_sel <- dev_ermod_lin_cov_sel(
#'   data = d_sim_lin,
#'   var_resp = "response",
#'   var_exposure = "AUCss",
#'   var_cov_candidates = c("BAGE", "SEX")
#' )
#'
#' ermod_lin_cov_sel
#' }
#'
dev_ermod_lin_cov_sel <- function(
    data,
    var_resp,
    var_exposure,
    var_cov_candidates,
    cv_method = c("LOO", "kfold"),
    k = 5,
    validate_search = FALSE,
    nterms_max = NULL,
    .reduce_obj_size = TRUE,
    prior = rstanarm::default_prior_coef(stats::binomial()),
    prior_intercept = rstanarm::default_prior_intercept(stats::binomial()),
    prior_aux = rstanarm::exponential(autoscale = TRUE),
    verbosity_level = 1,
    chains = 4,
    iter = 2000) {
  fun_dev_ermod <-
    purrr::partial(
      dev_ermod_lin,
      prior = prior,
      prior_intercept = prior_intercept,
      prior_aux = prior_aux
    )

  ll <- .dev_ermod_cov_sel(
    data = data,
    var_resp = var_resp,
    var_exposure = var_exposure,
    var_cov_candidates = var_cov_candidates,
    cv_method = cv_method,
    k = k,
    validate_search = validate_search,
    nterms_max = nterms_max,
    .reduce_obj_size = .reduce_obj_size,
    verbosity_level = verbosity_level,
    chains = chains,
    iter = iter,
    fun_dev_ermod = fun_dev_ermod,
    fun_family = quote(stats::gaussian()),
    prior = prior,
    prior_intercept = prior_intercept,
    prior_aux = prior_aux
  )

  with(ll, new_ermod_lin_cov_sel(
    mod = extract_mod(mod),
    data = data,
    var_resp = var_resp,
    var_exposure = var_exposure,
    var_cov_candidates = var_cov_candidates,
    var_cov = var_cov,
    var_selected = var_selected,
    cv_method = cv_method,
    cvvs = cvvs,
    rk = rk,
    input_args = mod$input_args
  ))
}

# Internal functions ----------------------------------------------------------

.dev_ermod_exp_sel <- function(
    data, var_resp, var_exp_candidates,
    verbosity_level = 1, chains = 4, iter = 2000,
    fun_dev_ermod) {
  stopifnot(verbosity_level %in% c(0, 1, 2, 3))

  verbose <- dplyr::if_else(verbosity_level == 2, TRUE, FALSE)

  check_data_columns(
    data = data,
    var_resp = var_resp,
    var_exp_candidates = var_exp_candidates
  )

  l_mod_exposures <-
    var_exp_candidates |>
    purrr::set_names() |>
    purrr::map(
      function(.x) {
        fun_dev_ermod(data, var_resp, .x,
          chains = chains, iter = iter,
          verbosity_level = verbosity_level
        )
      },
      .progress = verbose
    )

  if (length(var_exp_candidates) == 1) {
    if (verbosity_level >= 1) {
      cli::cli_alert_info(paste(
        "Only one exposure metric",
        "{var_exp_candidates[[1]]} was provided."
      ))
    }

    var_exposure <- var_exp_candidates[[1]]
    loo_comp_exposures <- NULL
  } else {
    l_loo_exposures <- purrr::map(
      l_mod_exposures,
      function(.x) loo(.x)
    )
    comp_exposures <- loo::loo_compare(l_loo_exposures)
    if ("model" %in% colnames(comp_exposures)) {
      var_exposure <- comp_exposures$model[[1]]
    } else {
      var_exposure <- rownames(comp_exposures)[[1]]
    }

    if (verbosity_level >= 1) {
      cli::cli_alert_info("The exposure metric selected was: {var_exposure}")
    }

    loo_comp_exposures <- comp_exposures
  }

  list(
    mod = extract_mod(l_mod_exposures[[var_exposure]]),
    data = data,
    var_resp = var_resp,
    var_exp_candidates = var_exp_candidates,
    var_exposure = var_exposure,
    l_mod_exposures = l_mod_exposures,
    loo_comp_exposures = loo_comp_exposures,
    input_args = l_mod_exposures[[var_exposure]]$input_args
  )
}


.dev_ermod_cov_sel <- function(
    data,
    var_resp,
    var_exposure,
    var_cov_candidates,
    cv_method = c("LOO", "kfold"),
    k = 5,
    validate_search = FALSE,
    nterms_max = NULL,
    .reduce_obj_size = TRUE,
    verbosity_level = 1,
    chains = 4,
    iter = 2000,
    fun_dev_ermod,
    fun_family,
    prior = rstanarm::default_prior_coef(stats::binomial()),
    prior_intercept = rstanarm::default_prior_intercept(stats::binomial()),
    prior_aux = rstanarm::exponential(autoscale = TRUE)) {
  stopifnot(verbosity_level %in% c(0, 1, 2, 3))

  rlang::check_installed("projpred")

  if (length(var_cov_candidates) == 0) {
    stop("At least one covariate candidate should be provided")
  }

  check_data_columns(
    data = data,
    var_resp = var_resp,
    var_exposure = var_exposure,
    var_cov_candidates = var_cov_candidates
  )

  if (verbosity_level >= 1) cli::cli_h2("Step 1: Full reference model fit")
  refm_obj <- .dev_ermod_refmodel(
    data = data,
    var_resp = var_resp,
    var_exposure = var_exposure,
    var_cov_candidates = var_cov_candidates,
    verbosity_level = verbosity_level,
    chains = chains, iter = iter,
    fun_family = fun_family,
    prior = prior,
    prior_intercept = prior_intercept,
    prior_aux = prior_aux
  )

  if (verbosity_level >= 1) cli::cli_h2("Step 2: Variable selection")
  var_selected <- .select_cov_projpred(
    refm_obj = refm_obj,
    var_exposure = var_exposure,
    var_cov_candidates = var_cov_candidates,
    nterms_max = nterms_max,
    cv_method = cv_method,
    k = k,
    .reduce_obj_size = .reduce_obj_size,
    validate_search = validate_search,
    verbosity_level = verbosity_level
  )

  if (verbosity_level >= 1) cli::cli_h2("Step 3: Final model fit")

  if (length(var_selected) == 1) {
    var_cov <- NULL
  } else {
    var_cov <- setdiff(var_selected, var_exposure)
  }

  mod_final <- fun_dev_ermod(
    data = data,
    var_resp = var_resp,
    var_exposure = var_exposure,
    var_cov = var_cov,
    verbosity_level = verbosity_level,
    chains = chains, iter = iter
  )

  if (verbosity_level >= 1) cli::cli_h2("Cov mod dev complete")

  list(
    mod = mod_final,
    data = data,
    var_resp = var_resp,
    var_exposure = var_exposure,
    var_cov_candidates = var_cov_candidates,
    var_cov = var_cov,
    var_selected = as.character(var_selected),
    cv_method = cv_method,
    cvvs = attr(var_selected, "cvvs"),
    rk = attr(var_selected, "rk")
  )
}


#' Internal functions for developing an ER model with covariates
#' for binary endpoint
#'
#' These functions are not intended to be used directly by users.
#'
#' @name dev_ermod_bin_cov_functions
#' @keywords internal
#' @inheritParams dev_ermod_bin
#' @inheritParams dev_ermod_bin_cov_sel
NULL


#' @export
#' @rdname dev_ermod_bin_cov_functions
#' @param fun_family Family function for the model. Default is binomial.
#' @keywords internal
#' @details
#' [.dev_ermod_refmodel()] is used to fit the refmodel (full reference
#' model) necessary for projpred
#' @return
#' [.dev_ermod_refmodel()]: The reference model object that can be used
#' for variable selection.
.dev_ermod_refmodel <- function(
    data, var_resp, var_exposure, var_cov_candidates,
    verbosity_level = 1, chains = 4, iter = 2000,
    fun_family = quote(stats::binomial()),
    prior = rstanarm::default_prior_coef(stats::binomial()),
    prior_intercept = rstanarm::default_prior_intercept(stats::binomial()),
    prior_aux = rstanarm::exponential(autoscale = TRUE)) {
  stopifnot(verbosity_level %in% c(0, 1, 2, 3))
  refresh <- dplyr::if_else(verbosity_level >= 3, iter %/% 4, 0)

  if (length(var_exposure) != 1) {
    stop("Only one exposure metric should be provided")
  }

  check_data_columns(
    data = data,
    var_resp = var_resp,
    var_exposure = var_exposure,
    var_cov_candidates = var_cov_candidates
  )

  varnames <- paste(c(var_exposure, var_cov_candidates), collapse = " + ")
  formula_full <-
    stats::formula(paste(var_resp, "~", varnames))

  # Need to construct call and then evaluate. Directly calling
  # rstanarm::stan_glm with formula_full does not work for the cross-validation
  call_fit_ref <-
    rlang::call2(rstanarm::stan_glm,
      formula = formula_full,
      family = fun_family, data = quote(data), QR = TRUE,
      refresh = refresh, chains = chains, iter = iter,
      prior = prior, prior_intercept = prior_intercept,
      prior_aux = prior_aux
    )
  fit_ref <- eval(call_fit_ref)

  refm_obj <- projpred::get_refmodel(fit_ref)

  return(refm_obj)
}


#' @export
#' @rdname dev_ermod_bin_cov_functions
#' @keywords internal
#' @param refm_obj Reference model object used for variable selection
#' @details
#' [.select_cov_projpred()] is used to select variables with `projpred` package
#' @return
#' [.select_cov_projpred()]: The selected variables
.select_cov_projpred <- function(
    refm_obj, var_exposure, var_cov_candidates,
    nterms_max = NULL,
    cv_method = c("LOO", "kfold"),
    k = 5,
    .reduce_obj_size = TRUE,
    validate_search = FALSE,
    verbosity_level = 1) {
  cv_method <- match.arg(cv_method)
  stopifnot(verbosity_level %in% c(0, 1, 2, 3))

  if (length(var_exposure) != 1) {
    stop("Only one exposure metric should be provided")
  }

  search_terms <- projpred::force_search_terms(
    forced_terms = var_exposure,
    optional_terms = var_cov_candidates
  )

  verbose <- dplyr::if_else(verbosity_level == 2, TRUE, FALSE)

  if (cv_method == "LOO" && validate_search) {
    stop(paste(
      "validate_search should be set to FALSE for LOO,",
      "otherwise run time would be too long"
    ))
  }

  cvvs <- projpred::cv_varsel(
    refm_obj,
    search_terms = search_terms,
    nterms_max = nterms_max,
    cv_method = cv_method,
    K = k,
    method = "forward",
    validate_search = validate_search,
    refit_prj = TRUE,
    verbose = verbose
  )

  rk <- projpred::ranking(cvvs)

  n_var_select <- projpred::suggest_size(cvvs)
  if (n_var_select == 0 && verbosity_level >= 1) {
    cli::cli_alert_info(paste(
      "No variables selected, exposure variable was",
      "forced into the final model."
    ))
  }
  n_var_select <- max(1, n_var_select)

  var_selected <- utils::head(rk[["fulldata"]], n_var_select)

  if (verbosity_level >= 1) {
    cli::cli_alert_info(
      paste(
        "The variables selected were:",
        paste(var_selected, collapse = ", ")
      )
    )
  }

  if (.reduce_obj_size) {
    cvvs <- .reduce_cvvs_size(cvvs)
  }

  attr(var_selected, "cvvs") <- cvvs
  attr(var_selected, "rk") <- rk
  attr(var_selected, "var_exposure") <- var_exposure

  return(var_selected)
}


.reduce_cvvs_size <- function(cvvs) {
  family <- cvvs$refmodel$family
  cvvs$refmodel <- NULL
  cvvs$refmodel$family <- family
  cvvs$search_path <- NULL

  return(cvvs)
}

check_data_columns <- function(
    data, var_resp = NULL, var_exp_candidates = NULL,
    var_exposure = NULL, var_cov_candidates = NULL, var_cov = NULL,
    is_binary = FALSE) {
  if (!is.data.frame(data)) {
    stop("data should be a data frame")
  }

  if (!is.null(var_resp) && length(var_resp) != 1) {
    stop("Only one response variable should be provided")
  }

  if (is_binary && !all(data[[var_resp]] %in% c(0, 1))) {
    stop("Response variable must be a numeric of 0 and 1 for binary models")
  }

  if (!is.null(var_exposure) && length(var_exposure) != 1) {
    stop("Only one exposure metric should be provided")
  }

  check_columns_exist(data, var_resp, "response")
  check_columns_exist(data, var_exp_candidates, "exposure metric")
  check_columns_exist(data, var_exposure, "exposure metric")
  check_columns_exist(data, var_cov_candidates, "covariate")
  check_columns_exist(data, var_cov, "covariate")

  check_na_values(data, c(var_exp_candidates, var_exposure), "exposure metric")
  check_na_values(data, var_cov_candidates, "covariate")
  check_na_values(data, var_cov, "covariate")
  check_na_values(data, var_resp, "response")
}

check_columns_exist <- function(data, cols, column_type) {
  if (!is.null(cols) && any(!(cols %in% names(data)))) {
    cols_str <- paste(cols, collapse = ", ")
    stop(sprintf('"%s" columns should be in data', cols_str))
  }
}

check_na_values <- function(data, cols, column_type) {
  if (!is.null(cols)) {
    # Need to separate, otherwise styler & lintr complains
    if (any(sapply(cols, function(col) any(is.na(data[[col]]))))) {
      stop(paste("NA values not allowed in the", column_type, "column(s)"))
    }
  }
}

# Capture the specified arguments and their values from the given environment
capture_selected_args <- function(arg_names, env) {
  input_args <- lapply(arg_names, function(arg) get(arg, envir = env))

  # Name the list with the argument names
  names(input_args) <- arg_names

  return(input_args)
}

.if_run_ex_covsel <- function() {
  requireNamespace("projpred", quietly = TRUE)
}

# Read d_sim_binom_cov data, which is exported to the package
d_sim_binom_cov <- d_sim_binom_cov

id_to_sample <- seq(1, max(d_sim_binom_cov$ID), by = 1)
id_to_sample2 <- seq(1, max(d_sim_binom_cov$ID), by = 5)

df_er_ae_covsel_test <-
  d_sim_binom_cov |>
  dplyr::filter(
    AETYPE == "ae_covsel_test",
    ID %in% id_to_sample
  ) |>
  dplyr::mutate(
    AUCss_1000 = AUCss / 1000, BAGE_10 = BAGE / 10,
    BWT_10 = BWT / 10, BHBA1C_5 = BHBA1C / 5
  )

df_er_ae_covsel_test_small <-
  d_sim_binom_cov |>
  dplyr::filter(
    AETYPE == "ae_covsel_test",
    ID %in% id_to_sample2
  ) |>
  dplyr::mutate(
    AUCss_1000 = AUCss / 1000, BAGE_10 = BAGE / 10,
    BWT_10 = BWT / 10, BHBA1C_5 = BHBA1C / 5
  )

df_er_dr2 <-
  d_sim_binom_cov |>
  dplyr::filter(
    AETYPE == "dr2",
    ID %in% id_to_sample
  ) |>
  dplyr::mutate(
    AUCss_1000 = AUCss / 1000, BAGE_10 = BAGE / 10,
    BWT_10 = BWT / 10
  )

var_resp <- "AEFLAG"
var_exp_candidates <- c("AUCss_1000", "Cmaxss", "Cminss")
# Exclude BGLUC to reduce test time
var_cov_ae_covsel_test <- c("BAGE_10", "BWT_10", "BHBA1C_5", "RACE", "VISC")


# Data for normal linear model
data(d_sim_lin)

# dev_ermod_bin_exp_sel -------------------------------------------------
set.seed(1234)
ermod_bin_exp_sel <-
  suppressWarnings(dev_ermod_bin_exp_sel(
    data = df_er_ae_covsel_test_small,
    var_resp = var_resp,
    var_exp_candidates = var_exp_candidates,
    prior = rstanarm::normal(location = 0, scale = 5, autoscale = TRUE),
    prior_intercept =
      rstanarm::normal(location = 0, scale = 4, autoscale = TRUE),
    verbosity_level = 0,
    chains = 1,
    iter = 200
  ))

ermod_bin_exp_sel_other_rank <- ermod_bin_exp_sel
rownames(ermod_bin_exp_sel_other_rank$loo_comp_exposures) <-
  c("Cminss", "AUCss_1000", "Cmaxss")

ermod_bin_exp_sel_one_metric <-
  suppressWarnings(dev_ermod_bin_exp_sel(
    data = df_er_ae_covsel_test_small,
    var_resp = var_resp,
    var_exp_candidates = "AUCss_1000",
    verbosity_level = 0,
    chains = 1,
    iter = 200
  ))


# dev_ermod_bin_cov_sel steps ------------------------------------------
if (requireNamespace("projpred")) {
  refm_obj <- .dev_ermod_refmodel(
    data = df_er_ae_covsel_test,
    var_resp = var_resp,
    var_exposure = "AUCss_1000",
    var_cov_candidates = var_cov_ae_covsel_test,
    verbosity_level = 0,
    # Below option to make the test fast
    chains = 2, iter = 1000
  )

  var_selected <- .select_cov_projpred(
    refm_obj = refm_obj,
    var_exposure = "AUCss_1000",
    var_cov_candidates = var_cov_ae_covsel_test,
    # Set nterms_max to 4 to make the test fast
    nterms_max = 3,
    cv_method = "LOO",
    validate_search = FALSE,
    verbosity_level = 0
  )

  mod_final <- dev_ermod_bin(
    data = df_er_ae_covsel_test,
    var_resp = var_resp,
    var_exposure = "AUCss_1000",
    var_cov = "BHBA1C_5",
    verbosity_level = 0,
    # Below option to make the test fast
    chains = 2, iter = 1000
  )

  ## Test k-fold CV
  set.seed(1234)
  var_selected_kfold <-
    suppressMessages(.select_cov_projpred(
      refm_obj = refm_obj,
      var_exposure = "AUCss_1000",
      var_cov_candidates = var_cov_ae_covsel_test,
      # Set nterms_max to 3 to make the test fast
      nterms_max = 3,
      cv_method = "kfold",
      k = 2,
      validate_search = TRUE,
      verbosity_level = 0
    ))

  ermod_bin_cov_sel_kfold_dummy <- list(
    cvvs = attr(var_selected_kfold, "cvvs"),
    rk = attr(var_selected_kfold, "rk")
  )
  class(ermod_bin_cov_sel_kfold_dummy) <-
    c("ermod_bin_cov_sel", "ermod_cov_sel", "ermod_bin", "ermod")

  # dev_ermod_bin_cov_sel ------------------------------------------------
  prior_intercept <-
    rstanarm::normal(location = 0, scale = 4, autoscale = TRUE)
  set.seed(1234)
  ermod_bin_cov_sel <-
    dev_ermod_bin_cov_sel(
      data = df_er_ae_covsel_test,
      var_resp = var_resp,
      var_exposure = "AUCss_1000",
      var_cov_candidates = var_cov_ae_covsel_test,
      cv_method = "LOO",
      prior = rstanarm::normal(location = 0, scale = 5, autoscale = TRUE),
      prior_intercept = prior_intercept,
      nterms_max = 3,
      verbosity_level = 0,
      chains = 2,
      iter = 1000
    )
}

# lm -------------------------------------------------------------------
set.seed(1234)

ermod_lin <- dev_ermod_lin(
  data = d_sim_lin,
  var_resp = "response",
  var_exposure = "AUCss",
  var_cov = c("SEX", "BAGE"),
  prior = rstanarm::normal(location = 0, scale = 5, autoscale = TRUE),
  prior_aux = rstanarm::exponential(rate = 6, autoscale = TRUE),
  chains = 2,
  iter = 1000
)

ermod_lin_exp_sel <- suppressMessages(dev_ermod_lin_exp_sel(
  data = d_sim_lin,
  var_resp = "response",
  var_exp_candidates = c("AUCss", "Cmaxss"),
  prior = rstanarm::normal(location = 0, scale = 5, autoscale = TRUE),
  prior_aux = rstanarm::exponential(rate = 6, autoscale = TRUE),
  chains = 2,
  iter = 1000
))


if (requireNamespace("projpred")) {
  ermod_lin_cov_sel <- suppressMessages(suppressWarnings(dev_ermod_lin_cov_sel(
    data = d_sim_lin,
    var_resp = "response",
    var_exposure = "AUCss",
    var_cov_candidates = c("BAGE", "SEX"),
    nterms_max = 2,
    prior = rstanarm::normal(location = 0, scale = 5, autoscale = TRUE),
    prior_aux = rstanarm::exponential(rate = 6, autoscale = TRUE),
    verbosity_level = 1,
    chains = 2,
    iter = 1000
  )))
}

# Convert factor as response variable into number
data("mtcars")
mtcars$am2 <- factor(mtcars$am, labels = c("Automatic", "Manual"))
mtcars$am3 <- as.numeric(mtcars$am2) - 1

mod_mtcars <- dev_ermod_bin(
  data = mtcars,
  var_resp = "am3",
  var_exposure = "cyl",
  verbosity_level = 0,
  # Below option to make the test fast
  chains = 2, iter = 1000
)

# Test ------------------------------------------------------------------

test_that("Exposure metrics selection", {
  expect_equal(extract_var_exposure(ermod_bin_exp_sel), "AUCss_1000")
  if ("model" %in% colnames(extract_exp_sel_comp(ermod_bin_exp_sel))) {
    expect_equal(
      extract_exp_sel_comp(ermod_bin_exp_sel)[, 2],
      c(0.000000, -0.6490312, -2.7563957)
    )
  } else {
    expect_equal(
      extract_exp_sel_comp(ermod_bin_exp_sel)[, 1],
      c(AUCss_1000 = 0.000000, Cmaxss = -0.6490312, Cminss = -2.7563957)
    )
  }

  priors <- prior_summary(ermod_bin_exp_sel$l_mod_exposures[[2]])
  expect_equal(priors$prior$scale, 5)
  expect_equal(priors$prior_intercept$scale, 4)

  if (.if_run_ex_plot_er()) {
    g1 <- plot_er_exp_sel(ermod_bin_exp_sel, n_draws_sim = 10)
    g2 <- plot_er_exp_sel(ermod_bin_exp_sel_other_rank, n_draws_sim = 10)

    expect_silent(plot(g1))

    expect_equal(
      levels(g1$data$.exp_met_fct), c("AUCss_1000", "Cmaxss", "Cminss")
    )
    expect_equal(
      levels(g2$data$.exp_met_fct), c("Cminss", "AUCss_1000", "Cmaxss")
    )
  }
})

test_that("Full model dev", {
  if (requireNamespace("projpred")) {
    expect_equal(coef(refm_obj$fit)[[1]], -12.407539)
  }
})

test_that("Variable selection", {
  if (requireNamespace("projpred")) {
    expect_equal(as.character(var_selected), c("AUCss_1000", "BHBA1C_5"))
    expect_equal(
      extract_var_selected(ermod_bin_cov_sel),
      c("AUCss_1000", "BHBA1C_5")
    )

    priors <- prior_summary(extract_mod(ermod_bin_cov_sel))
    expect_equal(priors$prior$scale, c(5, 5))
    expect_equal(priors$prior_intercept$scale, 4)

    rk <- attr(var_selected_kfold, "rk")
    expect_equal(rk$foldwise[, 3], c("RACE", "VISC"))

    plot_submod_performance(ermod_bin_cov_sel) |>
      expect_silent()
    plot_var_ranking(ermod_bin_cov_sel_kfold_dummy) |>
      expect_silent()
  }
})

test_that("Final model", {
  if (requireNamespace("projpred")) {
    expect_equal(coef(ermod_bin_cov_sel)[[1]], -10.9133063)
  }
})

test_that("No ER case", {
  set.seed(1234)

  if (requireNamespace("projpred")) {
    refm_obj_dr2 <- .dev_ermod_refmodel(
      data = df_er_dr2,
      var_resp = var_resp,
      var_exposure = "AUCss_1000",
      var_cov_candidates = c("BAGE_10", "BWT_10"),
      verbosity_level = 0,
      # Below option to make the test fast
      chains = 2, iter = 1000
    )

    .select_cov_projpred(
      refm_obj = refm_obj_dr2,
      var_exposure = "AUCss_1000",
      var_cov_candidates = c("BAGE_10", "BWT_10"),
      # Set nterms_max to 4 to make the test fast
      nterms_max = 1,
      cv_method = "LOO",
      validate_search = FALSE,
      verbosity_level = 1
    ) |>
      expect_message("No variables selected") |>
      expect_message("The variables selected were: AUCss_1000")
  }
})


test_that("NAs in data", {
  df_na_test <- df_er_ae_covsel_test
  df_na_test$AUCss_1000[1] <- NA

  check_data_columns(
    data = df_na_test,
    var_exp_candidates = var_exp_candidates
  ) |>
    expect_error("NA values not allowed in the exposure metric")

  df_na_test <- df_er_ae_covsel_test
  df_na_test$BHBA1C_5[1] <- NA

  check_data_columns(
    data = df_na_test,
    var_cov_candidates = var_cov_ae_covsel_test
  ) |>
    expect_error("NA values not allowed in the covariate column")

  df_na_test <- df_er_ae_covsel_test
  df_na_test$AEFLAG[1] <- NA

  check_data_columns(
    data = df_na_test,
    var_resp = var_resp
  ) |>
    expect_error("NA values not allowed in the response column")
})

test_that("Missing column", {
  df_na_test <- df_er_ae_covsel_test

  check_data_columns(
    data = df_na_test,
    var_exp_candidates = "AUCaa"
  ) |>
    expect_error('"AUCaa" columns should be in data')


  check_data_columns(
    data = df_na_test,
    var_cov_candidates = c("TEST", "TEST2", "BHBA1C_5", "test3")
  ) |>
    expect_error('"TEST, TEST2, BHBA1C_5, test3" columns should be in data')
})

test_that("lm", {
  as_draws(ermod_lin) |>
    dim() |>
    expect_equal(c(1000, 8))
  priors <- prior_summary(extract_mod(ermod_lin))
  expect_equal(priors$prior$scale, rep(5, 3))
  expect_equal(priors$prior_aux$rate, 6)

  ermod_lin_exp_sel$loo_comp_exposures |> expect_s3_class("compare.loo")
  priors <- prior_summary(extract_mod(ermod_lin_exp_sel$l_mod_exposures[[2]]))
  expect_equal(priors$prior$scale, 5)
  expect_equal(priors$prior_aux$rate, 6)

  if (requireNamespace("projpred")) {
    ermod_lin_cov_sel$cvvs |> expect_s3_class("vsel")
    expect_equal(ermod_lin_cov_sel$var_selected, c("AUCss", "BAGE"))
    priors <- prior_summary(extract_mod(ermod_lin_cov_sel))
    expect_equal(priors$prior$scale, c(5, 5))
    expect_equal(priors$prior_aux$rate, 6)
  }
})

test_that("plot_er_gof for lin", {
  if (.if_run_ex_plot_er()) {
    plot_er_gof(ermod_lin_exp_sel, show_coef_exp = TRUE) |>
      expect_silent()
  }
})

test_that("Convert factor as response variable into number", {
  dev_ermod_bin(
    data = mtcars,
    var_resp = "am2",
    var_exposure = "cyl",
  ) |>
    expect_error("Response variable must be a numeric of 0 and 1")

  dev_ermod_bin(
    data = mtcars,
    var_resp = "cyl",
    var_exposure = "am",
  ) |>
    expect_error("Response variable must be a numeric of 0 and 1")

  if (.if_run_ex_plot_er()) {
    plot_er_gof(mod_mtcars, n_bins = 4) |>
      expect_error("The breaks for the binned probability ")
    plot_er_gof(mod_mtcars, n_bins = 2) |> expect_silent()
  }
})

test_that("print.ermod_exp_sel prints correct information", {
  out <- cli::cli_fmt({
    print(ermod_bin_exp_sel)
  })
  expect_true(any(grepl("Binary ER model", out)))
  expect_true(any(grepl("& exposure metric selection", out)))
  expect_true(any(grepl("Use \\`plot_er_exp_sel\\(\\)\\` for ER curve", out)))
})

test_that("print.ermod_cov_sel prints correct information", {
  if (requireNamespace("projpred")) {
    out <- cli::cli_fmt({
      print(ermod_bin_cov_sel_kfold_dummy)
    })
    expect_true(any(grepl("Binary ER model", out)))
    expect_true(any(grepl("& covariate selection", out)))
    expect_true(any(grepl("Use \\`plot_submod_performance\\(\\)\\`", out)))
  }
})

# plot.ermod_exp_sel
test_that("plot.ermod_exp_sel calls plot_er_exp_sel", {
  if (.if_run_ex_plot_er()) {
    expect_silent(plot.ermod_exp_sel(ermod_bin_exp_sel))
  }
})

# plot.ermod_cov_sel
test_that("plot.ermod_cov_sel calls plot_submod_performance", {
  if (requireNamespace("projpred")) {
    expect_silent(plot.ermod_cov_sel(ermod_lin_cov_sel))
  }
})

test_that("test for errors and warnings", {
  if (requireNamespace("projpred")) {
    .select_cov_projpred(
      var_exposure = c("AUC", "Cmax"),
      validate_search = TRUE
    ) |>
      expect_error(
        "Only one exposure metric should be provided"
      )
    .select_cov_projpred(
      var_exposure = "AUC",
      var_cov_candidates = "cov",
      cv_method = "LOO",
      validate_search = TRUE
    ) |>
      expect_error(
        "validate_search should be set to FALSE for LOO,"
      )
  }
})

test_that("extract_coef_exp_ci for exp_candidates", {
  extract_coef_exp_ci(ermod_bin_exp_sel, exp_candidates = TRUE) |>
    dim() |>
    expect_equal(c(3, 2))
  extract_coef_exp_ci(ermod_lin, exp_candidates = TRUE) |>
    expect_error("exp_candidates = TRUE only supported")
})

if (.if_run_ex_p_dir() && .if_run_ex_covsel()) {
  require(bayestestR)

  d_sim_binom_cov <- d_sim_binom_cov

  id_to_sample <- seq(1, max(d_sim_binom_cov$ID), by = 5)

  df_er_ae_covsel_test <-
    d_sim_binom_cov |>
    dplyr::filter(
      AETYPE == "ae_covsel_test",
      ID %in% id_to_sample
    ) |>
    dplyr::mutate(
      AUCss_1000 = AUCss / 1000, BAGE_10 = BAGE / 10,
      BWT_10 = BWT / 10, BHBA1C_5 = BHBA1C / 5
    )


  # Create  ermod_bin objects for testing -------------------------------------
  set.seed(1234)
  ermod_bin <- dev_ermod_bin(
    data = df_er_ae_covsel_test,
    var_resp = "AEFLAG",
    var_exposure = "AUCss_1000",
    var_cov = "BHBA1C_5",
    verbosity_level = 0,
    # Need large number of samples to calculate p_direction precisely
    chains = 4, iter = 2000
  )

  ermod_bin_cov_sel <- new_ermod_bin_cov_sel(
    mod = extract_mod(ermod_bin),
    data = df_er_ae_covsel_test,
    var_resp = "AEFLAG",
    var_exposure = "AUCss_1000",
    var_cov_candidates = c("BAGE_10", "BWT_10", "BHBA1C_5", "RACE", "VISC"),
    var_cov = c("BHBA1C_5"),
    var_selected = c("AUCss_1000", "BHBA1C_5"),
    cv_method = c("LOO", "kfold"),
    cvvs = NULL,
    rk = NULL
  )

  test_that("p_direction", {
    expect_equal(p_direction(ermod_bin_cov_sel, as_num = TRUE), 0.9955)
    expect_equal(
      p_direction(ermod_bin_cov_sel, as_num = TRUE, as_p = TRUE),
      0.009
    )
  })
}

if (.if_run_ex_eval_mod()) {
  # Linear logistic regression ------------------------------------------------
  set.seed(1234)
  data(d_sim_binom_cov_hgly2)
  d_split <- rsample::initial_split(d_sim_binom_cov_hgly2)
  d_train <- rsample::training(d_split)
  d_test <- rsample::testing(d_split)

  ermod_bin <- dev_ermod_bin(
    data = d_train |> head(100),
    var_resp = "AEFLAG",
    var_exposure = "AUCss_1000",
    var_cov = "BHBA1C_5",
    chains = 2,
    iter = 500
  )

  met_bin_training <- eval_ermod(ermod_bin, eval_type = "training")
  met_bin_test <- eval_ermod(ermod_bin, eval_type = "test", newdata = d_test)
  met_bin_kfold <-
    eval_ermod(ermod_bin, eval_type = "kfold", newdata = d_test, k = 3) |>
    suppressWarnings()

  # Emax logistic regression --------------------------------------------------
  set.seed(1234)
  data_er_bin <- rstanemax::exposure.response.sample.binary
  ermod_bin_emax <-
    suppressWarnings(dev_ermod_bin_emax(
      data = data_er_bin,
      var_exposure = "conc",
      var_resp = "y_cov",
      l_var_cov = list(emax = "sex"),
      verbosity_level = 0,
      chains = 2,
      iter = 200,
      seed = 1
    ))

  met_bin_emax_training <- eval_ermod(ermod_bin_emax, eval_type = "training")
  met_bin_emax_kfold <-
    eval_ermod(ermod_bin_emax, eval_type = "kfold", k = 3) |>
    suppressWarnings()

  # Linear regression ---------------------------------------------------------
  set.seed(1234)
  ermod_lin <- dev_ermod_lin(
    data = d_sim_lin,
    var_resp = "response",
    var_exposure = "AUCss",
    var_cov = c("SEX", "BAGE"),
    chains = 2,
    iter = 500
  )

  met_lin_training <- eval_ermod(ermod_lin, eval_type = "training")
  met_lin_kfold <-
    eval_ermod(ermod_lin, eval_type = "kfold", k = 3) |>
    suppressWarnings()

  # Emax model ----------------------------------------------------------------
  set.seed(1234)
  data_er_cont_cov <- rstanemax::exposure.response.sample.with.cov

  ermod_emax <-
    suppressWarnings(dev_ermod_emax(
      data = data_er_cont_cov,
      var_exposure = "conc",
      var_resp = "resp",
      l_var_cov = list(emax = "cov2", ec50 = "cov3", e0 = "cov1"),
      verbosity_level = 0,
      chains = 2,
      iter = 200,
      seed = 1
    ))

  met_emax_training <- eval_ermod(ermod_emax, eval_type = "training")
  met_emax_kfold <-
    eval_ermod(ermod_emax, eval_type = "kfold", k = 3) |>
    suppressWarnings()

  # Test ----------------------------------------------------------------------
  test_that("Binary model", {
    # Training
    ersim_bin <- sim_er(ermod_bin, output_type = "median_qi") |>
      dplyr::mutate(AEFLAG = factor(AEFLAG, levels = c(1, 0)))

    estimates <- c(
      yardstick::roc_auc_vec(ersim_bin$AEFLAG, ersim_bin$.epred),
      yardstick::mn_log_loss_vec(ersim_bin$AEFLAG, ersim_bin$.epred)
    )

    expect_equal(met_bin_training$.estimate, estimates)

    # Test
    ersim_bin_test <-
      sim_er(ermod_bin, newdata = d_test, output_type = "median_qi") |>
      dplyr::mutate(AEFLAG = factor(AEFLAG, levels = c(1, 0)))

    estimates_test <- c(
      yardstick::roc_auc_vec(ersim_bin_test$AEFLAG, ersim_bin_test$.epred),
      yardstick::mn_log_loss_vec(ersim_bin_test$AEFLAG, ersim_bin_test$.epred)
    )

    expect_equal(met_bin_test$.estimate, estimates_test)

    # kfold
    expect_equal(dim(met_bin_kfold), c(6, 4))

    # Binary emax
    expect_error(
      eval_ermod(ermod_bin_emax, eval_type = "test", newdata = d_test),
      'conc" columns should be in data'
    )

    expect_equal(dim(met_bin_emax_training), c(2, 3))
    expect_equal(dim(met_bin_emax_kfold), c(6, 4))
  })

  test_that("Continuous model", {
    ersim_cont <- sim_er(ermod_lin, output_type = "median_qi")

    estimates <- c(
      yardstick::rmse_vec(ersim_cont$response, ersim_cont$.epred),
      yardstick::rsq_vec(ersim_cont$response, ersim_cont$.epred),
      yardstick::rsq_trad_vec(ersim_cont$response, ersim_cont$.epred)
    )

    expect_equal(met_lin_training$.estimate, estimates)

    # Emax
    expect_equal(dim(met_emax_training), c(3, 3))
    expect_equal(dim(met_emax_kfold), c(9, 4))
  })
}

# Emax ------------------------------------------------------------------------

#' Develop Emax model for continuous and binary endpoint
#'
#' These functions are used to develop an Emax model with continuous or binary
#' endpoint.
#' You can also specify covariates to be included in the model; note that only
#' categorical covariates are allowed.
#'
#' @export
#' @inheritParams dev_ermod_bin
#' @param l_var_cov a names list of categorical covariate variables in
#' character vector. See details in the `param.cov` argument of
#'  [rstanemax::stan_emax()] or [rstanemax::stan_emax_binary()]
#' @param gamma_fix Hill coefficient, default fixed to 1.
#' See details in [rstanemax::stan_emax()] or [rstanemax::stan_emax_binary()]
#' @param e0_fix See details in [rstanemax::stan_emax()] or
#' [rstanemax::stan_emax_binary()]
#' @param emax_fix See details in [rstanemax::stan_emax()] or
#'  [rstanemax::stan_emax_binary()]
#' @param priors See details in [rstanemax::stan_emax()] or
#' [rstanemax::stan_emax_binary()]
#' @param seed Random seed for Stan model execution, see details in
#' [rstan::sampling()] which is used in [rstanemax::stan_emax()] or
#' [rstanemax::stan_emax_binary()]
#'
#' @return An object of class `ermod_emax`.or `ermod_bin_emax`.
#'
#' @examplesIf BayesERtools:::.if_run_ex_plot_er()
#' \donttest{
#' data_er_cont <- rstanemax::exposure.response.sample
#'
#' ermod_emax <-
#'   dev_ermod_emax(
#'     data = data_er_cont,
#'     var_exposure = "exposure",
#'     var_resp = "response"
#'   )
#'
#' plot_er(ermod_emax, show_orig_data = TRUE)
#'
#' data_er_cont_cov <- rstanemax::exposure.response.sample.with.cov
#'
#' ermod_emax_w_cov <-
#'   dev_ermod_emax(
#'     data = data_er_cont_cov,
#'     var_exposure = "conc",
#'     var_resp = "resp",
#'     l_var_cov = list(emax = "cov2", ec50 = "cov3", e0 = "cov1")
#'   )
#' }
#'
dev_ermod_emax <- function(
    data,
    var_resp,
    var_exposure,
    l_var_cov = NULL,
    gamma_fix = 1,
    e0_fix = NULL,
    emax_fix = NULL,
    priors = NULL,
    verbosity_level = 1,
    chains = 4,
    iter = 2000,
    seed = sample.int(.Machine$integer.max, 1)) {
  input_args <- capture_selected_args(
    c(
      "gamma_fix", "e0_fix", "emax_fix",
      "priors", "chains", "iter"
    ),
    environment()
  )

  stopifnot(verbosity_level %in% c(0, 1, 2, 3))
  refresh <- dplyr::if_else(verbosity_level >= 3, iter %/% 4, 0)

  check_data_columns(
    data = data,
    var_exposure = var_exposure,
    var_resp = var_resp,
    var_cov = unlist(l_var_cov)
  )

  formula <-
    stats::formula(paste(var_resp, "~", var_exposure))

  mod <- rstanemax::stan_emax(
    formula,
    data = data,
    gamma.fix = gamma_fix,
    e0.fix = e0_fix,
    emax.fix = emax_fix,
    priors = priors,
    param.cov = l_var_cov,
    refresh = refresh,
    chains = chains,
    iter = iter,
    seed = seed
  )

  new_ermod_emax(
    mod = mod,
    data = data,
    var_resp = var_resp,
    var_exposure = var_exposure,
    l_var_cov = l_var_cov,
    input_args = input_args
  )
}


#' Exposure metrics selection for Emax models
#'
#' This functions is used to develop an Emax model with binary and continuous
#' endpoint, using various exposure metrics and selecting the best one.
#'
#' @export
#' @inheritParams dev_ermod_emax
#' @inheritParams dev_ermod_bin_exp_sel
#'
#' @return An object of class `ermod_emax_exp_sel` or `ermod_bin_emax_exp_sel`.
#'
#' @examples
#' \donttest{
#' data_er_cont <- rstanemax::exposure.response.sample
#' noise <- 1 + 0.5 * stats::rnorm(length(data_er_cont$exposure))
#' data_er_cont$exposure2 <- data_er_cont$exposure * noise
#' # Replace exposure < 0 with 0
#' data_er_cont$exposure2[data_er_cont$exposure2 < 0] <- 0
#'
#' ermod_emax_exp_sel <-
#'   dev_ermod_emax_exp_sel(
#'     data = data_er_cont,
#'     var_resp = "response",
#'     var_exp_candidates = c("exposure", "exposure2")
#'   )
#'
#' ermod_emax_exp_sel
#' }
#'
dev_ermod_emax_exp_sel <- function(
    data,
    var_resp,
    var_exp_candidates,
    verbosity_level = 1,
    chains = 4,
    iter = 2000,
    gamma_fix = 1,
    e0_fix = NULL,
    emax_fix = NULL,
    priors = NULL,
    seed = sample.int(.Machine$integer.max, 1)) {
  fun_dev_ermod <-
    purrr::partial(
      dev_ermod_emax,
      gamma_fix = gamma_fix,
      e0_fix = e0_fix,
      emax_fix = emax_fix,
      priors = priors,
      seed = seed
    )

  l_out <-
    .dev_ermod_exp_sel(
      data = data,
      var_resp = var_resp,
      var_exp_candidates = var_exp_candidates,
      verbosity_level = verbosity_level,
      chains = chains,
      iter = iter,
      fun_dev_ermod = fun_dev_ermod
    )

  new_ermod_emax_exp_sel(l_out)
}

# Binary Emax -----------------------------------------------------------------

#' @export
#' @rdname dev_ermod_emax
#' @examplesIf BayesERtools:::.if_run_ex_plot_er()
#' \donttest{
#' data_er_bin <- rstanemax::exposure.response.sample.binary
#'
#' ermod_bin_emax <-
#'   dev_ermod_bin_emax(
#'     data = data_er_bin,
#'     var_exposure = "conc",
#'     var_resp = "y"
#'   )
#'
#' plot_er(ermod_bin_emax, show_orig_data = TRUE)
#'
#' ermod_bin_emax_w_cov <-
#'   dev_ermod_bin_emax(
#'     data = data_er_bin,
#'     var_exposure = "conc",
#'     var_resp = "y_cov",
#'     l_var_cov = list(emax = "sex")
#'   )
#' }
#'
dev_ermod_bin_emax <- function(
    data,
    var_resp,
    var_exposure,
    l_var_cov = NULL,
    gamma_fix = 1,
    e0_fix = NULL,
    emax_fix = NULL,
    priors = NULL,
    verbosity_level = 1,
    chains = 4,
    iter = 2000,
    seed = sample.int(.Machine$integer.max, 1)) {
  # Warn when e0_fix is set
  if (!is.null(e0_fix) && e0_fix == 0) {
    warning(
      "e0_fix is set to 0 (on logit scale), which is equivalent to assuming",
      "that the probability of events at the exposure = 0 is 50%.\n",
      "You can ignore this warning if this was the intended behavior."
    )
  } else if (!is.null(e0_fix)) {
    message(
      "e0_fix is set at ", e0_fix,
      ". Note that this is on logit scale. ",
      "it is equivalent to fixing the probability at ",
      round(100 * stats::plogis(e0_fix)), "%.\n",
      "You can ignore this message if this was the intended behavior."
    )
  }

  # Warn when em_fix is set
  if (!is.null(emax_fix)) {
    message(
      "emax_fix is set at ", emax_fix,
      ". Note that this is on logit scale. ",
      "it is equivalent to fixing the probability at ",
      round(100 * stats::plogis(emax_fix)), "%.\n",
      "You can ignore this message if this was the intended behavior."
    )
  }

  input_args <- capture_selected_args(
    c(
      "gamma_fix", "e0_fix", "emax_fix",
      "priors", "chains", "iter"
    ),
    environment()
  )

  stopifnot(verbosity_level %in% c(0, 1, 2, 3))
  refresh <- dplyr::if_else(verbosity_level >= 3, iter %/% 4, 0)

  check_data_columns(
    data = data,
    var_exposure = var_exposure,
    var_resp = var_resp,
    var_cov = unlist(l_var_cov),
    is_binary = TRUE
  )

  formula <-
    stats::formula(paste(var_resp, "~", var_exposure))

  mod <- rstanemax::stan_emax_binary(
    formula,
    data = data,
    gamma.fix = gamma_fix,
    e0.fix = e0_fix,
    emax.fix = emax_fix,
    priors = priors,
    param.cov = l_var_cov,
    refresh = refresh,
    chains = chains,
    iter = iter,
    seed = seed
  )

  new_ermod_bin_emax(
    mod = mod,
    data = data,
    var_resp = var_resp,
    var_exposure = var_exposure,
    l_var_cov = l_var_cov,
    input_args = input_args
  )
}


#' @export
#' @rdname dev_ermod_emax_exp_sel
#'
#' @examples
#' \donttest{
#' data_er_bin <- rstanemax::exposure.response.sample.binary
#'
#' noise <- 1 + 0.5 * stats::rnorm(length(data_er_bin$conc))
#' data_er_bin$conc2 <- data_er_bin$conc * noise
#' data_er_bin$conc2[data_er_bin$conc2 < 0] <- 0
#'
#' ermod_bin_emax_exp_sel <-
#'   dev_ermod_bin_emax_exp_sel(
#'     data = data_er_bin,
#'     var_resp = "y",
#'     var_exp_candidates = c("conc", "conc2")
#'   )
#' }
#'
dev_ermod_bin_emax_exp_sel <- function(
    data,
    var_resp,
    var_exp_candidates,
    verbosity_level = 1,
    chains = 4,
    iter = 2000,
    gamma_fix = 1,
    e0_fix = NULL,
    emax_fix = NULL,
    priors = NULL,
    seed = sample.int(.Machine$integer.max, 1)) {
  fun_dev_ermod <-
    purrr::partial(
      dev_ermod_bin_emax,
      gamma_fix = gamma_fix,
      e0_fix = e0_fix,
      emax_fix = emax_fix,
      priors = priors,
      seed = seed
    )

  l_out <-
    .dev_ermod_exp_sel(
      data = data,
      var_resp = var_resp,
      var_exp_candidates = var_exp_candidates,
      verbosity_level = verbosity_level,
      chains = chains,
      iter = iter,
      fun_dev_ermod = fun_dev_ermod
    )

  new_ermod_bin_emax_exp_sel(l_out)
}

if (.if_run_ex_eval_mod()) {
  d_sim_binom_cov <- d_sim_binom_cov

  id_to_sample <- seq(1, max(d_sim_binom_cov$ID), by = 5)

  df_er_ae_covsel_test <-
    d_sim_binom_cov |>
    dplyr::filter(
      AETYPE == "ae_covsel_test",
      ID %in% id_to_sample
    ) |>
    dplyr::mutate(AUCss_1000 = AUCss / 1000, BHBA1C_5 = BHBA1C / 5)

  data_er_cont <- rstanemax::exposure.response.sample
  data_er_cont_cov <- rstanemax::exposure.response.sample.with.cov
  data_er_bin <- rstanemax::exposure.response.sample.binary

  # Create  ermod_bin objects for testing --------------------------------------
  set.seed(1234)
  ermod_bin <- dev_ermod_bin(
    data = df_er_ae_covsel_test,
    var_resp = "AEFLAG",
    var_exposure = "AUCss_1000",
    var_cov = "BHBA1C_5",
    verbosity_level = 0,
    # Below option to make the test fast
    chains = 2, iter = 1000
  )

  ermod_bin_wo_cov <- dev_ermod_bin(
    data = df_er_ae_covsel_test,
    var_resp = "AEFLAG",
    var_exposure = "AUCss_1000",
    verbosity_level = 0,
    # Below option to make the test fast
    chains = 2, iter = 1000
  )

  set.seed(1234)
  ermod_emax_w_cov <-
    dev_ermod_emax(
      data = data_er_cont_cov,
      var_exposure = "conc",
      var_resp = "resp",
      l_var_cov = list(emax = "cov2", ec50 = "cov3", e0 = "cov1"),
      verbosity_level = 0,
      chains = 2,
      iter = 1000,
      seed = 1
    )

  ermod_bin_emax <-
    suppressWarnings(dev_ermod_bin_emax(
      data = df_er_ae_covsel_test,
      var_resp = "AEFLAG",
      var_exposure = "AUCss_1000",
      verbosity_level = 0,
      # Increase iter for better convergence as exact reproducibility
      # over different machines doesn't seem realistic
      chains = 2,
      iter = 1000,
      seed = 1
    ))

  set.seed(1234)

  kfold_ermod_bin <- kfold(ermod_bin, k = 3, seed = 1)
  kfold_ermod_bin_rstanarm <- suppressMessages(rstanarm::kfold(
    extract_mod(ermod_bin),
    K = 3,
    folds = kfold_ermod_bin$d_truth$fold_id # Use the same fold
  ))

  comp <- loo::loo_compare(kfold_ermod_bin_rstanarm, kfold_ermod_bin)

  set.seed(1234)
  kfold_ermod_emax <- suppressWarnings(
    kfold(ermod_emax_w_cov, k = 3, seed = 123)
  )

  # Test ----------------------------------------------------------------------
  test_that("loo", {
    loo_ermod_bin <- loo(ermod_bin)
    loo_ermod_emax_w_cov <- suppressWarnings(loo(ermod_emax_w_cov))
    loo_ermod_bin_emax <- loo(ermod_bin_emax)

    expect_equal(
      loo_ermod_bin$estimates[, 1],
      c(elpd_loo = -38.5289466, p_loo = 3.3262640, looic = 77.0578931),
      tolerance = 0.1
    )
    expect_equal(
      loo_ermod_emax_w_cov$estimates[, 1],
      c(elpd_loo = -216.539552, p_loo = 7.765598, looic = 433.079103),
      tolerance = 0.1
    )
    expect_equal(
      loo_ermod_bin_emax$estimates[, 1],
      c(elpd_loo = -60.787274, p_loo = 1.427765, looic = 121.574548),
      tolerance = 0.1
    )

    expect_silent(loo::loo_compare(loo_ermod_bin, loo_ermod_bin_emax))
  })

  test_that("kfold", {
    if ("model" %in% colnames(comp)) {
      expect_gt(comp[[2, 2]], -0.5)
    } else {
      expect_gt(comp[[2, 1]], -0.5)
    }
    expect_equal(
      kfold_ermod_bin$estimates[, 1],
      c(elpd_kfold = -38.305782, p_kfold = 3.103099, kfoldic = 76.611563),
      tolerance = 0.05
    )
    expect_equal(
      class(extract_kfold_loo(kfold_ermod_bin)),
      c("kfold", "loo")
    )
    expect_equal(
      kfold_ermod_emax$estimates[, 1],
      c(elpd_kfold = -218, p_kfold = 9, kfoldic = 435),
      tolerance = 0.1
    )
  })

  # Test for other models are covered in test-eval_ermod.R with kfold-cv eval
  test_that("binary model", {
    expect_equal(
      names(kfold_ermod_bin$d_truth),
      c(".row", "truth", "fold_id")
    )
    expect_equal(
      names(kfold_ermod_bin$d_sim),
      c(".row", ".draw", "pred", "fold_id")
    )
    expect_equal(dim(kfold_ermod_bin$d_sim), c(100 * 1000, 4))

    out <- cli::cli_fmt({
      print(kfold_ermod_bin)
    })
    expect_true(any(grepl("k-fold Cross-Validation for ermod object", out)))
  })
}

set.seed(1234)
data_er_cont <- rstanemax::exposure.response.sample
data_er_cont_cov <- rstanemax::exposure.response.sample.with.cov

noise <- 1 + 0.5 * stats::rnorm(length(data_er_cont$exposure))
data_er_cont$exposure2 <- data_er_cont$exposure * noise
# Replace exposure < 0 with 0
data_er_cont$exposure2[data_er_cont$exposure2 < 0] <- 0


data_er_bin <- rstanemax::exposure.response.sample.binary

noise <- 1 + 0.5 * stats::rnorm(length(data_er_bin$conc))
data_er_bin$conc2 <- data_er_bin$conc * noise
# Replace exposure < 0 with 0
data_er_bin$conc2[data_er_bin$conc2 < 0] <- 0


set.seed(1234)

# Test ermod_emax ------------------------------------------------------------
ermod_emax <-
  dev_ermod_emax(
    data = data_er_cont,
    var_exposure = "exposure",
    var_resp = "response",
    verbosity_level = 0,
    # Increase iter for better convergence as exact reproducibility
    # over different machines doesn't seem realistic
    chains = 2,
    iter = 1000,
    seed = 1
  )

ermod_emax_w_cov <-
  suppressWarnings(dev_ermod_emax(
    data = data_er_cont_cov,
    var_exposure = "conc",
    var_resp = "resp",
    l_var_cov = list(emax = "cov2", ec50 = "cov3", e0 = "cov1"),
    verbosity_level = 0,
    chains = 2,
    iter = 200,
    seed = 1
  ))

ersim_med_qi <- sim_er(ermod_emax, output_type = "median_qi")


ermod_emax_exp_sel <-
  suppressWarnings(dev_ermod_emax_exp_sel(
    data = data_er_cont,
    var_resp = "response",
    var_exp_candidates = c("exposure", "exposure2"),
    emax_fix = 100,
    e0_fix = 0,
    verbosity_level = 0,
    chains = 2,
    iter = 200,
    seed = 1
  ))


# Test ermod_bin_emax ---------------------------------------------------------
ermod_bin_emax <-
  dev_ermod_bin_emax(
    data = data_er_bin,
    var_exposure = "conc",
    var_resp = "y",
    verbosity_level = 0,
    # Increase iter for better convergence as exact reproducibility
    # over different machines doesn't seem realistic
    chains = 2,
    iter = 1000,
    seed = 1
  )

ermod_bin_emax_w_cov <-
  suppressWarnings(dev_ermod_bin_emax(
    data = data_er_bin,
    var_exposure = "conc",
    var_resp = "y_cov",
    l_var_cov = list(emax = "sex"),
    verbosity_level = 0,
    chains = 2,
    iter = 200,
    seed = 1
  ))

ersim_bin_med_qi <- sim_er(ermod_bin_emax, output_type = "median_qi")

ermod_bin_emax_exp_sel <-
  suppressWarnings(dev_ermod_bin_emax_exp_sel(
    data = data_er_bin,
    var_resp = "y",
    var_exp_candidates = c("conc", "conc2"),
    verbosity_level = 0,
    chains = 2,
    iter = 200,
    seed = 1
  ))


# Convert factor as response variable into number
data("mtcars")
mtcars$am2 <- factor(mtcars$am, labels = c("Automatic", "Manual"))
mtcars$am3 <- as.numeric(mtcars$am2) - 1

mod_mtcars_emax <- suppressWarnings(dev_ermod_bin_emax(
  data = mtcars,
  var_resp = "am3",
  var_exposure = "cyl",
  verbosity_level = 0,
  # Below option to make the test fast
  chains = 2, iter = 1000
))

# Test ------------------------------------------------------------------------
test_that("sim_ermod", {
  expect_equal(dim(sim_er(ermod_emax_w_cov, n_draws_sim = 10)), c(600, 14))
  expect_equal(dim(ersim_med_qi), c(60, 17))

  par_names <- c("emax", "e0", "ec50", "gamma", "sigma")
  par_means <-
    rstan::summary(ermod_emax$mod$stanfit, pars = par_names)$summary[, 1]
  expect_equal(
    par_means,
    c(
      `emax[1]` = 92, `e0[1]` = 5.8, `ec50[1]` = 77,
      gamma = 1, sigma = 16.5
    ),
    tolerance = 0.1
  )
  expect_equal(
    unique(ersim_med_qi$.epred)[1:10],
    c(
      5.767171, 20.640204, 20.969320, 21.941925, 22.306456,
      23.911237, 24.403143, 24.444787, 26.956913, 27.852625
    ),
    tolerance = 0.1
  )
})

test_that("plot_ermod", {
  if (.if_run_ex_plot_er()) {
    plot(plot_er(ersim_med_qi)) |> expect_silent()
    plot(plot_er(ersim_med_qi, show_orig_data = TRUE)) |> expect_silent()
    plot_er(ermod_emax_w_cov) |>
      expect_error("Model has covariate\\(s\\), and you cannot use this")
    plot_er_exp_sel(ermod_emax_exp_sel) |> expect_silent()
    plot_er(ermod_emax,
      show_orig_data = TRUE,
      options_orig_data = list(var_group = "dose")
    ) |> expect_silent()
  }
})


test_that("sim_ermod_bin_emax", {
  expect_equal(
    dim(sim_er(ermod_bin_emax_w_cov, n_draws_sim = 10)),
    c(1010, 17)
  )
  expect_equal(dim(ersim_bin_med_qi), c(101, 23))

  par_names <- c("emax", "e0", "ec50", "gamma")
  stanfit <- ermod_bin_emax_w_cov$mod$stanfit
  par_means <-
    rstan::summary(stanfit, pars = par_names)$summary[, 1]
  expect_equal(
    par_means,
    c(
      `emax[1]` = 1.8, `emax[2]` = 1.25, `e0[1]` = -1, `ec50[1]` = 240,
      gamma = 1
    ),
    tolerance = 0.1
  )
  expect_equal(
    unique(ersim_med_qi$.epred)[1:10],
    c(
      6.153473, 20.922569, 21.249333, 22.248391, 22.616190,
      24.137570, 24.636718, 24.672612, 27.069008, 27.936741
    ),
    tolerance = 0.1
  )
})

test_that("plot_ermod_bin_emax", {
  if (.if_run_ex_plot_er()) {
    plot(plot_er(ersim_bin_med_qi)) |> expect_silent()
    plot(plot_er(ersim_bin_med_qi, show_orig_data = TRUE)) |> expect_silent()
    plot_er(ermod_bin_emax_w_cov) |>
      expect_error("Model has covariate\\(s\\), and you cannot use this")
    plot(plot_er_exp_sel(ermod_bin_emax_exp_sel)) |> expect_silent()
  }
})

test_that("Convert factor as response variable into number", {
  if (.if_run_ex_plot_er()) {
    plot_er_gof(mod_mtcars_emax, n_bins = 4) |>
      expect_error("The breaks for the binned probability ")
    plot_er_gof(mod_mtcars_emax, n_bins = 2) |> expect_silent()
  }
})

test_that("checking e0_fix and emax_fix in dev_ermod_bin_emax", {
  dev_ermod_bin_emax(
    e0_fix = 0,
    verbosity_level = -1 # To stop the function from running
  ) |>
    expect_warning(
      "e0_fix is set to 0"
    ) |>
    expect_error()

  dev_ermod_bin_emax(
    e0_fix = 1,
    emax_fix = 2,
    verbosity_level = -1 # To stop the function from running
  ) |>
    expect_message(
      "e0_fix is set at 1"
    ) |>
    expect_message(
      "emax_fix is set at 2"
    ) |>
    expect_error()
})

#' Build specifications for covariate effect simulation/visualization
#'
#' @name build_spec_coveff
#' @inheritParams sim_coveff
#' @param n_sigfig Number of significant figures to form value_label of
#' continuous variables. See [gt::vec_fmt_number()] for details.
#' @param use_seps Whether to use separators for thousands in printing numbers.
#' See [gt::vec_fmt_number()] for details.
#' @param drop_trailing_dec_mark Whether to drop the trailing decimal mark
#' (".") in value_label of continuous variables. See [gt::vec_fmt_number()]
#' for details.
#'
#' @examplesIf BayesERtools:::.if_run_ex_coveff()
#' \donttest{
#' data(d_sim_binom_cov_hgly2)
#'
#' ermod_bin <- dev_ermod_bin(
#'   data = d_sim_binom_cov_hgly2,
#'   var_resp = "AEFLAG",
#'   var_exposure = "AUCss_1000",
#'   var_cov = c("BHBA1C_5", "RACE"),
#' )
#'
#' spec_coveff <- build_spec_coveff(ermod_bin)
#' plot_coveff(ermod_bin, spec_coveff = spec_coveff)
#' }
#'
NULL

#' @export
#' @rdname build_spec_coveff
#' @return
#' `spec_coveff` (return object) is a data frame for the specification
#' of the covariate effects to be visualized. This is internally generated by
#' [build_spec_coveff()] if you run [sim_coveff()] or [plot_coveff()]
#' directly. Alternatively, you can develop your own or modify the one
#' generated by [build_spec_coveff()] and supply it to [sim_coveff()] or
#' [plot_coveff()]. The data frame should have the following columns (but
#' it's probably easier to try [build_spec_coveff()] and see the structure):
#'
#' - `var_order`: The order of the covariate in the forest plot. The exposure
#'   variable is always the first one and the covariates are ordered by the
#'   order they are supplied in the `var_cov` argument of the `dev_ermod_*`
#'   function. If you used a model from [dev_ermod_bin_cov_sel()], then the
#'   order is determined by the variable selection process.
#' - `var_name`: The name of the variable.
#' - `var_label`: The label of the variable to be used for plot.
#'    This is the same as `var_name` by default.
#' - `value_order`: The order of the value of the variable to be evaluated.
#' - `value_annot`: The annotation of the value of the variable to be
#'   evaluated. This appears on the right hand side of the forest plot.
#' - `value_label`: The label of the value of the variable to be evaluated.
#' - `value_cont`: The value for continuous variables.
#' - `value_cat`: The value for categorical variables.
#' - `is_ref_value`: Whether the value is the reference value.
#' - `show_ref_value`: Whether to show the reference value in the plot and
#'   table. This is TRUE by default for is_ref_value == TRUE, otherwise NA
#'   (and ignored).
#' - `is_covariate`: Whether the variable is a covariate (TRUE) or exposure
#'   variable (FALSE).
#'
build_spec_coveff <- function(
    ermod,
    data = NULL,
    qi_width_cov = 0.9,
    n_sigfig = 3,
    use_seps = TRUE,
    drop_trailing_dec_mark = TRUE) {
  stopifnot(inherits(ermod, "ermod"))
  if (is.null(data)) {
    data <- ermod$data
  }
  stopifnot(is.data.frame(data))

  var_exposure <- extract_var_exposure(ermod)
  var_cov <- extract_var_cov(ermod)

  check_data_columns(
    data = data,
    var_exposure = var_exposure,
    var_cov = var_cov
  )

  data <- data |>
    dplyr::select(
      dplyr::all_of(var_exposure),
      dplyr::all_of(var_cov)
    )

  spec_var_exp <-
    calc_summary_col(
      data[[var_exposure]],
      col_name = var_exposure, var_order = 1, qi_width_cov = qi_width_cov
    ) |>
    dplyr::mutate(is_covariate = FALSE)

  spec_var_cov <-
    purrr::pmap(list(
      dplyr::select(data, dplyr::all_of(var_cov)),
      var_cov, seq_along(var_cov) + 1
    ), function(.x, .y, .z) {
      calc_summary_col(.x, .y, .z,
        qi_width_cov = qi_width_cov,
        n_sigfig = n_sigfig, use_seps = use_seps,
        drop_trailing_dec_mark = drop_trailing_dec_mark
      )
    }) |>
    purrr::list_rbind() |>
    dplyr::mutate(is_covariate = TRUE)

  # Create dummy value_cat column when there were no categorical covariates
  if (!"value_cat" %in% colnames(spec_var_cov)) {
    spec_var_cov <- spec_var_cov |>
      dplyr::mutate(value_cat = NA_character_)
  }

  spec_coveff <-
    dplyr::bind_rows(spec_var_exp, spec_var_cov) |>
    dplyr::mutate(
      var_label = var_name, show_ref_value = ifelse(is_ref_value, TRUE, NA)
    ) |>
    dplyr::select(
      var_order, var_name, var_label, value_order, value_annot, value_label,
      value_cont, value_cat, is_ref_value, show_ref_value, is_covariate
    )

  return(spec_coveff)
}



calc_summary_col <- function(
    x, col_name, var_order, qi_width_cov = 0.9,
    n_sigfig = 3, use_seps = TRUE,
    drop_trailing_dec_mark = TRUE) {
  if (is.numeric(x)) {
    calc_summary_col_cont(x,
      qi_width_cov = qi_width_cov,
      n_sigfig = n_sigfig, use_seps = use_seps,
      drop_trailing_dec_mark = drop_trailing_dec_mark
    ) |>
      dplyr::mutate(var_name = col_name, var_order = var_order)
  } else {
    calc_summary_col_categ(x) |>
      dplyr::mutate(var_name = col_name, var_order = var_order)
  }
}

calc_summary_col_cont <- function(
    x, qi_width_cov = 0.9, n_sigfig = 3,
    use_seps = TRUE, drop_trailing_dec_mark = TRUE) {
  rlang::check_installed("gt")

  summary_col_cont <-
    dplyr::tibble(
      value_cont = c(
        as.numeric(stats::quantile(x, probs = 0.5 - qi_width_cov / 2)),
        stats::median(x),
        as.numeric(stats::quantile(x, probs = 0.5 + qi_width_cov / 2))
      ),
      value_order = 1:3,
      value_annot = get_val_annot_cont(qi_width_cov),
      is_ref_value = c(FALSE, TRUE, FALSE)
    ) |>
    dplyr::mutate(value_label = gt::vec_fmt_number(value_cont,
      n_sigfig = n_sigfig,
      use_seps = use_seps
    ))

  # For some reason drop_trailing_dec_mark is not working, so manually
  # remove trailing decimal mark
  if (drop_trailing_dec_mark) {
    summary_col_cont <-
      summary_col_cont |>
      dplyr::mutate(value_label = sub("\\.$", "", value_label))
  }
}

get_val_annot_cont <- function(qi_width_cov = 0.9) {
  c(
    paste0(
      as.character(round((0.5 - qi_width_cov / 2) * 100, digits = 2)), "th"
    ),
    "median",
    paste0(
      as.character(round((0.5 + qi_width_cov / 2) * 100, digits = 2)), "th"
    )
  )
}

calc_summary_col_categ <- function(x) {
  # Replicating forcats::fct_infreq
  x_fct_ord <- factor(x, levels = names(sort(table(x), decreasing = TRUE)))

  dplyr::tibble(value_cat = levels(x_fct_ord)) |>
    dplyr::mutate(
      value_order = dplyr::row_number(),
      value_annot = paste0(add_ordinal_suffix(value_order), " freq"),
      is_ref_value = value_order == 1
    ) |>
    dplyr::mutate(value_label = value_cat)
}

add_ordinal_suffix <- function(x) {
  paste0(
    x,
    c("th", "st", "nd", "rd", rep("th", 6))[1 + x %% 10 * !x %% 100 == 11]
  )
}



# TODO
# edit_spec_coveff_raw(var_name, values_vec) function to replace the spec
# edit_spec_coveff_summary(var_name, value_cont, value_cat, value_order,
# value_label (optional), value_annot (optional?), is_ref_value, is_covariate)
# function to replace the spec

#' Customize specifications for covariate effect simulations/visualizations
#'
#' @description
#' - [build_spec_coveff_one_variable()] is a helper function to create a new
#'   specification for a single variable. This is useful when you want to
#'   customize the specification for a single variable.
#' - [replace_spec_coveff()] is used to replace the specification for some (or
#'   all) variables in the original specification data frame. If you want to
#'   replace multiple variables, you can just stack the specifications
#'   together.
#'
#' @export
#' @name edit_spec_coveff
#' @inheritParams build_spec_coveff
#' @param var_name The name of the variable for which a new spec is to be
#' created.
#' @param values_vec The vector of the values for creating a new spec.
#' @param var_label The label of the variable to be used for plot. If NULL
#' (default), it is set to `var_name`.
#' @param show_ref_value Whether to show the reference value in the plot and
#' table. Setting this results in the `show_ref_value` column in the
#' specification data frame.
#' @return See [build_spec_coveff()] for the structure of the return object.
#' [build_spec_coveff_one_variable()] returns a data frame corresponding to
#' the specification for a single variable, which can be used as an input to
#' [replace_spec_coveff()].
#'
#' @examplesIf BayesERtools:::.if_run_ex_coveff()
#' \donttest{
#' set.seed(1234)
#' data(d_sim_binom_cov_hgly2)
#'
#' ermod_bin <- suppressWarnings(dev_ermod_bin(
#'   data = d_sim_binom_cov_hgly2, var_resp = "AEFLAG",
#'   var_exposure = "AUCss_1000", var_cov = c("BGLUC", "RACE"),
#'   verbosity_level = 0,
#'   # Below option to make the example run fast
#'   chains = 2, iter = 1000
#' ))
#'
#' spec_coveff <- build_spec_coveff(ermod_bin)
#' spec_new_bgluc <- build_spec_coveff_one_variable(
#'   "BGLUC", seq(4, 8, by = 0.1),
#'   var_label = "Baseline Glucose (mmol/L)",
#'   qi_width_cov = 0.8, show_ref_value = FALSE
#' )
#' spec_coveff_new <- replace_spec_coveff(spec_coveff, spec_new_bgluc)
#' plot_coveff(ermod_bin, spec_coveff = spec_coveff_new)
#' }
#'
build_spec_coveff_one_variable <- function(
    var_name, values_vec,
    var_label = NULL,
    qi_width_cov = 0.9, n_sigfig = 3, use_seps = TRUE,
    drop_trailing_dec_mark = TRUE, show_ref_value = TRUE) {
  stopifnot(is.vector(values_vec))

  spec_one_var <-
    calc_summary_col(values_vec, var_name,
      var_order = 1,
      qi_width_cov = qi_width_cov,
      n_sigfig = n_sigfig, use_seps = use_seps,
      drop_trailing_dec_mark = drop_trailing_dec_mark
    ) |>
    dplyr::mutate(
      var_label = var_name,
      show_ref_value = ifelse(is_ref_value, show_ref_value, NA)
    ) |>
    dplyr::select(
      var_name, var_label, value_order, value_annot, value_label,
      dplyr::any_of(c("value_cont", "value_cat")), is_ref_value, show_ref_value
    )

  if (!is.null(var_label)) spec_one_var$var_label <- var_label

  return(spec_one_var)
}

#' @export
#' @name edit_spec_coveff
#' @param spec_orig Original specification data frame.
#' @param spec_new New specification data frame. It can be generated by
#' [build_spec_coveff_one_variable()] or manually crafting with the
#' following variables: `var_name`, `var_label`, `value_order`, `value_annot`,
#' `value_label`, `value_cont` or `value_cat`, `is_ref_value`,
#' `show_ref_value`. You can have multiple variables stacked together.
#' @param replace_ref_value Whether to replace the reference values from the
#' original specification data frame. Default is FALSE; in this case,
#' show_ref_value is set to FALSE as it can be confusing.
#' If you set replace_ref_value to TRUE,
#' the reference calculation for the forest plot is also done with the one
#' in spec_new.
replace_spec_coveff <- function(
    spec_orig, spec_new,
    replace_ref_value = FALSE) {
  stopifnot(inherits(spec_orig, "data.frame"))
  stopifnot(inherits(spec_new, "data.frame"))

  # Check columns
  ## Check if value_cont and/or value_cat columns are present
  value_col_to_expect <- c()
  if ("value_cont" %in% colnames(spec_new)) {
    value_col_to_expect <- c(value_col_to_expect, "value_cont")
  }
  if ("value_cat" %in% colnames(spec_new)) {
    value_col_to_expect <- c(value_col_to_expect, "value_cat")
  }
  if (length(value_col_to_expect) == 0) {
    stop("At least one of value_cont or value_cat columns must be present.")
  }

  ## Make sure spec_new has the following columns
  col_names_expect_spec_new <-
    c(
      "var_name", "var_label", "value_order", "value_annot", "value_label",
      value_col_to_expect, "is_ref_value", "show_ref_value"
    )

  cond1 <- all(col_names_expect_spec_new %in% colnames(spec_new))
  cond2 <- all(colnames(spec_new) %in% col_names_expect_spec_new)
  if (!(cond1 && cond2)) {
    stop(
      "`spec_new` expected to have the following columns:\n",
      paste0(col_names_expect_spec_new, collapse = ", "),
      ".\nThe following columns are found:\n",
      paste0(colnames(spec_new), collapse = ", "),
      # Show the difference
      ".\nDifference: ",
      paste0(
        dplyr::symdiff(col_names_expect_spec_new, colnames(spec_new)),
        collapse = ", "
      )
    )
  }

  # Check var_name
  ## Make sure all var_name in spec_new exist in spec_orig
  if (any(!spec_new$var_name %in% spec_orig$var_name)) {
    ## Which var_name in spec_new does not exist in spec_orig?
    var_name_mismatch <-
      spec_new$var_name[!spec_new$var_name %in% spec_orig$var_name] |>
      unique()
    stop(
      "Following var_name in spec_new do not exist in spec_orig: ",
      paste0(var_name_mismatch, collapse = ", ")
    )
  }

  map_name_label <-
    spec_new |>
    dplyr::distinct(var_name, var_label)

  if (nrow(map_name_label) != length(unique(spec_new$var_name))) {
    stop("var_label should be the same for the same var_name.")
  }

  add_to_spec_new <- spec_orig |>
    dplyr::filter(var_name %in% spec_new$var_name) |>
    dplyr::select(var_order, var_name, is_covariate) |>
    dplyr::distinct()

  if (replace_ref_value) {
    spec_new_2 <-
      spec_new |>
      dplyr::left_join(add_to_spec_new, by = "var_name")

    spec_updated <-
      spec_orig |>
      dplyr::filter(!var_name %in% spec_new$var_name) |>
      dplyr::bind_rows(spec_new_2) |>
      dplyr::arrange(var_order, value_order)
  } else {
    spec_keep <- spec_orig |>
      dplyr::filter(!var_name %in% spec_new$var_name)

    spec_ref_orig <- spec_orig |>
      dplyr::filter(is_ref_value, var_name %in% spec_new$var_name) |>
      dplyr::select(
        var_name, value_order, value_annot, value_label,
        dplyr::any_of(c("value_cont", "value_cat")), is_ref_value,
        show_ref_value
      ) |>
      dplyr::left_join(map_name_label, by = "var_name") |>
      dplyr::mutate(
        show_ref_value = FALSE,
        value_annot = NA
      )

    spec_new_2 <-
      spec_new |>
      dplyr::arrange(value_order) |>
      dplyr::mutate(is_ref_value = FALSE, show_ref_value = NA)

    spec_updated <-
      dplyr::bind_rows(spec_ref_orig, spec_new_2) |>
      dplyr::mutate(value_order = dplyr::row_number(), .by = var_name) |>
      dplyr::left_join(add_to_spec_new, by = "var_name") |>
      dplyr::bind_rows(spec_keep) |>
      dplyr::arrange(var_order, value_order)
  }

  return(spec_updated)
}

if (.if_run_ex_coveff()) {
  data(d_sim_binom_cov_hgly2)

  set.seed(1234)

  ermod_bin <- suppressWarnings(dev_ermod_bin(
    data = d_sim_binom_cov_hgly2, var_resp = "AEFLAG",
    var_exposure = "AUCss_1000", var_cov = c("BGLUC", "RACE"),
    verbosity_level = 0,
    # Below option to make the example run fast
    chains = 2, iter = 1000
  ))


  spec_coveff <- build_spec_coveff(ermod_bin)

  spec_new_bgluc <- build_spec_coveff_one_variable(
    "BGLUC", seq(4, 8, by = 0.1),
    qi_width_cov = 0.8, show_ref_value = TRUE
  )
  spec_var_race <- build_spec_coveff_one_variable(
    "RACE", c("Black", "Black", "Asian", "White"),
    show_ref_value = TRUE
  )

  spec_coveff_new <- replace_spec_coveff(
    spec_coveff,
    dplyr::bind_rows(spec_var_race, spec_new_bgluc)
  )
  spec_coveff_no_keep_ref <-
    replace_spec_coveff(spec_coveff, spec_new_bgluc, replace_ref_value = TRUE)
  spec_coveff_no_keep_ref_race <-
    replace_spec_coveff(spec_coveff, spec_var_race, replace_ref_value = TRUE)

  coveffsim1 <- sim_coveff(ermod_bin, spec_coveff = spec_coveff_new)
  coveffsim2 <- sim_coveff(ermod_bin, spec_coveff = spec_coveff_no_keep_ref)
  coveffsim3 <-
    sim_coveff(ermod_bin, spec_coveff = spec_coveff_no_keep_ref_race)


  # Tests ---------------------------------------------------------------------
  test_that("input col & row error", {
    expect_error(replace_spec_coveff(
      spec_coveff, spec_new_bgluc |> dplyr::select(-value_cont)
    ), "At least one of value_cont or value_cat columns must be present.")

    expect_error(replace_spec_coveff(
      spec_coveff, spec_new_bgluc |> dplyr::mutate(test = "a")
    ), "`spec_new` expected to have the")
    expect_error(replace_spec_coveff(
      spec_coveff, spec_new_bgluc |> dplyr::select(-is_ref_value)
    ), "`spec_new` expected to have the")

    expect_error(replace_spec_coveff(
      spec_coveff, spec_new_bgluc |> dplyr::mutate(var_name = "TEST")
    ), "Following var_name in spec_new do not")
  })


  test_that("sim with modified spec", {
    expect_equal(coveffsim1$.odds_ratio,
      c(
        0.5362096, 1, 4.1778986, 1, 0.2176921, 0.9138167, 3.8359728, 1,
        1.0926579, 1.8544957, 1
      ),
      tolerance = 1e-6
    )
    expect_equal(coveffsim2$.odds_ratio,
      c(
        0.5362096, 1, 4.1778986, 0.2382229, 1, 4.1977486, 1, 1.8544957,
        1.0926579
      ),
      tolerance = 1e-6
    )
    expect_equal(coveffsim3$.odds_ratio,
      c(
        0.5362096, 1, 4.1778986, 0.2242438, 1, 3.8026690, 1, 1.7090927,
        0.9151996
      ),
      tolerance = 1e-6
    )
  })
}

#' Class constructors for simulations from exposure-response models
#'
#' @name ersim-class
#' @aliases ersim
#' @aliases ersim_marg
#' @aliases ersim_med_qi
#' @aliases ersim_marg_med_qi
#' @noRd
#' @param simdata A data frame with the simulated data
#' @param ermod An exposure-response model
#' @param nrow_cov_data The number of rows in the covariate data
#'
new_ersim <- function(simdata, ermod, nrow_cov_data) {
  stopifnot(is.data.frame(simdata))
  stopifnot(inherits(ermod, "ermod"))

  ersim <- simdata
  origdata <- extract_data(ermod)

  # Add attributes
  attr(ersim, "var_exposure") <- ermod$var_exposure
  attr(ersim, "var_resp") <- ermod$var_resp
  attr(ersim, "var_cov") <- ermod$var_cov
  attr(ersim, "origdata") <- origdata
  attr(ersim, "nrow_cov_data") <- nrow_cov_data
  attr(ersim, "coef_exp_draws") <- ermod$coef_exp_draws
  attr(ersim, "ermod_class") <- class(ermod)
  attr(ersim, "endpoint_type") <- ermod$endpoint_type

  # Add ersim class
  class(ersim) <- c("ersim", class(ersim))

  return(ersim)
}

#' @rdname ersim-class
#' @noRd
new_ersim_marg <- function(simdata, ermod, nrow_cov_data) {
  ersim <- new_ersim(simdata, ermod, nrow_cov_data)

  # Add ersim_marg class
  class(ersim) <- c("ersim_marg", class(ersim))

  return(ersim)
}

#' @rdname ersim-class
#' @noRd
new_ersim_med_qi <- function(simdata, ermod, nrow_cov_data, qi_width) {
  ersim_med_qi <- new_ersim(simdata, ermod, nrow_cov_data)

  # Add attributes
  attr(ersim_med_qi, "qi_width") <- qi_width

  # Replace ersim class with ersim_med_qi
  class(ersim_med_qi) <-
    c(
      "ersim_med_qi",
      setdiff(class(ersim_med_qi), "ersim")
    )

  return(ersim_med_qi)
}

#' @rdname ersim-class
#' @noRd
new_ersim_marg_med_qi <- function(simdata, ermod, nrow_cov_data, qi_width) {
  ersim_marg_med_qi <-
    new_ersim_med_qi(simdata, ermod, nrow_cov_data, qi_width)

  # Add ersim_marg class
  class(ersim_marg_med_qi) <-
    c("ersim_marg_med_qi", class(ersim_marg_med_qi))

  return(ersim_marg_med_qi)
}

#' Probability of Direction (pd)
#'
#' Compute the **Probability of Direction** (***pd***). Although differently
#' expressed, this index is fairly similar (*i.e.*, is strongly correlated) to
#' the frequentist **p-value**. See [bayestestR::p_direction()] and
#' `vignette("overview_of_vignettes", package = "bayestestR")` >
#' "Probability of Direction (pd)" page for details.
#' For converting **pd** to a frequentist **p-value**,
#' see [bayestestR::pd_to_p()].
#'
#' For the class `ermod_bin_*`, it only calculates the **pd** for
#' the exposure variable.
#'
#' @exportS3Method bayestestR::p_direction
#' @name p_direction
#' @param x An object of class \code{ermod_bin_*}
#' @param null The null hypothesis value. Default is 0.
#' @param as_num If `TRUE`, the output is converted to a numeric value.
#' @param as_p If `TRUE`, the p-direction (pd) values are converted to a
#' frequentist p-value using [bayestestR::pd_to_p()]. Only works when
#' `as_num = TRUE`.
#' @param direction What type of p-value is requested or provided with
#' as_p = TRUE. Can be `"two-sided"` (default, two tailed) or `"one-sided"`
#' (one tailed).
#' @param ... Additional arguments passed to [bayestestR::p_direction()].
#'
#' @return See [bayestestR::p_direction()] for details.
#' @examplesIf BayesERtools:::.if_run_ex_p_dir()
#' \donttest{
#' library(bayestestR)
#'
#' df_er_dr2 <-
#'   d_sim_binom_cov |>
#'   dplyr::filter(
#'     AETYPE == "dr2",
#'     ID %in% seq(1, 500, by = 5)
#'   ) |>
#'   dplyr::mutate(AUCss_1000 = AUCss / 1000, BHBA1C_5 = BHBA1C / 5)
#'
#' ermod_bin <- dev_ermod_bin(
#'   data = df_er_dr2,
#'   var_resp = "AEFLAG",
#'   var_exposure = "AUCss_1000",
#'   var_cov = "BHBA1C_5"
#' )
#'
#' p_direction(ermod_bin, as_num = TRUE, as_p = TRUE)
#' }
#'
p_direction.ermod_bin <- function(
    x,
    null = 0,
    as_p = FALSE,
    as_num = FALSE,
    direction = "two-sided",
    ...) {
  if (as_p && !as_num) {
    stop("as_p = TRUE only works when as_num = TRUE")
  }

  out_p_direction <-
    extract_mod(x) |>
    bayestestR::p_direction(
      parameters = extract_var_exposure(x),
      null = null,
      ...
    )

  if (!as_num) {
    return(out_p_direction)
  }

  out <- as.numeric(out_p_direction)

  if (as_p) out <- bayestestR::pd_to_p(out, direction = direction)

  return(out)
}

.if_run_ex_p_dir <- function() {
  requireNamespace("bayestestR", quietly = TRUE)
}

.posterior_draws <- function(fn, mod, newdata, n_draws, seed) {
  set.seed(seed)
  if (inherits(mod, c("stanemax", "stanemaxbin"))) {
    fn(mod, newdata = newdata, ndraws = n_draws)
  } else {
    fn(mod, newdata = newdata, draws = n_draws)
  }
}

.pp_matrix_to_draws_tbl <- function(mat, newdata, col_name) {
  n_draws <- nrow(mat)
  n_obs <- nrow(newdata)
  draw_rows <- expand.grid(
    .draw = seq_len(n_draws),
    .row = seq_len(n_obs)
  )
  newdata |>
    dplyr::mutate(.row = dplyr::row_number()) |>
    dplyr::left_join(
      dplyr::tibble(
        .row = draw_rows$.row,
        .chain = NA_integer_,
        .iteration = NA_integer_,
        .draw = draw_rows$.draw,
        !!col_name := as.vector(mat)
      ),
      by = ".row"
    )
}

#' Efficient approximate leave-one-out cross-validation (LOO)
#'
#' See [loo::loo()] for details.
#'
#' @name loo
#' @param x An object of class `ermod`
#' @param ... Additional arguments passed to `loo::loo()`
#' @importFrom loo loo
#'
#' @return An object of class `loo`
#' @export
loo::loo

#' @rdname loo
#' @export
loo.ermod <- function(x, ...) {
  loo::loo(x$mod, ...)
}

#' @rdname loo
#' @export
loo.ermod_emax <- function(x, ...) {
  rlang::check_installed("digest")

  out <- loo::loo(x$mod$stanfit, ...)

  y <- x$mod$standata$response
  attributes(y) <- NULL
  attr(out, "yhash") <- digest::sha1(y)

  return(out)
}

#' @rdname loo
#' @export
loo.ermod_bin_emax <- function(x, ...) {
  rlang::check_installed("digest")

  out <- loo::loo(x$mod$stanfit, ...)

  y <- x$mod$standata$response
  attributes(y) <- NULL
  attr(out, "yhash") <- digest::sha1(y)

  return(out)
}

#' Run k-fold cross-validation
#'
#' This function performs k-fold cross-validation using the appropriate model
#' development function based on the class of the `ermod` object.
#' It is internally used by [eval_ermod()]. The output is compatible with
#' `loo` ecosystem, e.g. it can be used for [loo::loo_compare()] function.
#' See [loo::kfold()] for details.
#'
#' @rdname kfold
#' @param x An `ermod` object containing the model and data.
#' @param k The number of folds for cross-validation. Default is 5.
#' @param newdata Optional new dataset to use instead of the original data.
#' Default is NULL.
#' @param seed Random seed for reproducibility. Default is NULL.
#' @param ... Currently not used.
#'
#' @return [kfold()] returns `kfold_ermod` class object containing the fitted
#' models and holdout predictions for each fold.
#'
#' @examplesIf BayesERtools:::.if_run_ex_eval_mod()
#' \donttest{
#' data(d_sim_binom_cov_hgly2)
#'
#' ermod_bin <- dev_ermod_bin(
#'   data = d_sim_binom_cov_hgly2,
#'   var_resp = "AEFLAG",
#'   var_exposure = "AUCss_1000",
#'   var_cov = "BHBA1C_5",
#'   # Settings to make the example run faster
#'   chains = 2,
#'   iter = 1000
#' )
#'
#' cv_results <- kfold(ermod_bin, k = 3, seed = 123)
#'
#' print(cv_results)
#' }
#'
#' @export
kfold.ermod <- function(x, k = 5, newdata = NULL, seed = NULL, ...) {
  rlang::check_installed("digest")

  ermod <- x

  # Set seed for reproducibility
  if (!is.null(seed)) {
    rng_state_old <- .Random.seed
    on.exit(assign(".Random.seed", rng_state_old, envir = .GlobalEnv))
    set.seed(seed)
  }

  # Use newdata if provided, otherwise use data from ermod object
  data <- if (!is.null(newdata)) newdata else ermod$data

  # Convert non-numeric columns to factors
  # Need to do this first, otherwise there can be discrepancies in categorical
  # levels between training and testing data
  data <- data |>
    dplyr::mutate(dplyr::across(dplyr::where(~ !is.numeric(.)), as.factor)) |>
    dplyr::mutate(.row_orig = dplyr::row_number())

  # Determine the model development function based on the class of the ermod
  if (inherits(ermod, "ermod_emax")) {
    model_dev_fun <- dev_ermod_emax
  } else if (inherits(ermod, "ermod_bin_emax")) {
    model_dev_fun <- dev_ermod_bin_emax
  } else if (inherits(ermod, "ermod_lin")) {
    model_dev_fun <- dev_ermod_lin
  } else if (inherits(ermod, "ermod_bin")) {
    model_dev_fun <- dev_ermod_bin
  } else {
    stop("Unsupported model class")
  }

  # Create k-fold cross-validation splits manually
  fold_ids <- sample(rep(1:k, length.out = nrow(data)))

  fit_and_sim <- function(fold_id) {
    train_data <- data[fold_ids != fold_id, ]
    test_data <- data[fold_ids == fold_id, ]

    dev_args <- c(
      list(
        data = train_data,
        var_resp = ermod$var_resp,
        var_exposure = ermod$var_exposure
      ),
      if (inherits(ermod, "ermod_lin") || inherits(ermod, "ermod_bin")) {
        list(var_cov = ermod$var_cov)
      } else {
        list(l_var_cov = ermod$l_var_cov)
      },
      ermod$input_args
    )

    ermod_k <- do.call(model_dev_fun, dev_args)
    ersim_k <- sim_er(ermod_k, newdata = test_data)

    d_truth_k <- test_data |>
      dplyr::select(.row = .row_orig, truth = !!rlang::sym(ermod$var_resp)) |>
      dplyr::mutate(fold_id = fold_id)

    d_sim_k <- ersim_k |>
      dplyr::ungroup() |>
      dplyr::select(.row = .row_orig, .draw, pred = .epred) |>
      dplyr::mutate(fold_id = fold_id)

    loglik_k <- rstanarm::log_lik(extract_mod(ermod_k), newdata = test_data)

    list(
      ermod = ermod_k,
      d_truth = d_truth_k,
      d_sim = d_sim_k,
      test_id = which(fold_ids == fold_id),
      loglik = loglik_k
    )
  }

  results <- purrr::map(1:k, fit_and_sim)
  results <- purrr::list_transpose(results)
  results$l_ermod <- results$ermod
  results$ermod <- NULL

  results$d_truth <- dplyr::bind_rows(results$d_truth) |>
    dplyr::arrange(.row) |>
    dplyr::group_by(fold_id)
  results$d_sim <- dplyr::bind_rows(results$d_sim) |>
    dplyr::arrange(.row) |>
    dplyr::group_by(fold_id, .row)

  results$k <- k

  loglik <- do.call(cbind, results$loglik)
  elpds_unord <- apply(loglik, 2, logmeanexp)
  order_test_id <- order(unlist(results$test_id))
  elpds <- elpds_unord[order_test_id]

  # for computing effective number of parameters
  loglik_orig <- rstanarm::log_lik(extract_mod(ermod))
  lpds <- apply(loglik_orig, 2, logmeanexp)
  ps <- lpds - elpds

  pointwise <- cbind(elpd_kfold = elpds, p_kfold = ps, kfoldic = -2 * elpds)
  est <- colSums(pointwise)
  se_est <- sqrt(nrow(data) * apply(pointwise, 2, stats::var))

  estimates <- cbind(Estimate = est, SE = se_est)
  rownames(estimates) <- colnames(pointwise)

  results$estimates <- estimates
  results$pointwise <- pointwise

  y <- extract_data(ermod)[[extract_var_resp(ermod)]]
  attributes(y) <- NULL
  attr(results, "yhash") <- digest::sha1(y)

  results$loglik <- NULL
  class(results) <- c("kfold_ermod", "kfold", "loo")
  return(results)
}

logmeanexp <- function(x) {
  xmax <- which.max(x)
  log1p(sum(exp(x[-xmax] - x[xmax]))) + x[xmax] - log(length(x))
}

#' @name kfold
#' @importFrom loo kfold
#' @export
loo::kfold

#' @export
print.kfold_ermod <- function(x, ...) {
  class(x) <- c("kfold", "loo") # Needed for print to work

  cli::cli({
    cli::cli_h1("k-fold Cross-Validation for ermod object")
    cli::cli_alert_info(paste(
      "Number of folds: ", x$k
    ))
    cli::cli_h2("Structure of the object")
    cli::cli_li("$l_ermod: list of ermod objects")
    cli::cli_li("$d_truth: data frame with true response values")
    cli::cli_li("$d_sim: data frame with holdout predictions")
    cli::cli_h2("elpd (used in `loo` package)")
    print(x) |>
      utils::capture.output() |>
      cli::cli_code()
  })
}


#' @export
#' @name kfold
#' @param kfold_ermod An object of class `kfold_ermod` from [kfold()]
#' @return [extract_kfold_loo()] returns  `c("kfold", "loo")` class object
#' that works well with `loo` ecosystem
extract_kfold_loo <- function(kfold_ermod) {
  class(kfold_ermod) <- c("kfold", "loo")
  return(kfold_ermod)
}

# ermod_lin -------------------------------------------------------------------

#' Class constructors for the binary exposure-response models
#'
#' @name ermod_lin-class
#' @noRd
#' @param mod A stanreg object
#' @param data Data frame used to fit the model
#' @param var_resp Name of the response variable
#' @param var_exposure Name of the exposure variable
#' @param var_cov Name of the covariate variable
new_ermod_lin <- function(
    mod,
    data,
    var_resp = character(),
    var_exposure = character(),
    var_cov = NULL,
    input_args = list()) {
  coef_exp_draws <- .get_coef_exp_draws(mod, var_exposure)

  check_input_new_ermod(
    mod = mod, data = data, var_resp = var_resp,
    var_exposure = var_exposure, var_cov = var_cov,
    input_args = input_args, coef_exp_draws = coef_exp_draws,
    basemodclass = "stanreg"
  )

  structure(
    list(
      mod = mod,
      data = data,
      var_resp = var_resp,
      var_exposure = var_exposure,
      var_cov = var_cov,
      input_args = input_args,
      coef_exp_draws = coef_exp_draws,
      endpoint_type = "continuous"
    ),
    class = c("ermod_lin", "ermod")
  )
}


#' @rdname ermod_lin-class
#' @noRd
new_ermod_lin_exp_sel <- function(l_ermod_exp_sel) {
  check_l_ermod_exp_sel(l_ermod_exp_sel)

  coef_exp_draws <-
    .get_coef_exp_draws(l_ermod_exp_sel$mod, l_ermod_exp_sel$var_exposure)

  l_ermod_exp_sel$coef_exp_draws <- coef_exp_draws
  l_ermod_exp_sel$endpoint_type <- "continuous"

  structure(
    l_ermod_exp_sel,
    class = c("ermod_lin_exp_sel", "ermod_exp_sel", "ermod_lin", "ermod")
  )
}


#' @rdname ermod_lin-class
#' @noRd
new_ermod_lin_cov_sel <- function(
    mod,
    data,
    var_resp = character(),
    var_exposure = character(),
    var_cov_candidates = character(),
    var_cov = NULL,
    var_selected = character(),
    cv_method = c("LOO", "kfold"),
    cvvs = NULL,
    rk = NULL,
    input_args = list()) {
  cv_method <- match.arg(cv_method)
  coef_exp_draws <- .get_coef_exp_draws(mod, var_exposure)

  check_input_new_ermod(
    mod = mod, data = data, var_resp = var_resp, var_exposure = var_exposure,
    var_cov_candidates = var_cov_candidates, var_cov = var_cov,
    var_selected = var_selected, cvvs = cvvs, rk = rk,
    input_args = input_args, coef_exp_draws = coef_exp_draws,
    basemodclass = "stanreg"
  )

  structure(
    list(
      mod = mod,
      data = data,
      var_resp = var_resp,
      var_exposure = var_exposure,
      var_cov_candidates = var_cov_candidates,
      var_cov = var_cov,
      var_selected = var_selected,
      cv_method = cv_method,
      cvvs = cvvs,
      rk = rk,
      input_args = input_args,
      coef_exp_draws = coef_exp_draws,
      endpoint_type = "continuous"
    ),
    class = c("ermod_lin_cov_sel", "ermod_cov_sel", "ermod_lin", "ermod")
  )
}

# ermod_bin -------------------------------------------------------------------

#' Class constructors for the binary exposure-response models
#'
#' @name ermod_bin-class
#' @noRd
#' @param mod A stanreg object
#' @param data Data frame used to fit the model
#' @param var_resp Name of the response variable
#' @param var_exposure Name of the exposure variable
#' @param var_cov Name of the covariate variable
new_ermod_bin <- function(
    mod,
    data,
    var_resp = character(),
    var_exposure = character(),
    var_cov = NULL,
    input_args = list()) {
  coef_exp_draws <- .get_coef_exp_draws(mod, var_exposure)

  check_input_new_ermod(
    mod = mod, data = data, var_resp = var_resp,
    input_args = input_args, coef_exp_draws = coef_exp_draws,
    basemodclass = "stanreg"
  )

  structure(
    list(
      mod = mod,
      data = data,
      var_resp = var_resp,
      var_exposure = var_exposure,
      var_cov = var_cov,
      input_args = input_args,
      coef_exp_draws = coef_exp_draws,
      endpoint_type = "binary"
    ),
    class = c("ermod_bin", "ermod")
  )
}


#' @rdname ermod_bin-class
#' @noRd
new_ermod_bin_exp_sel <- function(l_ermod_exp_sel) {
  check_l_ermod_exp_sel(l_ermod_exp_sel)

  coef_exp_draws <-
    .get_coef_exp_draws(l_ermod_exp_sel$mod, l_ermod_exp_sel$var_exposure)

  l_ermod_exp_sel$coef_exp_draws <- coef_exp_draws
  l_ermod_exp_sel$endpoint_type <- "binary"

  structure(
    l_ermod_exp_sel,
    class = c("ermod_bin_exp_sel", "ermod_exp_sel", "ermod_bin", "ermod")
  )
}


#' @rdname ermod_bin-class
#' @noRd
new_ermod_bin_cov_sel <- function(
    mod,
    data,
    var_resp = character(),
    var_exposure = character(),
    var_cov_candidates = character(),
    var_cov = NULL,
    var_selected = character(),
    cv_method = c("LOO", "kfold"),
    cvvs = NULL,
    rk = NULL,
    input_args = list()) {
  cv_method <- match.arg(cv_method)

  coef_exp_draws <- .get_coef_exp_draws(mod, var_exposure)

  check_input_new_ermod(
    mod = mod, data = data, var_resp = var_resp, var_exposure = var_exposure,
    var_cov_candidates = var_cov_candidates, var_cov = var_cov,
    var_selected = var_selected, cvvs = cvvs, rk = rk,
    input_args = input_args, coef_exp_draws = coef_exp_draws,
    basemodclass = "stanreg"
  )

  structure(
    list(
      mod = mod,
      data = data,
      var_resp = var_resp,
      var_exposure = var_exposure,
      var_cov_candidates = var_cov_candidates,
      var_cov = var_cov,
      var_selected = var_selected,
      cv_method = cv_method,
      cvvs = cvvs,
      rk = rk,
      input_args = input_args,
      coef_exp_draws = coef_exp_draws,
      endpoint_type = "binary"
    ),
    class = c("ermod_bin_cov_sel", "ermod_cov_sel", "ermod_bin", "ermod")
  )
}

# ermod_emax ------------------------------------------------------------------

#' Class constructors for the Emax models
#'
#' @name ermod_emax-class
#' @noRd
#' @param mod A stanreg object
#' @param data Data frame used to fit the model
#' @param var_resp Name of the response variable
#' @param var_exposure Name of the exposure variable
#' @param l_var_cov List of the covariate variables
new_ermod_emax <- function(
    mod,
    data,
    var_resp = character(),
    var_exposure = character(),
    l_var_cov = NULL,
    input_args = list()) {
  check_input_new_ermod(
    mod = mod, data = data, var_resp = var_resp, var_exposure = var_exposure,
    l_var_cov = l_var_cov, basemodclass = "stanemax"
  )

  structure(
    list(
      mod = mod,
      data = data,
      var_resp = var_resp,
      var_exposure = var_exposure,
      l_var_cov = l_var_cov,
      input_args = input_args,
      endpoint_type = "continuous"
    ),
    class = c("ermod_emax", "ermod")
  )
}


#' @rdname ermod_emax-class
#' @noRd
new_ermod_emax_exp_sel <- function(l_ermod_exp_sel) {
  check_l_ermod_exp_sel(l_ermod_exp_sel, basemodclass = "stanemax")

  l_ermod_exp_sel$endpoint_type <- "continuous"

  structure(
    l_ermod_exp_sel,
    class = c("ermod_emax_exp_sel", "ermod_exp_sel", "ermod_emax", "ermod")
  )
}

# ermod_bin_emax --------------------------------------------------------------

#' Class constructors for the binary Emax models
#'
#' @name ermod_bin_emax-class
#' @noRd
#' @param mod A stanreg object
#' @param data Data frame used to fit the model
#' @param var_resp Name of the response variable
#' @param var_exposure Name of the exposure variable
#' @param l_var_cov List of the covariate variables
new_ermod_bin_emax <- function(
    mod,
    data,
    var_resp = character(),
    var_exposure = character(),
    l_var_cov = NULL,
    input_args = list()) {
  check_input_new_ermod(
    mod = mod, data = data, var_resp = var_resp, var_exposure = var_exposure,
    l_var_cov = l_var_cov, basemodclass = "stanemaxbin"
  )

  structure(
    list(
      mod = mod,
      data = data,
      var_resp = var_resp,
      var_exposure = var_exposure,
      l_var_cov = l_var_cov,
      input_args = input_args,
      endpoint_type = "binary"
    ),
    class = c("ermod_bin_emax", "ermod")
  )
}


#' @rdname ermod_bin_emax-class
#' @noRd
new_ermod_bin_emax_exp_sel <- function(l_ermod_exp_sel) {
  check_l_ermod_exp_sel(l_ermod_exp_sel, basemodclass = "stanemaxbin")

  l_ermod_exp_sel$endpoint_type <- "binary"

  structure(
    l_ermod_exp_sel,
    class = c(
      "ermod_bin_emax_exp_sel", "ermod_exp_sel",
      "ermod_bin_emax", "ermod"
    )
  )
}

# utils -----------------------------------------------------------------------

check_input_new_ermod <- function(
    mod,
    data,
    var_resp = character(),
    var_exposure = character(),
    var_exp_candidates = character(),
    var_cov_candidates = character(),
    var_cov = character(),
    l_var_cov = NULL,
    var_selected = character(),
    l_mod_exposures = NULL,
    cvvs = NULL,
    rk = NULL,
    input_args = list(),
    coef_exp_draws = NULL,
    basemodclass = "stanreg") {
  stopifnot(inherits(mod, basemodclass))
  stopifnot(is.data.frame(data))
  stopifnot(is.character(var_resp))
  stopifnot(is.character(var_exposure))
  stopifnot(is.null(var_exp_candidates) || is.character(var_exp_candidates))
  stopifnot(is.null(var_cov) || is.character(var_cov))
  stopifnot(is.null(l_var_cov) || is.list(l_var_cov))
  stopifnot(is.character(var_cov_candidates))
  stopifnot(is.character(var_selected))
  stopifnot(is.null(cvvs) || inherits(cvvs, "vsel"))
  stopifnot(is.null(rk) || inherits(rk, "ranking"))
  stopifnot(is.list(input_args))
  stopifnot(is.null(coef_exp_draws) || inherits(coef_exp_draws, "numeric"))
  stopifnot(is.null(l_mod_exposures) || inherits(l_mod_exposures, "list"))
}


check_l_ermod_exp_sel <- function(l_ermod_exp_sel, basemodclass = "stanreg") {
  ll <- l_ermod_exp_sel

  check_input_new_ermod(
    mod = ll$mod, data = ll$data, var_resp = ll$var_resp,
    var_exp_candidates = ll$var_exp_candidates, var_exposure = ll$var_exposure,
    l_mod_exposures = ll$l_mod_exposures, basemodclass = basemodclass
  )
}

.get_coef_exp_draws <- function(mod, var_exposure) {
  posterior::as_draws_df(mod)[[var_exposure]]
}

# `ermod_bin*` S3 methods for print, plot, ... --------------------------------
## Regular `ermod` class ------------------------------------------------------

#' S3 methods for the classes \code{ermod_*}
#' @name ermod_method
#' @param x An object of class \code{ermod_*}
#' @param object An object of class \code{ermod_*}
#' @param digits Number of digits to print
#' @param ... Additional arguments passed to functions
#' @return
#' - `print()` and `plot()`: No return value, called for side effects
#' - `coef()`: Coefficients of the model
#' - `summary()`: Summary of the model
NULL


get_mod_type_name <- function(mod) {
  if (inherits(mod, "ermod_bin")) {
    return("Binary ER model")
  } else if (inherits(mod, "ermod_lin")) {
    return("Linear ER model")
  } else if (inherits(mod, "ermod_emax")) {
    return("Emax model")
  } else if (inherits(mod, "ermod_bin_emax")) {
    return("Binary Emax model")
  } else {
    stop("Unknown model type")
  }
}

#' @export
#' @rdname ermod_method
print.ermod <- function(x, digits = 2, ...) {
  mod_type_name <- get_mod_type_name(x)

  cli::cli({
    cli::cli_h1(mod_type_name)
    cli::cli_alert_info(paste(
      "Use `plot_er()` to visualize ER curve"
    ))
    cli::cli_h2("Developed model")
    print(x$mod, digits = digits, ...) |>
      utils::capture.output() |>
      cli::cli_code()
  })
}

#' @export
#' @rdname ermod_method
#' @inheritParams plot_er
plot.ermod_bin <- function(x, show_orig_data = FALSE, ...) {
  plot_er(x, show_orig_data = show_orig_data)
}

#' @export
#' @rdname ermod_method
coef.ermod <- function(object, ...) {
  if (!inherits(object, c("ermod_bin", "ermod_lin"))) {
    stop("coef() only supported for linear models")
  }

  stats::coef(object$mod, ...)
}

#' @export
#' @rdname ermod_method
summary.ermod <- function(object, ...) {
  if (!inherits(object, c("ermod_bin", "ermod_lin"))) {
    stop("summary() only supported for linear models")
  }

  summary(object$mod, ...)
}

## ermod_exp_sel class ----------------------------------------------------

#' S3 methods for the classes `ermod_exp_sel`
#'
#' @export
#' @name ermod_exp_sel_method
#' @inheritParams ermod_method
#' @param x An object of class `ermod_bin_exp_sel`
#' @return No return value, called for print or plot side effects
print.ermod_exp_sel <- function(x, digits = 2, ...) {
  mod_type_name <- get_mod_type_name(x)

  cli::cli({
    cli::cli_h1(paste(mod_type_name, "& exposure metric selection"))
    cli::cli_alert_info(paste(
      "Use `plot_er_exp_sel()` for ER curve of all exposure metrics"
    ))
    cli::cli_alert_info(paste(
      "Use `plot_er()` with `show_orig_data = TRUE` for ER curve of the",
      "selected exposure metric"
    ))

    if (length(x$var_exp_candidates) > 1) {
      cli::cli_h2("Exposure metrics comparison")
      print(x$loo_comp_exposures, digits = digits, ...) |>
        utils::capture.output() |>
        cli::cli_code()
    } else {
      cli::cli_alert_info(paste0(
        "Only `",
        x$var_exp_candidates,
        "` was provided, no selection done"
      ))
    }

    cli::cli_h2("Selected model")
    print(x$mod, digits = digits, ...) |>
      utils::capture.output() |>
      cli::cli_code()
  })
}

#' @export
#' @rdname ermod_exp_sel_method
plot.ermod_exp_sel <- function(x, ...) {
  plot_er_exp_sel(x, ...)
}

## ermod_cov_sel class ----------------------------------------------------

#' S3 methods for the classes `ermod_bin_cov_sel`
#' @export
#' @name ermod_cov_sel_method
#' @inheritParams ermod_method
#' @param x An object of class `ermod_bin_cov_sel`
#' @return No return value, called for print or plot side effects
print.ermod_cov_sel <- function(x, digits = 2, ...) {
  mod_type_name <- get_mod_type_name(x)

  cli::cli({
    cli::cli_h1(paste(mod_type_name, "& covariate selection"))

    cli::cli_alert_info(paste(
      "Use `plot_submod_performance()` to see variable",
      "selection performance\n"
    ))
    if (!is.null(x$rk$foldwise)) {
      cli::cli_alert_info(paste(
        "Use `plot_var_ranking()` to see variable ranking"
      ))
    }
    cli::cli_alert_info(paste(
      "Use `plot_er()` with `marginal = TRUE` to visualize",
      "marginal ER curve"
    ))

    cli::cli_h2("Selected model")
    print(x$mod, digits = digits, ...) |>
      utils::capture.output() |>
      cli::cli_code()
  })
}

#' @export
#' @rdname ermod_cov_sel_method
plot.ermod_cov_sel <- function(x, ...) {
  plot_submod_performance(x, ...)
}


# Extract elements ------------------------------------------------------------

#' Extract elements from an object of class \code{ermod_*}
#'
#' @export
#' @name extract_ermod
#' @keywords internal
#' @inherit extract_method return
#' @param x An object of class \code{ermod_*}
#'
extract_data.ermod <- function(x) x$data

#' @export
#' @rdname extract_ermod
extract_mod.ermod <- function(x) x$mod

#' @export
#' @rdname extract_ermod
extract_var_resp.ermod <- function(x) x$var_resp

#' @export
#' @rdname extract_ermod
extract_var_exposure.ermod <- function(x) x$var_exposure

#' @export
#' @rdname extract_ermod
extract_var_cov.ermod <- function(x) {
  if (inherits(x, c("ermod_bin", "ermod_lin"))) {
    return(x$var_cov)
  } else if (inherits(x, c("ermod_emax", "ermod_bin_emax"))) {
    return(x$l_var_cov |> unlist())
  } else {
    stop(
      "extract_var_cov() only supported for `ermod_bin`, `ermod_lin`",
      "`ermod_emax`, and `ermod_bin_emax`, and their subclasses"
    )
  }
}

#' @export
#' @rdname extract_ermod
extract_exp_sel_list_model.ermod_exp_sel <- function(x) x$l_mod_exposures

#' @export
#' @rdname extract_ermod
extract_exp_sel_comp.ermod_exp_sel <- function(x) x$loo_comp_exposures

#' @export
#' @rdname extract_ermod
extract_var_selected.ermod_cov_sel <- function(x) x$var_selected


#' Extract credible interval of the exposure coefficient
#'
#' @export
#' @param x An object of class `ermod_bin` or `ermod_lin`
#' @param ci_width Width of the credible interval
#' @param exp_candidates Logical, whether to extract the credible interval for
#' all exposure candidates. Default is `FALSE`. Only supported for models with
#' exposure selection, created with [dev_ermod_bin_exp_sel()] or
#' [dev_ermod_lin_exp_sel()] functions.
#' @return A named vector of length 2 with the lower and upper bounds of the
#'  credible interval (.lower, .upper). If `exp_candidates = TRUE`, a matrix
#'  with the same structure is returned, with each row corresponding to an
#'  exposure candidate.
#'
extract_coef_exp_ci <- function(
    x, ci_width = 0.95, exp_candidates = FALSE) {
  # Check that input x is linear ermod object
  if (!inherits(x, c("ermod_bin", "ermod_lin"))) {
    stop("extract_coef_exp_ci() only supported for linear models")
  }
  if (exp_candidates && !inherits(x, "ermod_exp_sel")) {
    stop(
      "exp_candidates = TRUE only supported for",
      " models with exposure selection"
    )
  }

  if (exp_candidates) {
    coef_exp_ci <- lapply(
      x$l_mod_exposures,
      function(mod) {
        stats::quantile(
          mod$coef_exp_draws,
          c(0.5 - ci_width / 2, 0.5 + ci_width / 2)
        ) |>
          stats::setNames(c(".lower", ".upper"))
      }
    )

    coef_exp_ci <- do.call(rbind, coef_exp_ci)
  } else {
    coef_exp_ci <-
      stats::quantile(
        x$coef_exp_draws,
        c(0.5 - ci_width / 2, 0.5 + ci_width / 2)
      )

    names(coef_exp_ci) <- c(".lower", ".upper")
  }

  return(coef_exp_ci)
}

# as_draws --------------------------------------------------------------------

#' Transform to `draws` objects
#'
#' See [posterior::as_draws()] for details.
#'
#' @importFrom posterior as_draws as_draws_list as_draws_array as_draws_df
#' as_draws_matrix as_draws_rvars
#' @param x An object of class `ermod`
#' @param ... Arguments passed to individual methods (if applicable).
#' @return A draws object from the `posterior` package.
#' @name as_draws
NULL

#' @rdname as_draws
#' @importFrom posterior as_draws
#' @export
posterior::as_draws

#' @rdname as_draws
#' @importFrom posterior as_draws_list
#' @export
posterior::as_draws_list

#' @rdname as_draws
#' @importFrom posterior as_draws_array
#' @export
posterior::as_draws_array

#' @rdname as_draws
#' @importFrom posterior as_draws_df
#' @export
posterior::as_draws_df

#' @rdname as_draws
#' @importFrom posterior as_draws_matrix
#' @export
posterior::as_draws_matrix

#' @rdname as_draws
#' @importFrom posterior as_draws_rvars
#' @export
posterior::as_draws_rvars

#' @rdname as_draws
#' @export
as_draws.ermod <- function(x, ...) {
  posterior::as_draws(x$mod, ...)
}

#' @rdname as_draws
#' @export
as_draws_list.ermod <- function(x, ...) {
  posterior::as_draws_list(x$mod, ...)
}

#' @rdname as_draws
#' @export
as_draws_array.ermod <- function(x, ...) {
  posterior::as_draws_array(x$mod, ...)
}

#' @rdname as_draws
#' @export
as_draws_df.ermod <- function(x, ...) {
  posterior::as_draws_df(x$mod, ...)
}

#' @rdname as_draws
#' @export
as_draws_matrix.ermod <- function(x, ...) {
  posterior::as_draws_matrix(x$mod, ...)
}

#' @rdname as_draws
#' @export
as_draws_rvars.ermod <- function(x, ...) {
  posterior::as_draws_rvars(x$mod, ...)
}

# prior_summary --------------------------------------------------------------


#' @name prior_summary
#' @importFrom rstanarm prior_summary
#' @export
rstanarm::prior_summary

#' Summarize the priors used for linear or linear logistic regression models
#'
#' See [rstanarm::prior_summary()] for details.
#'
#' @export
#' @rdname prior_summary
#' @param object An object of class `ermod`
#' @param ... Additional arguments passed to `rstanarm::prior_summary()`
#' @return An object of class `prior_summary.stanreg`
#'
prior_summary.ermod <- function(object, ...) {
  # Check that input x is linear ermod object
  if (!inherits(object, c("ermod_bin", "ermod_lin"))) {
    stop("prior_summary.ermod() only supported for linear models")
  }

  rstanarm::prior_summary(object$mod, ...)
}

d_sim_binom_cov <- d_sim_binom_cov

id_to_sample <- seq(1, max(d_sim_binom_cov$ID), by = 5)

df_er_ae_covsel_test <-
  d_sim_binom_cov |>
  dplyr::filter(
    AETYPE == "ae_covsel_test",
    ID %in% id_to_sample
  ) |>
  dplyr::mutate(AUCss_1000 = AUCss / 1000, BHBA1C_5 = BHBA1C / 5)

data_er_cont <- rstanemax::exposure.response.sample
data_er_cont_cov <- rstanemax::exposure.response.sample.with.cov
data_er_bin <- rstanemax::exposure.response.sample.binary

# Create  ermod_bin objects for testing ---------------------------------------
set.seed(1234)
ermod_bin <- dev_ermod_bin(
  data = df_er_ae_covsel_test,
  var_resp = "AEFLAG",
  var_exposure = "AUCss_1000",
  var_cov = "BHBA1C_5",
  verbosity_level = 0,
  # Below option to make the test fast
  chains = 2, iter = 1000
)

ermod_bin_wo_cov <- suppressWarnings(dev_ermod_bin(
  data = df_er_ae_covsel_test,
  var_resp = "AEFLAG",
  var_exposure = "AUCss_1000",
  verbosity_level = 0,
  # Below option to make the test fast
  chains = 2, iter = 50
))

set.seed(1234)
ermod_emax_w_cov <-
  suppressWarnings(dev_ermod_emax(
    data = data_er_cont_cov,
    var_exposure = "conc",
    var_resp = "resp",
    l_var_cov = list(emax = "cov2", ec50 = "cov3", e0 = "cov1"),
    verbosity_level = 0,
    chains = 2,
    iter = 50,
    seed = 1
  ))

ermod_bin_emax <-
  suppressWarnings(dev_ermod_bin_emax(
    data = df_er_ae_covsel_test,
    var_resp = "AEFLAG",
    var_exposure = "AUCss_1000",
    verbosity_level = 0,
    # Increase iter for better convergence as exact reproducibility
    # over different machines doesn't seem realistic
    chains = 2,
    iter = 50,
    seed = 1
  ))


# Test ------------------------------------------------------------------------
test_that("as_draws", {
  as_draws_df(ermod_bin)$AUCss_1000[1:10] |>
    expect_equal(
      c(
        0.5200371, 0.6083116, 0.7032639, 0.8712742, 0.4418383,
        0.9465231, 1.0348286, 0.5353301, 0.5980297, 0.5244656
      ),
      tolerance = 0.001
    )
  as_draws_df(ermod_emax_w_cov) |>
    names() |>
    expect_equal(c(
      "ec50[C1]", "ec50[C0]", "sigma", "gamma", "e0[A0]", "e0[A1]",
      "emax[B0]", "emax[B2]", "emax[B3]", ".chain", ".iteration", ".draw"
    ))
  as_draws_rvars(ermod_bin_emax) |> expect_s3_class("draws_rvars")
})


# extract_coef_exp_ci
test_that("extract_coef_exp_ci", {
  extract_coef_exp_ci(ermod_bin) |>
    expect_equal(
      c(.lower = 0.1717, .upper = 1.01064),
      tolerance = 0.001
    )
  expect_error(extract_coef_exp_ci(ermod_emax_w_cov), "extract_coef_exp_ci")
})


test_that("print.ermod prints correct information", {
  out <- cli::cli_fmt({
    print(ermod_bin)
  })
  expect_true(any(grepl("Binary ER model", out)))
  expect_true(any(grepl("Use \\`plot_er\\(\\)\\` to visualize ER curve", out)))
  expect_true(any(grepl("Developed model", out)))
})

# plot.ermod_bin
test_that("plot.ermod_bin calls plot_er", {
  if (.if_run_ex_plot_er()) {
    expect_silent(plot.ermod_bin(ermod_bin_wo_cov))
  }
})

# Additional extract function tests
test_that("extract functions", {
  expect_equal(extract_data.ermod(ermod_bin), ermod_bin$data)
  expect_equal(extract_mod.ermod(ermod_bin), ermod_bin$mod)
  expect_equal(extract_var_resp.ermod(ermod_bin), ermod_bin$var_resp)
  expect_equal(extract_var_exposure.ermod(ermod_bin), ermod_bin$var_exposure)
  expect_equal(extract_var_cov.ermod(ermod_bin), ermod_bin$var_cov)
  expect_equal(
    extract_var_cov.ermod(ermod_emax_w_cov),
    unlist(ermod_emax_w_cov$l_var_cov)
  )
  expect_equal(
    extract_exp_sel_list_model.ermod_exp_sel(ermod_bin_emax),
    ermod_bin_emax$l_mod_exposures
  )
  expect_equal(
    extract_exp_sel_comp.ermod_exp_sel(ermod_bin_emax),
    ermod_bin_emax$loo_comp_exposures
  )
  expect_equal(
    extract_var_selected.ermod_cov_sel(ermod_bin_emax),
    ermod_bin_emax$var_selected
  )
})

# get_mod_type_name
test_that("get_mod_type_name", {
  expect_equal(get_mod_type_name(ermod_emax_w_cov), "Emax model")
  expect_equal(get_mod_type_name(ermod_bin_emax), "Binary Emax model")
  expect_error(get_mod_type_name("1"), "Unknown model type")
})

# Expect errors for nonlinear models
test_that("only supported for linear models", {
  expect_error(coef(ermod_emax_w_cov), "coef")
  expect_error(summary(ermod_emax_w_cov), "summary")
  expect_error(prior_summary(ermod_emax_w_cov), "prior_summary.ermod")
})

#' Plot ER model simulations
#'
#' @export
#' @param x an object of class `ermod`, `ersim`,`ersim_med_qi`,
#' or their subclasses
#' @param ... currently not used
#' @param show_orig_data logical, whether to show the data points in the
#' model development dataset. Default is `FALSE`. Only support plotting
#' with data that was used in the model development. If you want to use
#' other data, consider adding geom_point() to the plot manually.
#' @param show_coef_exp logical, whether to show the credible interval
#' of the exposure coefficient. Default is `FALSE`. This is only available
#' for linear and linear logistic regression models.
#' @param show_caption logical, whether to show the caption note for the plot.
#' Default is `FALSE`.
#' @param options_orig_data List of options for configuring how original data
#' is displayed. Possible options include:
#'   - `add_boxplot`: Logical, whether to add a boxplot of exposure values.
#'     Default is `FALSE`.
#'   - `boxplot_height`: Height of the boxplot relative to the main plot.
#'     Default is `0.15`.
#'   - `show_boxplot_y_title`: Logical, whether to show the y-axis title
#'     for the boxplot. Default is `TRUE`.
#'   - `var_group`: The column to use for grouping data for plotting.
#'     If specified, observed data points and boxplot will be grouped
#'     and colored by this column. Default is `NULL`.
#'   - `n_bins`: Number of bins to use for observed probability
#'     summary. Only relevant for binary models. Default is `4`.
#'   - `qi_width`: Width of the quantile interval (confidence interval) for
#'     the observed probability summary. Only relevant for binary models.
#'     Default is `0.95`.
#'   - `return_components`: Logical, whether to return plot components as a
#'     list instead of a combined plot. When `TRUE`, returns a list with
#'     `$main` (main plot), `$boxplot` (boxplot if applicable), `$caption`
#'     (caption text if applicable), and `$metadata`. Default is `FALSE`.
#' @param options_coef_exp List of options for configuring how the exposure
#' coefficient credible interval is displayed. Possible options include:
#'  - `qi_width`: Width of the quantile interval (credible interval) for
#'    the exposure coefficient. Default is `0.95`.
#'  - `n_sigfig`: Number of significant figures to display. Default is `3`.
#'  - `pos_x`: x-coordinate of the text label. If `NULL` (default), it is
#'    set to the minimum value for the exposure variable.
#'  - `pos_y`: y-coordinate of the text label. If `NULL` (default), it is
#'    set to 0.9 for logistic regression models and the maximum value of the
#'    response variable in the original data for linear regression models.
#'  - `size`: Size of the text label. Default is `4`.
#' @param options_caption List of options for configuring the caption note.
#' Possible options include:
#'   - `orig_data`: Logical, whether to show the caption note for the
#'   observed data. Default is `FALSE`.
#'   - `orig_data_summary`: Logical, whether to show the caption note for the
#'   observed data summary. Default is `FALSE`.
#'   Only relevant for binary models.
#'   - `coef_exp`: Logical, whether to show the caption note for the
#'   exposure coefficient credible interval. Default is `FALSE`.
#' @param qi_width_sim Width of the quantile interval to summarize simulated
#' draws.
#' @param ... currently not used
#'
#' @return A ggplot object
#' @details
#' Plotting with `ermod` is done with some default values. If they are not
#' suitable, you can always perform the simulation manually and use
#' `plot_er()` on the simulated data.
#'
#' @examplesIf BayesERtools:::.if_run_ex_plot_er()
#' \donttest{
#' data(d_sim_binom_cov_hgly2)
#'
#' ermod_bin <- dev_ermod_bin(
#'   data = d_sim_binom_cov_hgly2,
#'   var_resp = "AEFLAG",
#'   var_exposure = "AUCss_1000"
#' )
#'
#' ersim_med_qi <- sim_er_curve(
#'   ermod_bin,
#'   output_type = "median_qi"
#' )
#'
#' plot_er(ersim_med_qi, show_orig_data = TRUE) +
#'   # Use log10 scale for exposure
#'   xgxr::xgx_scale_x_log10(guide = ggplot2::guide_axis(minor.ticks = TRUE))
#' }
#'
plot_er <- function(x, ...) {
  UseMethod("plot_er")
}

#' @export
#' @rdname plot_er
plot_er.ersim_med_qi <- function(
    x,
    show_orig_data = FALSE,
    show_coef_exp = FALSE,
    show_caption = FALSE,
    options_orig_data = list(),
    options_coef_exp = list(),
    options_caption = list(),
    ...) {
  options_orig_data <-
    utils::modifyList(
      list(
        add_boxplot = FALSE, boxplot_height = 0.15,
        show_boxplot_y_title = TRUE, var_group = NULL,
        n_bins = 4, bin_breaks = NULL, qi_width = 0.95,
        return_components = FALSE
      ),
      options_orig_data
    )
  options_coef_exp <-
    utils::modifyList(
      list(
        qi_width = 0.95, n_sigfig = 3, pos_x = NULL, pos_y = NULL, size = 4
      ),
      options_coef_exp
    )
  options_caption <-
    utils::modifyList(
      list(
        orig_data = FALSE, orig_data_summary = FALSE, coef_exp = FALSE
      ),
      options_caption
    )

  origdata <- extract_data(x)
  var_exposure <- extract_var_exposure(x)
  add_boxplot <- options_orig_data$add_boxplot
  boxplot_height <- options_orig_data$boxplot_height
  return_components <- options_orig_data$return_components

  # Check input
  check_plot_er_input(x, show_orig_data)
  origdata <-
    check_group_col(origdata, show_orig_data, options_orig_data$var_group)

  # Main plot -----------------------------------------------------------------
  gg <-
    ggplot2::ggplot(
      data = x,
      ggplot2::aes(x = .data[[var_exposure]], y = .epred)
    ) +
    ggplot2::geom_ribbon(
      ggplot2::aes(ymin = .epred.lower, ymax = .epred.upper),
      alpha = 0.3
    ) +
    ggplot2::geom_line()

  if (attr(x, "endpoint_type") == "binary") {
    gg <- .plot_er_binary(
      gg, x, origdata, show_orig_data, options_orig_data
    )
  } else {
    gg <- .plot_er_continuous(
      gg, x, origdata, show_orig_data, options_orig_data
    )
  }

  # Add exposure coefficient CI annotation ------------------------------------
  if (show_coef_exp) {
    coef_exp <- calc_coef_exp(x, show_coef_exp, options_coef_exp)
    coef_exp_str <-
      gt::vec_fmt_number(coef_exp, n_sigfig = options_coef_exp$n_sigfig)

    pos_ci_annot <-
      set_pos_ci_annot(
        x, options_coef_exp$pos_x, options_coef_exp$pos_y,
        var_exposure, extract_var_resp(x)
      )

    gg <- gg +
      ggplot2::annotate(
        "label",
        x = pos_ci_annot[1], y = pos_ci_annot[2],
        label = paste0(
          coef_exp_str[1], "-", coef_exp_str[2],
          " (", options_coef_exp$qi_width * 100, "% CI)"
        ),
        label.padding = grid::unit(0.4, "lines"),
        alpha = 0.7,
        hjust = 0, vjust = 1, size = options_coef_exp$size
      )
  }

  # Return components if requested --------------------------------------------
  if (return_components) {
    # Build boxplot if needed
    gg_boxplot <- NULL
    if (add_boxplot) {
      var_group <- options_orig_data$var_group

      if (is.null(var_group)) {
        gg_boxplot <-
          ggplot2::ggplot(data = origdata, ggplot2::aes(
            x = .data[[var_exposure]],
            y = 0
          )) +
          ggplot2::geom_boxplot(alpha = 0.5) +
          ggplot2::scale_y_continuous(expand = ggplot2::expansion(mult = 0.2)) +
          ggplot2::theme(
            panel.grid.major.y = ggplot2::element_blank(),
            panel.grid.minor.y = ggplot2::element_blank(),
            axis.ticks.y = ggplot2::element_blank(),
            axis.title.y = ggplot2::element_blank(),
            axis.text.y = ggplot2::element_blank()
          )
      } else {
        # Define the numeric variable name with a leading dot
        var_group_num <- paste0(".", var_group, "_num")

        # Convert the categorical variable to a numeric index
        origdata[[var_group_num]] <- as.numeric(factor(origdata[[var_group]]))

        gg_boxplot <-
          ggplot2::ggplot(data = origdata, ggplot2::aes(
            x = .data[[var_exposure]],
            y = .data[[var_group_num]]
          )) +
          ggplot2::geom_boxplot(
            ggplot2::aes(
              fill = .data[[var_group]],
              color = .data[[var_group]]
            ),
            alpha = 0.5
          ) +
          ggplot2::scale_y_continuous(
            name = var_group,
            breaks = unique(origdata[[var_group_num]]),
            labels = levels(factor(origdata[[var_group]])),
            expand = ggplot2::expansion(mult = 0.2)
          ) +
          ggplot2::theme(
            panel.grid.minor.y = ggplot2::element_blank()
          )
        if (!options_orig_data$show_boxplot_y_title) {
          gg_boxplot <- gg_boxplot +
            ggplot2::theme(axis.title.y = ggplot2::element_blank())
        }
      }

      gg_boxplot <- gg_boxplot +
        ggplot2::theme(
          legend.position = "none",
          plot.margin = ggplot2::margin(t = 0)
        )
    }

    # Build caption if needed
    caption <- NULL
    if (show_caption) {
      caption <- .build_caption(
        x, show_orig_data, show_coef_exp,
        options_orig_data, options_coef_exp, options_caption
      )
    }

    # Create components list
    components <- list(
      main = gg,
      boxplot = gg_boxplot,
      caption = caption
    )
    attr(components, "metadata") <- list(
      boxplot_height = boxplot_height,
      var_exposure = var_exposure,
      var_resp = extract_var_resp(x),
      endpoint_type = attr(x, "endpoint_type")
    )
    class(components) <- c("er_plot_components", "list")
    return(components)
  }

  # Add boxplot ---------------------------------------------------------------
  if (add_boxplot) {
    rlang::check_installed("patchwork")

    var_group <- options_orig_data$var_group

    if (is.null(var_group)) {
      gg_boxplot <-
        ggplot2::ggplot(data = origdata, ggplot2::aes(
          x = .data[[var_exposure]],
          y = 0
        )) +
        ggplot2::geom_boxplot(alpha = 0.5) +
        ggplot2::scale_y_continuous(expand = ggplot2::expansion(mult = 0.2)) +
        ggplot2::theme(
          panel.grid.major.y = ggplot2::element_blank(),
          panel.grid.minor.y = ggplot2::element_blank(),
          axis.ticks.y = ggplot2::element_blank(),
          axis.title.y = ggplot2::element_blank(),
          axis.text.y = ggplot2::element_blank()
        )
    } else {
      # Define the numeric variable name with a leading dot
      var_group_num <- paste0(".", var_group, "_num")

      # Convert the categorical variable to a numeric index
      origdata[[var_group_num]] <- as.numeric(factor(origdata[[var_group]]))

      gg_boxplot <-
        ggplot2::ggplot(data = origdata, ggplot2::aes(
          x = .data[[var_exposure]],
          y = .data[[var_group_num]]
        )) +
        ggplot2::geom_boxplot(
          ggplot2::aes(
            fill = .data[[var_group]],
            color = .data[[var_group]]
          ),
          alpha = 0.5
        ) +
        ggplot2::scale_y_continuous(
          name = var_group,
          breaks = unique(origdata[[var_group_num]]),
          labels = levels(factor(origdata[[var_group]])),
          expand = ggplot2::expansion(mult = 0.2)
        ) +
        ggplot2::theme(
          panel.grid.minor.y = ggplot2::element_blank()
        )
      if (!options_orig_data$show_boxplot_y_title) {
        gg_boxplot <- gg_boxplot +
          ggplot2::theme(axis.title.y = ggplot2::element_blank())
      }
    }

    gg_boxplot <- gg_boxplot +
      ggplot2::theme(
        legend.position = "none",
        plot.margin = ggplot2::margin(t = 0)
      )

    gg <- gg +
      ggplot2::theme(
        plot.margin = ggplot2::margin(b = 0)
      )

    gg <-
      patchwork::wrap_plots(list(gg, gg_boxplot),
        nrow = 2, heights = c(1 - boxplot_height, boxplot_height)
      ) +
      patchwork::plot_layout(axes = "collect", guides = "collect")
  }

  # Add caption note ----------------------------------------------------------
  if (show_caption) {
    caption <- .build_caption(
      x, show_orig_data, show_coef_exp,
      options_orig_data, options_coef_exp, options_caption
    )
    gg <- gg +
      ggplot2::labs(caption = caption) +
      ggplot2::theme(
        plot.caption = ggplot2::element_text(family = "mono", hjust = 0)
      )
  }

  return(gg)
}


.build_caption <- function(
    x, show_orig_data, show_coef_exp,
    options_orig_data, options_coef_exp, options_caption) {
  qi_width_sim <- attr(x, "qi_width")

  # Base caption line
  lines <- list(
    paste0(
      "Line/Shade: Median/",
      qi_width_sim * 100, "% CI of model sim."
    )
  )

  # Append additional lines based on conditions
  if (show_orig_data && options_caption$orig_data_summary) {
    if (attr(x, "endpoint_type") == "binary") {
      lines <- c(lines, paste0(
        "Squares/bars: Median/", options_orig_data$qi_width * 100,
        "% CI of obs prob."
      ))
    }
  }

  if (show_orig_data && options_caption$orig_data) {
    lines <- c(lines, "Circles: Observed data points")
  }

  if (show_coef_exp && options_caption$coef_exp) {
    lines <- c(lines, paste0(
      "Text: ", options_coef_exp$qi_width * 100,
      "% CI of exposure effect coef."
    ))
  }

  # Combine lines into a single caption string
  caption <- paste(lines, collapse = "\n")

  return(caption)
}

.plot_er_binary <- function(
    gg, x, origdata, show_orig_data, options_orig_data) {
  var_resp <- extract_var_resp(x)
  var_exposure <- extract_var_exposure(x)

  coord_bin_default <- ggplot2::coord_cartesian(ylim = c(-0.05, 1.05))
  coord_bin_default$default <- TRUE

  gg <-
    gg + coord_bin_default +
    ggplot2::scale_y_continuous(
      breaks = c(0, .5, 1),
      labels = scales::percent
    ) +
    ggplot2::labs(x = var_exposure, y = "Probability of event")

  if (show_orig_data) {
    rlang::check_installed("xgxr")

    var_group <- options_orig_data$var_group
    n_bins <- options_orig_data$n_bins
    bin_breaks <- options_orig_data$bin_breaks
    qi_width <- options_orig_data$qi_width

    # Convert the response variable to a numeric index
    origdata$.resp_num <- as.numeric(factor(origdata[[var_resp]])) - 1

    # binned probability ------------------------------------------------------
    if (is.null(bin_breaks)) {
      bin_breaks <- stats::quantile(
        origdata[[var_exposure]],
        probs = seq(0, 1, 1 / n_bins)
      )
    }

    # Show error when the breaks are not unique
    if (length(unique(bin_breaks)) != length(bin_breaks)) {
      stop(
        "The breaks for the binned probability are not unique, ",
        "possibly due to too few unique values in the exposure variable.\n",
        "Please adjust the number of bins or the exposure variable."
      )
    }

    gg <- gg +
      ggplot2::geom_vline(
        xintercept = bin_breaks,
        linetype = "dashed",
        alpha = 0.3
      ) +
      xgxr::xgx_stat_ci(
        data = origdata,
        ggplot2::aes(
          x = .data[[var_exposure]],
          y = .data[[".resp_num"]]
        ),
        breaks = bin_breaks,
        conf_level = qi_width,
        distribution = "binomial",
        geom = c("point"),
        shape = 0,
        size = 4
      ) +
      xgxr::xgx_stat_ci(
        data = origdata,
        ggplot2::aes(
          x = .data[[var_exposure]],
          y = .data[[".resp_num"]]
        ),
        breaks = bin_breaks,
        conf_level = qi_width,
        distribution = "binomial",
        geom = c("errorbar"),
        linewidth = 0.5
      )


    # jitter plots -----------------------------------------------------------
    if (!is.null(var_group)) {
      gg <- gg +
        ggplot2::geom_jitter(
          data = origdata,
          ggplot2::aes(
            x = .data[[var_exposure]], y = .data[[".resp_num"]],
            color = .data[[var_group]]
          ),
          width = 0, height = 0.05, alpha = 0.5
        ) +
        ggplot2::scale_color_discrete(
          guide = ggplot2::guide_legend(reverse = TRUE)
        )
    } else {
      gg <- gg +
        ggplot2::geom_jitter(
          data = origdata,
          ggplot2::aes(x = .data[[var_exposure]], y = .data[[".resp_num"]]),
          width = 0, height = 0.05, alpha = 0.5
        )
    }
  }

  return(gg)
}

.plot_er_continuous <- function(
    gg, x, origdata, show_orig_data, options_orig_data) {
  var_resp <- extract_var_resp(x)
  var_exposure <- extract_var_exposure(x)

  gg <- gg +
    ggplot2::geom_ribbon(
      ggplot2::aes(ymin = .prediction.lower, ymax = .prediction.upper),
      alpha = 0.15
    ) +
    ggplot2::labs(x = var_exposure, y = "Response")

  if (show_orig_data) {
    var_group <- options_orig_data$var_group

    if (!is.null(var_group)) {
      gg <- gg +
        ggplot2::geom_point(
          data = origdata,
          ggplot2::aes(
            x = .data[[var_exposure]], y = .data[[var_resp]],
            color = .data[[var_group]]
          )
        ) +
        ggplot2::scale_color_discrete(
          guide = ggplot2::guide_legend(reverse = TRUE)
        )
    } else {
      gg <- gg +
        ggplot2::geom_point(
          data = origdata,
          ggplot2::aes(x = .data[[var_exposure]], y = .data[[var_resp]])
        )
    }
  }

  return(gg)
}

#' @export
#' @rdname plot_er
plot_er.ersim <- function(
    x,
    show_orig_data = FALSE,
    show_coef_exp = FALSE,
    show_caption = FALSE,
    options_orig_data = list(),
    options_coef_exp = list(),
    options_caption = list(),
    qi_width_sim = 0.95,
    ...) {
  ersim_med_qi <- calc_ersim_med_qi(x, qi_width = qi_width_sim)

  plot_er(
    ersim_med_qi,
    show_orig_data = show_orig_data,
    show_coef_exp = show_coef_exp,
    show_caption = show_caption,
    options_orig_data = options_orig_data,
    options_coef_exp = options_coef_exp,
    options_caption = options_caption
  )
}


#' @export
#' @rdname plot_er
#' @inheritParams sim_er
#' @param n_draws_sim Number of draws to simulate response for each exposure
#' value. Set to NULL to use all draws in the model object. Default is NULL
#' unless marginal is set to TRUE (in that case 200 by default to reduce
#' computation time).
#' @param marginal logical, whether to use marginal ER simulation. Default
#' to `FALSE`. Need to set to `TRUE` if the model has covariates for the
#' plot to work.
#' @param exposure_range Only relevant when the input x is an `ermod` object.
#' Range of exposure values to simulate. If NULL
#' (default), it is set to the range of the exposure variable in the original
#' data for model development.
#' @param num_exposures Only relevant as with `exposure_range`.
#' Number of exposure values to simulate.
#'
plot_er.ermod <- function(
    x,
    show_orig_data = FALSE,
    show_coef_exp = FALSE,
    show_caption = FALSE,
    options_orig_data = list(),
    options_coef_exp = list(),
    options_caption = list(),
    n_draws_sim = if (marginal) 200 else NULL,
    seed_sample_draws = NULL,
    marginal = FALSE,
    exposure_range = NULL,
    num_exposures = 51,
    qi_width_sim = 0.95,
    ...) {
  if (!marginal && !is.null(extract_var_cov(x))) {
    stop(
      "Model has covariate(s), and you cannot use this function directly ",
      "on `ermod` object unless you set `marginal = TRUE`, if",
      "that is what you want.\n",
      "Otherwise, perform the simulation manually with e.g. `sim_er_curve()`",
      "and use `plot_er()` on the simulated data."
    )
  }

  if (marginal) {
    ersim_curve_med_pi <- sim_er_curve_marg(
      x,
      exposure_range = exposure_range,
      num_exposures = num_exposures,
      n_draws_sim = n_draws_sim,
      seed_sample_draws = seed_sample_draws,
      output_type = "median_qi",
      qi_width = qi_width_sim
    )
  } else {
    ersim_curve_med_pi <- sim_er_curve(
      x,
      exposure_range = exposure_range,
      num_exposures = num_exposures,
      n_draws_sim = n_draws_sim,
      seed_sample_draws = seed_sample_draws,
      output_type = "median_qi",
      qi_width = qi_width_sim
    )
  }

  plot_er(
    ersim_curve_med_pi,
    show_orig_data = show_orig_data,
    show_coef_exp = show_coef_exp,
    show_caption = show_caption,
    options_orig_data = options_orig_data,
    options_coef_exp = options_coef_exp,
    options_caption = options_caption
  )
}

#' Default GOF plot for ER model
#'
#' This is a wrapper function for [plot_er()] with default options for
#' goodness-of-fit (GOF) plots for ER models.
#'
#' @export
#' @inheritParams plot_er
#' @param add_boxplot Logical, whether to add a boxplot of exposure values.
#' Default is `TRUE` if `var_group` is specified, otherwise `FALSE`.
#' @param boxplot_height Height of the boxplot relative to the main plot.
#' Default is `0.15`.
#' @param show_boxplot_y_title Logical, whether to show the y-axis title
#' for the boxplot. Default is `FALSE`.
#' @param var_group The column to use for grouping data for plotting.
#' If specified, observed data points and boxplot will be grouped
#' and colored by this column. Default is `NULL`.
#' @param n_bins Number of bins to use for observed probability
#' summary. Only relevant for binary models. Default is `4`.
#' @param bin_breaks Manually specify bin breaks for binary models. If specified
#' this overrides `n_bins`.
#' @param qi_width_obs Confidence level for the observed probability
#' summary. Default is `0.95`.
#' @param show_coef_exp Logical, whether to show the credible interval
#' of the exposure coefficient. Default is `FALSE`. This is only available
#' for linear and linear logistic regression models.
#' @param coef_pos_x x-coordinate of the text label. If `NULL` (default), it is
#' set to the minimum value for the exposure variable.
#' @param coef_pos_y y-coordinate of the text label. If `NULL` (default), it is
#' set to 0.9 for logistic regression models and the maximum value of the
#' response variable in the original data for linear regression models.
#' @param coef_size Size of the text label. Default is `4`.
#' @param qi_width_coef Width of the credible interval for the exposure
#' coefficient. Default is `0.95`.
#' @param qi_width_sim Width of the quantile interval to summarize simulated
#' draws. Default is `0.95`.
#' @param show_caption Logical, whether to show the caption note for the
#' plot. Default is `TRUE`.
#' @param return_components Logical, whether to return plot components as a
#' list instead of a combined plot. When `TRUE`, returns a list with
#' `$main` (main plot), `$boxplot` (boxplot if applicable), `$caption`
#' (caption text if applicable), and `$metadata`. This allows users to
#' customize individual plot components before combining them. Default is
#' `FALSE`.
#'
#' @details
#' The following code will generate the same plot:
#' \preformatted{
#' plot_er(
#'   x,
#'   show_orig_data = TRUE,
#'   show_coef_exp = show_coef_exp,
#'   show_caption = show_caption,
#'   options_orig_data = list(
#'     add_boxplot = add_boxplot, boxplot_height = boxplot_height,
#'     show_boxplot_y_title = show_boxplot_y_title,
#'     var_group = var_group,
#'     n_bins = n_bins, qi_width = qi_width_obs
#'   ),
#'   options_coef_exp = list(
#'     qi_width = qi_width_coef, pos_x = coef_pos_x, pos_y = coef_pos_y,
#'     size = coef_size
#'   ),
#'   options_caption = list(
#'     orig_data_summary = TRUE, coef_exp = show_coef_exp
#'   ),
#'   qi_width_sim = qi_width_sim
#' )
#' }
#'
#' To customize plot elements (titles, shapes, themes), use
#' `return_components = TRUE` to get individual plot components, modify them,
#' then recombine with `combine_er_components()`.
#'
#' @return A ggplot object
#' @examplesIf BayesERtools:::.if_run_ex_plot_er()
#' \donttest{
#' data(d_sim_binom_cov_hgly2)
#'
#' ermod_bin <- dev_ermod_bin(
#'   data = d_sim_binom_cov_hgly2,
#'   var_resp = "AEFLAG",
#'   var_exposure = "AUCss_1000"
#' )
#'
#' plot_er_gof(ermod_bin, var_group = "Dose_mg", show_coef_exp = TRUE) *
#'   # Use log10 scale for exposure, need to use `*` instead of `+`
#'   xgxr::xgx_scale_x_log10(guide = ggplot2::guide_axis(minor.ticks = TRUE))
#'
#' # Customize plot components
#' comps <- plot_er_gof(ermod_bin, var_group = "Dose_mg", return_components = TRUE)
#' comps$main <- comps$main +
#'   ggplot2::labs(title = "Exposure-Response Analysis", x = "AUC (ng*h/mL)") +
#'   ggplot2::theme_bw()
#' combine_er_components(comps)
#' }
#'
plot_er_gof <- function(
    x, add_boxplot = !is.null(var_group),
    boxplot_height = 0.15,
    show_boxplot_y_title = FALSE,
    var_group = NULL,
    n_bins = 4,
    bin_breaks = NULL,
    qi_width_obs = 0.95,
    show_coef_exp = FALSE,
    coef_pos_x = NULL,
    coef_pos_y = NULL,
    coef_size = 4,
    qi_width_coef = 0.95,
    qi_width_sim = 0.95,
    show_caption = TRUE,
    return_components = FALSE) {
  plot_er(
    x,
    show_orig_data = TRUE,
    show_coef_exp = show_coef_exp,
    show_caption = show_caption,
    options_orig_data = list(
      add_boxplot = add_boxplot,
      boxplot_height = boxplot_height,
      show_boxplot_y_title = show_boxplot_y_title,
      var_group = var_group,
      n_bins = n_bins,
      bin_breaks = bin_breaks,
      qi_width = qi_width_obs,
      return_components = return_components
    ),
    options_coef_exp = list(
      qi_width = qi_width_coef,
      pos_x = coef_pos_x,
      pos_y = coef_pos_y,
      size = coef_size
    ),
    options_caption = list(
      orig_data_summary = TRUE,
      coef_exp = show_coef_exp
    ),
    qi_width_sim = qi_width_sim
  )
}

# Check if covariates were used in the model, which can affect potting
# and comparison with data
# x: ersim object
check_plot_er_input <- function(x, show_orig_data) {
  var_cov <- extract_var_cov(x)
  nrow_cov_data <- attr(x, "nrow_cov_data")
  is_marginal <- inherits(x, "ersim_marg_med_qi")

  if (is.null(var_cov)) {
    return()
  }

  if (nrow_cov_data > 1) {
    stop(
      "Model has covariate(s) and multiple covariate data rows were ",
      "provided. Consider manual plotting or using marginal ER simulation."
    )
  }

  if (show_orig_data == TRUE && !is_marginal) {
    warning(
      "Model has covariate(s), and only one covariate data row was provided. ",
      "Data points shown can be misleading. Consider manual plotting or ",
      "using marginal ER simulation."
    )
  }
}

# Check var_group if original data is to be shown
check_group_col <- function(origdata, show_orig_data, var_group) {
  if (!show_orig_data || is.null(var_group)) {
    return(origdata)
  }

  if (!var_group %in% names(origdata)) {
    stop(
      "Column `", var_group, "` not found in the original data. ",
      "Please check the column name."
    )
  }
  # if var_group is numeric, warn
  if (is.numeric(origdata[[var_group]])) {
    # Stop if there are 10 or more levels
    if (length(unique(origdata[[var_group]])) > 10) {
      stop(
        "Column `", var_group, "` is numeric and has > 10 unique values. ",
        "Please convert to factor for grouping."
      )
    }

    origdata[[var_group]] <- factor(origdata[[var_group]])
  }

  return(origdata)
}


calc_coef_exp <- function(x, show_coef_exp, options_coef_exp) {
  if (any(c("ermod_bin", "ermod_lin") %in% attr(x, "ermod_class"))) {
    qi_width <- options_coef_exp$qi_width
    coef_exp_draws <- attr(x, "coef_exp_draws")
    coef_exp <-
      stats::quantile(
        coef_exp_draws,
        c(0.5 - qi_width / 2, 0.5 + qi_width / 2)
      )
  } else {
    stop("`show_coef_exp =TRUE` is only available for linear models.")
  }

  return(coef_exp)
}

set_pos_ci_annot <- function(x, pos_x, pos_y, var_exposure, var_resp) {
  if (is.null(pos_x)) {
    pos_x <- min(x[[var_exposure]])
  }
  if (is.null(pos_y)) {
    if (attr(x, "endpoint_type") == "binary") {
      pos_y <- 0.9
    } else {
      pos_y <- max(extract_data(x)[[var_resp]])
    }
  }
  return(c(pos_x, pos_y))
}

.if_run_ex_plot_er <- function() {
  requireNamespace("xgxr", quietly = TRUE)
}

#' Combine ER plot components
#'
#' @param components An object of class `er_plot_components` returned by
#' `plot_er()` or `plot_er_gof()` with `return_components = TRUE` or
#' `options_orig_data = list(return_components = TRUE)`.
#' @param heights Numeric vector of length 2 specifying the relative heights
#' of the main plot and boxplot. If `NULL` (default), uses the height from
#' the metadata (based on `boxplot_height` parameter).
#' @param add_caption Logical, whether to add the caption to the combined plot.
#' Default is `TRUE` if caption is available.
#' @param ... Additional arguments passed to `patchwork::wrap_plots()`.
#'
#' @return A combined ggplot object.
#' @export
#'
#' @examples
#' \dontrun{
#' # Get components
#' comps <- plot_er_gof(ermod_bin, return_components = TRUE)
#'
#' # Modify components
#' comps$main <- comps$main + labs(title = "Custom Title")
#'
#' # Recombine
#' combine_er_components(comps)
#' }
combine_er_components <- function(components, heights = NULL,
                                  add_caption = !is.null(components$caption),
                                  ...) {
  if (!inherits(components, "er_plot_components")) {
    stop(
      "Input must be an object of class 'er_plot_components' ",
      "returned by plot_er() or plot_er_gof() with return_components = TRUE."
    )
  }

  gg <- components$main

  # Add boxplot if present
  if (!is.null(components$boxplot)) {
    rlang::check_installed("patchwork")

    # Set default heights if not provided
    if (is.null(heights)) {
      boxplot_height <- attr(components, "metadata")$boxplot_height
      heights <- c(1 - boxplot_height, boxplot_height)
    }

    # Adjust margins for combination
    gg <- gg +
      ggplot2::theme(plot.margin = ggplot2::margin(b = 0))

    gg <-
      patchwork::wrap_plots(
        list(gg, components$boxplot),
        nrow = 2, heights = heights, ...
      ) +
      patchwork::plot_layout(axes = "collect", guides = "collect")
  }

  # Add caption if requested
  if (add_caption && !is.null(components$caption)) {
    gg <- gg +
      ggplot2::labs(caption = components$caption) +
      ggplot2::theme(
        plot.caption = ggplot2::element_text(family = "mono", hjust = 0)
      )
  }

  return(gg)
}

#' @export
print.er_plot_components <- function(x, ...) {
  cat("ER plot components (class: er_plot_components)\n")
  cat("Components:\n")
  cat("  $main    : Main ER plot\n")
  if (!is.null(x$boxplot)) {
    cat("  $boxplot : Exposure boxplot\n")
  }
  if (!is.null(x$caption)) {
    cat("  $caption : Caption text\n")
  }
  cat("\nMetadata:\n")
  metadata <- attr(x, "metadata")
  for (name in names(metadata)) {
    cat(sprintf("  %s: %s\n", name, paste(metadata[[name]], collapse = ", ")))
  }
  cat("\nUse combine_er_components() to recombine after customization.\n")
  cat("Or access individual components for manual combination.\n")
  invisible(x)
}

# Read d_sim_binom_cov data, which is exported to the package
d_sim_binom_cov <- d_sim_binom_cov

id_to_sample <- seq(1, max(d_sim_binom_cov$ID), by = 1)
id_to_sample2 <- seq(1, max(d_sim_binom_cov$ID), by = 5)

df_er_ae_hgly2 <-
  d_sim_binom_cov |>
  dplyr::filter(
    AETYPE == "hgly2",
    ID %in% id_to_sample
  ) |>
  dplyr::mutate(
    AUCss_1000 = AUCss / 1000, BAGE_10 = BAGE / 10,
    BWT_10 = BWT / 10, BHBA1C_5 = BHBA1C / 5
  )

df_er_dr2 <-
  d_sim_binom_cov |>
  dplyr::filter(
    AETYPE == "dr2",
    ID %in% id_to_sample
  ) |>
  dplyr::mutate(
    AUCss_1000 = AUCss / 1000, BAGE_10 = BAGE / 10,
    BWT_10 = BWT / 10
  )

var_resp <- "AEFLAG"
var_exp_candidates <- c("AUCss_1000", "Cmaxss", "Cminss")


# develop models --------------------------------------------------------------

set.seed(1234)
ermod_bin <- dev_ermod_bin(
  data = df_er_ae_hgly2,
  var_resp = var_resp,
  var_exposure = "AUCss_1000",
  var_cov = NULL,
  verbosity_level = 0,
  # Below option to make the test fast
  chains = 2, iter = 1000
)

set.seed(1234)
ermod_bin_w_cov <- dev_ermod_bin(
  data = df_er_ae_hgly2,
  var_resp = var_resp,
  var_exposure = "AUCss_1000",
  var_cov = "BHBA1C_5",
  verbosity_level = 0,
  # Below option to make the test fast
  chains = 2, iter = 1000
)

set.seed(1234)
ermod_bin_exp_sel <-
  dev_ermod_bin_exp_sel(
    data = df_er_ae_hgly2,
    var_resp = var_resp,
    var_exp_candidates = var_exp_candidates,
    verbosity_level = 0,
    # Below option to make the test fast
    chains = 2, iter = 1000
  )

# Simulate responses ----------------------------------------------------------
ersim_curve <- sim_er_curve(
  ermod_bin,
  n_draws_sim = 200,
  num_exposures = 31,
  output_type = "draws"
)

ersim_curve_med_qi <- sim_er_curve(
  ermod_bin,
  n_draws_sim = 200,
  num_exposures = 31,
  output_type = "median_qi",
  qi_width = 0.95
)

ersim_curve_2_med_qi <- sim_er_curve(
  ermod_bin_w_cov,
  data_cov = dplyr::tibble(BHBA1C_5 = 8),
  num_exposures = 31,
  output_type = "median_qi",
  qi_width = 0.95
)

ersim_curve_3_med_qi <- sim_er_curve(
  ermod_bin_w_cov,
  data_cov = dplyr::tibble(BHBA1C_5 = c(4, 8)),
  num_exposures = 31,
  output_type = "median_qi",
  qi_width = 0.95
)

ersim_new_exp_marg_med_qi <- sim_er_new_exp_marg(
  ermod_bin_w_cov,
  exposure_to_sim_vec = c(2, 2:6),
  data_cov = dplyr::tibble(BHBA1C_5 = 4:10, AUCss_1000 = 4:10),
  n_subj_sim = NULL,
  n_draws_sim = 200,
  output_type = "median_qi"
)

# Test ------------------------------------------------------------------------
test_that("plot_er.ermod", {
  if (requireNamespace("xgxr")) {
    g1 <- plot_er(ermod_bin, show_orig_data = TRUE)
    g2 <- plot_er(ermod_bin_w_cov,
      show_orig_data = TRUE, marginal = TRUE,
      n_draws_sim = 50, num_exposures = 31, exposure_range = c(1, 3)
    )

    expect_equal(nrow(g1$data), 51)
    expect_equal(g2$data$AUCss_1000, seq(1, 3, length.out = 31))
  }
  plot_er(ermod_bin_w_cov) |>
    expect_error("Model has covariate\\(s\\), and you cannot use this")
})

test_that("plot_er.ersim", {
  g1 <- plot_er(ersim_curve)
  g2 <- plot_er(ersim_curve_med_qi)
  if (requireNamespace("xgxr")) {
    g3 <- plot_er(ersim_curve_med_qi, show_orig_data = TRUE)
  }

  expect_equal(nrow(g1$data), 31) # Make sure sim_er_curve worked fine
  expect_equal(g2$labels$x, "AUCss_1000")

  expect_silent(plot(g1))

  if (requireNamespace("xgxr")) {
    plot_er(ersim_curve_2_med_qi, show_orig_data = TRUE) |>
      expect_warning("Model has covariate\\(s\\), and only one covariate data")
  }
  plot_er(ersim_curve_3_med_qi) |>
    expect_error("Model has covariate\\(s\\) and multiple covariate data rows")
})

test_that("plot_er with groups", {
  if (requireNamespace("xgxr")) {
    plot_er(ermod_bin,
      show_orig_data = TRUE,
      options_orig_data = list(var_group = "AUCss_1000")
    ) |>
      expect_error("Column `AUCss_1000` is numeric and has > 10 unique values")

    plot_er(ermod_bin,
      show_orig_data = TRUE,
      options_orig_data = list(var_group = "Dose_mg")
    ) |>
      expect_silent()

    plot_er(ermod_bin,
      show_orig_data = TRUE,
      options_orig_data = list(var_group = "Dose_mg", add_boxplot = TRUE)
    ) |>
      expect_silent()

    plot_er(ermod_bin,
      show_orig_data = TRUE,
      options_orig_data = list(add_boxplot = TRUE)
    ) |>
      expect_silent()
  }
})

test_that("plot_er add CI", {
  plot_er(ermod_bin,
    show_coef_exp = TRUE,
    options_coef_exp = list(size = 6)
  ) |>
    expect_silent()
})

test_that("plot_er show caption", {
  if (requireNamespace("xgxr")) {
    plot_er(
      ersim_curve_med_qi,
      show_orig_data = TRUE,
      show_coef_exp = TRUE,
      show_caption = TRUE,
      options_coef_exp = list(size = 6),
      options_caption = list(orig_data = TRUE, orig_data_summary = TRUE)
    ) |>
      expect_silent()
  }

  plot_er(ermod_bin,
    show_coef_exp = TRUE,
    show_caption = TRUE,
    options_coef_exp = list(size = 6),
    options_caption = list(coef_exp = TRUE)
  ) |>
    expect_silent()
})

test_that("plot_er_gof", {
  if (requireNamespace("xgxr")) {
    plot_er_gof(ermod_bin) |>
      expect_silent()
    plot_er_gof(ermod_bin, var_group = "Dose_mg") |>
      expect_silent()
    plot_er_gof(ermod_bin, show_coef_exp = TRUE, show_caption = TRUE) |>
      expect_silent()
  }
})

test_that("plot_er return_components works", {
  if (requireNamespace("xgxr")) {
    # Test return_components with plot_er
    comps <- plot_er(
      ersim_curve_med_qi,
      show_orig_data = TRUE,
      show_caption = TRUE,
      options_orig_data = list(
        add_boxplot = TRUE,
        return_components = TRUE
      )
    )

    expect_s3_class(comps, "er_plot_components")
    expect_true("main" %in% names(comps))
    expect_true("boxplot" %in% names(comps))
    expect_true("caption" %in% names(comps))
    expect_s3_class(comps$main, "ggplot")
    expect_s3_class(comps$boxplot, "ggplot")
    expect_type(comps$caption, "character")

    # Check metadata
    metadata <- attr(comps, "metadata")
    expect_true("boxplot_height" %in% names(metadata))
    expect_true("var_exposure" %in% names(metadata))
    expect_true("endpoint_type" %in% names(metadata))

    # Test without boxplot
    comps_no_box <- plot_er(
      ersim_curve_med_qi,
      show_orig_data = TRUE,
      options_orig_data = list(return_components = TRUE)
    )
    expect_null(comps_no_box$boxplot)
  }
})

test_that("plot_er_gof return_components works", {
  if (requireNamespace("xgxr")) {
    comps <- plot_er_gof(
      ermod_bin,
      var_group = "Dose_mg",
      show_caption = TRUE,
      return_components = TRUE
    )

    expect_s3_class(comps, "er_plot_components")
    expect_s3_class(comps$main, "ggplot")
    expect_s3_class(comps$boxplot, "ggplot")
    expect_type(comps$caption, "character")
  }
})

test_that("combine_er_components works", {
  if (requireNamespace("xgxr") && requireNamespace("patchwork")) {
    comps <- plot_er_gof(
      ermod_bin,
      var_group = "Dose_mg",
      show_caption = TRUE,
      return_components = TRUE
    )

    # Modify the main plot
    comps$main <- comps$main +
      ggplot2::labs(title = "Test Title", x = "Custom X Label")

    # Recombine
    combined <- combine_er_components(comps)
    expect_s3_class(combined, "patchwork")

    # Test without caption
    combined_no_caption <- combine_er_components(comps, add_caption = FALSE)
    expect_s3_class(combined_no_caption, "patchwork")

    # Test with custom heights
    combined_custom <- combine_er_components(comps, heights = c(0.7, 0.3))
    expect_s3_class(combined_custom, "patchwork")
  }
})

test_that("combine_er_components without boxplot works", {
  if (requireNamespace("xgxr")) {
    comps <- plot_er(
      ersim_curve_med_qi,
      show_orig_data = TRUE,
      show_caption = TRUE,
      options_orig_data = list(return_components = TRUE)
    )

    # Modify the main plot
    comps$main <- comps$main +
      ggplot2::labs(title = "Test Title")

    # Recombine (should return ggplot since no boxplot)
    combined <- combine_er_components(comps)
    expect_s3_class(combined, "ggplot")
  }
})

test_that("combine_er_components errors on invalid input", {
  expect_error(
    combine_er_components(list(main = NULL)),
    "Input must be an object of class 'er_plot_components'"
  )
})

test_that("print.er_plot_components works", {
  if (requireNamespace("xgxr")) {
    comps <- plot_er_gof(
      ermod_bin,
      var_group = "Dose_mg",
      show_caption = TRUE,
      return_components = TRUE
    )

    # Capture output
    output <- capture.output(print(comps))
    expect_true(any(grepl("ER plot components", output)))
    expect_true(any(grepl("\\$main", output)))
    expect_true(any(grepl("\\$boxplot", output)))
    expect_true(any(grepl("combine_er_components", output)))
  }
})

#' Evaluate exposure-response model prediction performance
#'
#' This function evaluates the performance of an exposure-response model
#' using various metrics.
#'
#' @export
#' @param ermod An object of class `ermod`.
#' @param eval_type A character string specifying the evaluation dataset.
#' Options are:
#'   - `training`: Use the training dataset.
#'   - `test`: Use a new dataset for evaluation.
#'   - `kfold`: Perform k-fold cross-validation (uses `newdata` if provided,
#'     otherwise uses the training dataset).
#' @param newdata A data frame containing new data for evaluation when
#' `eval_type` is set to `test` or `kfold`.
#' @param summary_method A character string specifying how to summarize the
#' simulation draws. Default is `median`.
#' @param k The number of folds for cross-validation. Default is 5.
#' @param seed_kfold Random seed for k-fold cross-validation.
#'
#' @return A tibble with calculated performance metrics, such as AUROC or
#' RMSE, depending on the model type.
#'
#' @examplesIf BayesERtools:::.if_run_ex_eval_mod()
#' \donttest{
#' data(d_sim_binom_cov_hgly2)
#' d_split <- rsample::initial_split(d_sim_binom_cov_hgly2)
#' d_train <- rsample::training(d_split)
#' d_test <- rsample::testing(d_split)
#'
#' ermod_bin <- dev_ermod_bin(
#'   data = d_train,
#'   var_resp = "AEFLAG",
#'   var_exposure = "AUCss_1000",
#'   var_cov = "BHBA1C_5",
#'   # Settings to make the example run faster
#'   chains = 2,
#'   iter = 1000
#' )
#'
#' metrics_training <- eval_ermod(ermod_bin, eval_type = "training")
#' metrics_test <- eval_ermod(ermod_bin, eval_type = "test", newdata = d_test)
#' metrics_kfold <- eval_ermod(ermod_bin, eval_type = "kfold", k = 3)
#'
#' print(metrics_training)
#' print(metrics_test)
#' print(metrics_kfold)
#' }
#'
eval_ermod <- function(
    ermod, eval_type = c("training", "kfold", "test"),
    newdata = NULL, summary_method = c("median", "mean"),
    k = 5, seed_kfold = NULL) {
  rlang::check_installed("yardstick")

  eval_type <- match.arg(eval_type)
  summary_method <- match.arg(summary_method)

  var_resp <- ermod$var_resp

  # Set the data to use for evaluation
  if (is.null(newdata)) {
    if (eval_type == "test") {
      stop("For 'test' evaluation, please provide the 'newdata' parameter.")
    }
    data_to_use <- ermod$data
  } else {
    if (eval_type == "training") {
      warning("The 'newdata' parameter is ignored for 'training' evaluation.")
    }
    data_to_use <- newdata
  }

  if (eval_type == "kfold") {
    cv_results <- kfold(
      ermod,
      newdata = data_to_use, k = k, seed = seed_kfold
    )

    d_sim <- cv_results$d_sim
    d_truth <- cv_results$d_truth |>
      dplyr::ungroup() |>
      dplyr::select(-fold_id)
  } else {
    ersim <- sim_er(
      ermod = ermod,
      newdata = data_to_use,
      output_type = "draws"
    )

    d_sim <- ersim |>
      dplyr::ungroup() |>
      dplyr::select(.row, .draw, pred = .epred) |>
      dplyr::group_by(.row)

    d_truth <- data_to_use |>
      dplyr::mutate(.row = dplyr::row_number()) |>
      dplyr::select(.row, truth = !!rlang::sym(var_resp))
  }

  # Summarize the predictions
  fn_summary <- if (summary_method == "median") stats::median else mean

  d_sim_summary <- d_sim |>
    dplyr::summarize(
      pred = fn_summary(pred),
      .groups = "drop_last"
    )

  # Join with actual response values
  results <-
    dplyr::left_join(d_sim_summary, d_truth, by = ".row")

  if (ermod$endpoint_type == "binary") {
    results <- results |>
      dplyr::mutate(
        truth = factor(truth, levels = c(1, 0))
      )

    class_metrics <- yardstick::metric_set(
      yardstick::roc_auc, yardstick::mn_log_loss
    )
  } else {
    class_metrics <- yardstick::metric_set(
      yardstick::rmse, yardstick::rsq, yardstick::rsq_trad
    )
  }

  metrics <- class_metrics(results, truth, pred)

  return(metrics)
}

.if_run_ex_eval_mod <- function() {
  requireNamespace("rsample", quietly = TRUE) &&
    requireNamespace("yardstick", quietly = TRUE) &&
    requireNamespace("digest", quietly = TRUE) &&
    rlang::is_installed("rstanemax", version = "0.1.9")
}

---
title: "0. Overview of the package"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{0. Overview of the package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

This vignette provides an overview of `BayesERtools`. 

## 0. Analysis workflow & supported model types

Analysis can be performed in the following simple steps.

![Figure: Overall analysis workflow](img/fig_overview_1.png){width=70%}
<br />

Supported model types are as follows:

```{r, echo = FALSE, warning=FALSE, message=FALSE}
library(gt)
library(dplyr)
set.seed(1234) # Needed to stablize div id

# Saved csv from README.Rmd
tab_mod <- read.csv("data/supported_models.csv")

tab_mod |>
  select(-.row_id) |>
  gt() |>
  fmt_markdown() |>
  fmt_url(
    columns = !1,
    rows = 2,
    label = "🔗",
    show_underline = FALSE
  ) |>
  tab_spanner(
    label = "Binary endpoint",
    columns = c(lin_logit, emax_logit)
  ) |>
  tab_spanner(
    label = "Continuous endpoint",
    columns = c(linear, emax)
  ) |>
  cols_label(
    feature_name = "",
    lin_logit = "Linear  \n(logit)",
    emax_logit = md("E<sub/>max</sub> (logit)"),
    linear = "Linear",
    emax = md("E<sub/>max</sub>"),
  ) |>
  tab_style(
    style = cell_text(v_align = "top", align = "center"),
    locations = cells_column_labels()
  ) |>
  tab_style(
    style = cell_text(v_align = "middle", align = "center"),
    locations = cells_body()
  ) |>
  tab_style(
    style = cell_text(v_align = "middle", align = "right"),
    locations = cells_body(columns = feature_name)
  ) |>
  tab_footnote(
    footnote = paste(
      "✅ Available",
      "🟡 In plan/under development",
      "❌ Not in a current plan",
      sep = ", "
    )
  )
```

## 1. ER model development

The package provides a set of functions to develop ER models. 
The following functions are available:
<br />
<br />

**Binary endpoint**

- Linear logistic regression: 
  - `dev_ermod_bin()`, `dev_ermod_bin_exp_sel()`, `dev_ermod_bin_cov_sel()`
- E~max~ logistic regression: 
  - `dev_ermod_bin_emax()`, `dev_ermod_bin_emax_exp_sel()`
  
**Continuous endpoint**

- Linear regression: 
  - `dev_ermod_lin()`, `dev_ermod_lin_exp_sel()`, `dev_ermod_lin_cov_sel()`
- E~max~ model: 
  - `dev_ermod_emax()`, `dev_ermod_emax_exp_sel()`

![Figure: dev_ermod_*() 
functions](img/fig_overview_dev_ermod.png){width=100%}
<br />

## 2. Simulation from developed ER model

The following functions are available for simulation from developed ER models:

- `sim_er()`
- `sim_er_new_exp()`, `sim_er_curve()`
- `sim_er_new_exp_marg()`, `sim_er_curve_marg()`

![Figure: sim_er_*() functions](img/fig_overview_sim_er.png){width=100%}
<br />

## 3. Plot simulated ER curve

Simulated ER curve can be visualized with the following functions:

- `plot_er()`
- `plot_er_exp_sel()`

![Figure: plot_er_*() functions](img/fig_overview_plot_er.png){width=100%}
<br />

## Acknowledgement

Figures created with https://www.biorender.com

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/p_direction.R
\name{p_direction}
\alias{p_direction}
\alias{p_direction.ermod_bin}
\title{Probability of Direction (pd)}
\usage{
\method{p_direction}{ermod_bin}(
  x,
  null = 0,
  as_p = FALSE,
  as_num = FALSE,
  direction = "two-sided",
  ...
)
}
\arguments{
\item{x}{An object of class \code{ermod_bin_*}}

\item{null}{The null hypothesis value. Default is 0.}

\item{as_p}{If \code{TRUE}, the p-direction (pd) values are converted to a
frequentist p-value using \code{\link[bayestestR:pd_to_p]{bayestestR::pd_to_p()}}. Only works when
\code{as_num = TRUE}.}

\item{as_num}{If \code{TRUE}, the output is converted to a numeric value.}

\item{direction}{What type of p-value is requested or provided with
as_p = TRUE. Can be \code{"two-sided"} (default, two tailed) or \code{"one-sided"}
(one tailed).}

\item{...}{Additional arguments passed to \code{\link[bayestestR:p_direction]{bayestestR::p_direction()}}.}
}
\value{
See \code{\link[bayestestR:p_direction]{bayestestR::p_direction()}} for details.
}
\description{
Compute the \strong{Probability of Direction} (\emph{\strong{pd}}). Although differently
expressed, this index is fairly similar (\emph{i.e.}, is strongly correlated) to
the frequentist \strong{p-value}. See \code{\link[bayestestR:p_direction]{bayestestR::p_direction()}} and
\code{vignette("overview_of_vignettes", package = "bayestestR")} >
"Probability of Direction (pd)" page for details.
For converting \strong{pd} to a frequentist \strong{p-value},
see \code{\link[bayestestR:pd_to_p]{bayestestR::pd_to_p()}}.
}
\details{
For the class \verb{ermod_bin_*}, it only calculates the \strong{pd} for
the exposure variable.
}
\examples{
\dontshow{if (BayesERtools:::.if_run_ex_p_dir()) withAutoprint(\{ # examplesIf}
\donttest{
library(bayestestR)

df_er_dr2 <-
  d_sim_binom_cov |>
  dplyr::filter(
    AETYPE == "dr2",
    ID \%in\% seq(1, 500, by = 5)
  ) |>
  dplyr::mutate(AUCss_1000 = AUCss / 1000, BHBA1C_5 = BHBA1C / 5)

ermod_bin <- dev_ermod_bin(
  data = df_er_dr2,
  var_resp = "AEFLAG",
  var_exposure = "AUCss_1000",
  var_cov = "BHBA1C_5"
)

p_direction(ermod_bin, as_num = TRUE, as_p = TRUE)
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BayesERtools-package.R
\docType{package}
\name{BayesERtools-package}
\alias{BayesERtools}
\alias{BayesERtools-package}
\title{BayesERtools: Bayesian Exposure-Response Analysis Tools}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Suite of tools that facilitate exposure-response analysis using Bayesian methods. The package provides a streamlined workflow for fitting types of models that are commonly used in exposure-response analysis - linear and Emax for continuous endpoints, logistic linear and logistic Emax for binary endpoints, as well as performing simulation and visualization. Learn more about the workflow at \url{https://genentech.github.io/BayesERbook/}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://genentech.github.io/BayesERtools/}
  \item \url{https://genentech.github.io/BayesERbook/}
}

}
\author{
\strong{Maintainer}: Kenta Yoshida \email{yoshida.kenta.6@gmail.com} (\href{https://orcid.org/0000-0003-4967-3831}{ORCID})

Authors:
\itemize{
  \item François Mercier \email{francois.mercier@roche.com} (\href{https://orcid.org/0000-0002-5685-1408}{ORCID})
  \item Danielle Navarro \email{djnavarro@protonmail.com} (\href{https://orcid.org/0000-0001-7648-6578}{ORCID})
}

Other contributors:
\itemize{
  \item Genentech, Inc. [copyright holder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ermod-methods.R
\name{ermod_method}
\alias{ermod_method}
\alias{print.ermod}
\alias{plot.ermod_bin}
\alias{coef.ermod}
\alias{summary.ermod}
\title{S3 methods for the classes \code{ermod_*}}
\usage{
\method{print}{ermod}(x, digits = 2, ...)

\method{plot}{ermod_bin}(x, show_orig_data = FALSE, ...)

\method{coef}{ermod}(object, ...)

\method{summary}{ermod}(object, ...)
}
\arguments{
\item{x}{An object of class \code{ermod_*}}

\item{digits}{Number of digits to print}

\item{...}{Additional arguments passed to functions}

\item{show_orig_data}{logical, whether to show the data points in the
model development dataset. Default is \code{FALSE}. Only support plotting
with data that was used in the model development. If you want to use
other data, consider adding geom_point() to the plot manually.}

\item{object}{An object of class \code{ermod_*}}
}
\value{
\itemize{
\item \code{print()} and \code{plot()}: No return value, called for side effects
\item \code{coef()}: Coefficients of the model
\item \code{summary()}: Summary of the model
}
}
\description{
S3 methods for the classes \code{ermod_*}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dev_ermod_lin.R
\name{dev_ermod_bin_exp_sel}
\alias{dev_ermod_bin_exp_sel}
\alias{dev_ermod_lin_exp_sel}
\title{Exposure metrics selection for linear ER models}
\usage{
dev_ermod_bin_exp_sel(
  data,
  var_resp,
  var_exp_candidates,
  prior = rstanarm::default_prior_coef(stats::binomial()),
  prior_intercept = rstanarm::default_prior_intercept(stats::binomial()),
  verbosity_level = 1,
  chains = 4,
  iter = 2000
)

dev_ermod_lin_exp_sel(
  data,
  var_resp,
  var_exp_candidates,
  prior = rstanarm::default_prior_coef(stats::binomial()),
  prior_intercept = rstanarm::default_prior_intercept(stats::binomial()),
  prior_aux = rstanarm::exponential(autoscale = TRUE),
  verbosity_level = 1,
  chains = 4,
  iter = 2000
)
}
\arguments{
\item{data}{Input data for E-R analysis}

\item{var_resp}{Response variable name in character}

\item{var_exp_candidates}{Candidate exposure variable names
in character vector}

\item{prior, prior_intercept, prior_aux}{See \code{\link[rstanarm:stan_glm]{rstanarm::stan_glm()}}}

\item{verbosity_level}{Verbosity level. 0: No output, 1: Display steps,
2: Display progress in each step, 3: Display MCMC sampling.}

\item{chains}{Number of chains for Stan.}

\item{iter}{Number of iterations for Stan.}
}
\value{
An object of class \code{ermod_bin_exp_sel}.or \code{ermod_lin_exp_sel}
}
\description{
This functions is used to develop an linear ER model with binary
and continuous endpoint, using various exposure metrics and
selecting the best one.
}
\examples{
\donttest{
data(d_sim_binom_cov_hgly2)

ermod_bin_exp_sel <-
  dev_ermod_bin_exp_sel(
    data = d_sim_binom_cov_hgly2,
    var_resp = "AEFLAG",
    var_exp_candidates = c("AUCss_1000", "Cmaxss", "Cminss")
  )

ermod_bin_exp_sel
}

\donttest{
data(d_sim_lin)

ermod_lin_exp_sel <- dev_ermod_lin_exp_sel(
  data = d_sim_lin,
  var_resp = "response",
  var_exp_candidates = c("AUCss", "Cmaxss")
)

ermod_lin_exp_sel
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dev_ermod_lin.R
\name{dev_ermod_bin_cov_sel}
\alias{dev_ermod_bin_cov_sel}
\alias{dev_ermod_lin_cov_sel}
\title{Perform covariate selection for linear ER model}
\usage{
dev_ermod_bin_cov_sel(
  data,
  var_resp,
  var_exposure,
  var_cov_candidates,
  cv_method = c("LOO", "kfold"),
  k = 5,
  validate_search = FALSE,
  nterms_max = NULL,
  .reduce_obj_size = TRUE,
  prior = rstanarm::default_prior_coef(stats::binomial()),
  prior_intercept = rstanarm::default_prior_intercept(stats::binomial()),
  verbosity_level = 1,
  chains = 4,
  iter = 2000
)

dev_ermod_lin_cov_sel(
  data,
  var_resp,
  var_exposure,
  var_cov_candidates,
  cv_method = c("LOO", "kfold"),
  k = 5,
  validate_search = FALSE,
  nterms_max = NULL,
  .reduce_obj_size = TRUE,
  prior = rstanarm::default_prior_coef(stats::binomial()),
  prior_intercept = rstanarm::default_prior_intercept(stats::binomial()),
  prior_aux = rstanarm::exponential(autoscale = TRUE),
  verbosity_level = 1,
  chains = 4,
  iter = 2000
)
}
\arguments{
\item{data}{Input data for E-R analysis}

\item{var_resp}{Response variable name in character}

\item{var_exposure}{Exposure variable names in character}

\item{var_cov_candidates}{Candidate covariate names in character vector}

\item{cv_method}{Cross-validation method. Default is "LOO" (recommended).
Use "kfold" if you see warnings on Pareto k estimates.}

\item{k}{Number of folds for kfold CV.
Only used if cv_method is "kfold".}

\item{validate_search}{Whether to validate the search. Default is FALSE.
Recommend to set to TRUE for kfold CV. Do not use for LOO (run time would
become too long).}

\item{nterms_max}{Maximum number of terms to consider in the model.
Default is NULL (all terms are considered).}

\item{.reduce_obj_size}{Whether to reduce object size by removing some
elements from projpred outputs that are not necessary for the functionality
of this package.}

\item{prior, prior_intercept, prior_aux}{See \code{\link[rstanarm:stan_glm]{rstanarm::stan_glm()}}}

\item{verbosity_level}{Verbosity level. 0: No output, 1: Display steps,
2: Display progress in each step, 3: Display MCMC sampling.}

\item{chains}{Number of chains for Stan.}

\item{iter}{Number of iterations for Stan.}
}
\value{
An object of class \code{ermod_bin_cov_sel} or \code{ermod_lin_cov_sel}.
}
\description{
This functions is used to develop an ER model with covariates for binary
and continuous endpoints.
\code{projpred} package is used for variable selection.
}
\examples{
\dontshow{if (BayesERtools:::.if_run_ex_covsel()) withAutoprint(\{ # examplesIf}
\donttest{
data(d_sim_binom_cov_hgly2)

er_binary_cov_model <- dev_ermod_bin_cov_sel(
  data = d_sim_binom_cov_hgly2,
  var_resp = "AEFLAG",
  var_exposure = "AUCss_1000",
  var_cov_candidates = c(
    "BAGE_10", "BWT_10", "BGLUC",
    "BHBA1C_5", "RACE", "VISC"
  )
)

er_binary_cov_model
}
\dontshow{\}) # examplesIf}
\dontshow{if (BayesERtools:::.if_run_ex_covsel()) withAutoprint(\{ # examplesIf}
\donttest{
data(d_sim_lin)

ermod_lin_cov_sel <- dev_ermod_lin_cov_sel(
  data = d_sim_lin,
  var_resp = "response",
  var_exposure = "AUCss",
  var_cov_candidates = c("BAGE", "SEX")
)

ermod_lin_cov_sel
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/yyy.R
\docType{data}
\name{d_sim_placebo}
\alias{d_sim_placebo}
\title{Sample simulated data for Emax exposure-response models with covariates and placebo}
\format{
A data frame with columns:
\describe{
\item{id}{Subject identifier}
\item{dose}{Nominal dose, units not specified}
\item{exp_1}{Exposure value on metric 1, units and metric not specified}
\item{exp_2}{Exposure value on metric 2, units and metric not specified}
\item{rsp_1}{Continuous response value (units not specified)}
\item{rsp_2}{Binary response value (group labels not specified)}
\item{cnt_a}{Continuous valued covariate}
\item{cnt_b}{Continuous valued covariate}
\item{cnt_c}{Continuous valued covariate}
\item{bin_d}{Binary valued covariate}
\item{bin_e}{Binary valued covariate}
}
}
\usage{
d_sim_placebo
}
\description{
Sample simulated data for Emax exposure-response models with covariates and placebo
}
\details{
This simulated dataset is entirely synthetic. It is a generic data set that can be used
to illustrate Emax modeling with placebo data. It contains variables corresponding to
dose and exposure, and includes both a continuous response variable and a binary response
variable. Three continuous valued covariates are included, along with two binary covariates.
Data from two exposure metrics are included.

You can find the data generating code in the package source code,
under \code{data-raw/d_sim_placebo.R}.
}
\examples{
d_sim_placebo
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coveff.R
\name{sim_coveff}
\alias{sim_coveff}
\title{Perform simulation of covariate effects for ER model}
\usage{
sim_coveff(
  ermod,
  data = NULL,
  spec_coveff = NULL,
  output_type = "median_qi",
  qi_width = 0.9,
  qi_width_cov = 0.9
)
}
\arguments{
\item{ermod}{an object of class \code{ermod} (supports \code{ermod_bin} and \code{ermod_lin})}

\item{data}{an optional data frame to derive the covariate values for
forest plots. If NULL (default), the data used to fit the model is used.}

\item{spec_coveff}{you can supply spec_coveff to \code{\link[=sim_coveff]{sim_coveff()}} or
\code{\link[=plot_coveff]{plot_coveff()}}, if you have already built it manually or with
\code{\link[=build_spec_coveff]{build_spec_coveff()}}. See \code{\link[=build_spec_coveff]{build_spec_coveff()}} for detail.}

\item{output_type}{Type of output. Currently only supports "median_qi"
which returns the median and quantile interval.}

\item{qi_width}{the width of the credible interval on the covariate effect.
This translate to the width of the error bars in the forest plot.}

\item{qi_width_cov}{the width of the quantile interval for continuous
covariates in the forest plot. Default is 0.9 (i.e. visualize effect of
covariate effect at their 5th and 95th percentile values).}
}
\value{
A data frame with class \code{coveffsim} containing the median and
quantile interval of the covariate effects. For binary models (\code{ermod_bin}),
returns odds ratios. For linear models (\code{ermod_lin}), returns response differences.
}
\description{
Perform simulation of covariate effects for ER model
}
\examples{
\dontshow{if (BayesERtools:::.if_run_ex_coveff()) withAutoprint(\{ # examplesIf}
\donttest{
data(d_sim_binom_cov_hgly2)

ermod_bin <- dev_ermod_bin(
  data = d_sim_binom_cov_hgly2,
  var_resp = "AEFLAG",
  var_exposure = "AUCss_1000",
  var_cov = "BHBA1C_5",
)

sim_coveff(ermod_bin)

# Linear regression model example
data(d_sim_lin)

ermod_lin <- dev_ermod_lin(
  data = d_sim_lin,
  var_resp = "response",
  var_exposure = "AUCss",
  var_cov = c("SEX", "BAGE"),
)

sim_coveff(ermod_lin)
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dev_ermod_emax.R
\name{dev_ermod_emax_exp_sel}
\alias{dev_ermod_emax_exp_sel}
\alias{dev_ermod_bin_emax_exp_sel}
\title{Exposure metrics selection for Emax models}
\usage{
dev_ermod_emax_exp_sel(
  data,
  var_resp,
  var_exp_candidates,
  verbosity_level = 1,
  chains = 4,
  iter = 2000,
  gamma_fix = 1,
  e0_fix = NULL,
  emax_fix = NULL,
  priors = NULL,
  seed = sample.int(.Machine$integer.max, 1)
)

dev_ermod_bin_emax_exp_sel(
  data,
  var_resp,
  var_exp_candidates,
  verbosity_level = 1,
  chains = 4,
  iter = 2000,
  gamma_fix = 1,
  e0_fix = NULL,
  emax_fix = NULL,
  priors = NULL,
  seed = sample.int(.Machine$integer.max, 1)
)
}
\arguments{
\item{data}{Input data for E-R analysis}

\item{var_resp}{Response variable name in character}

\item{var_exp_candidates}{Candidate exposure variable names
in character vector}

\item{verbosity_level}{Verbosity level. 0: No output, 1: Display steps,
2: Display progress in each step, 3: Display MCMC sampling.}

\item{chains}{Number of chains for Stan.}

\item{iter}{Number of iterations for Stan.}

\item{gamma_fix}{Hill coefficient, default fixed to 1.
See details in \code{\link[rstanemax:stan_emax]{rstanemax::stan_emax()}} or \code{\link[rstanemax:stan_emax_binary]{rstanemax::stan_emax_binary()}}}

\item{e0_fix}{See details in \code{\link[rstanemax:stan_emax]{rstanemax::stan_emax()}} or
\code{\link[rstanemax:stan_emax_binary]{rstanemax::stan_emax_binary()}}}

\item{emax_fix}{See details in \code{\link[rstanemax:stan_emax]{rstanemax::stan_emax()}} or
\code{\link[rstanemax:stan_emax_binary]{rstanemax::stan_emax_binary()}}}

\item{priors}{See details in \code{\link[rstanemax:stan_emax]{rstanemax::stan_emax()}} or
\code{\link[rstanemax:stan_emax_binary]{rstanemax::stan_emax_binary()}}}

\item{seed}{Random seed for Stan model execution, see details in
\code{\link[rstan:stanmodel-method-sampling]{rstan::sampling()}} which is used in \code{\link[rstanemax:stan_emax]{rstanemax::stan_emax()}} or
\code{\link[rstanemax:stan_emax_binary]{rstanemax::stan_emax_binary()}}}
}
\value{
An object of class \code{ermod_emax_exp_sel} or \code{ermod_bin_emax_exp_sel}.
}
\description{
This functions is used to develop an Emax model with binary and continuous
endpoint, using various exposure metrics and selecting the best one.
}
\examples{
\donttest{
data_er_cont <- rstanemax::exposure.response.sample
noise <- 1 + 0.5 * stats::rnorm(length(data_er_cont$exposure))
data_er_cont$exposure2 <- data_er_cont$exposure * noise
# Replace exposure < 0 with 0
data_er_cont$exposure2[data_er_cont$exposure2 < 0] <- 0

ermod_emax_exp_sel <-
  dev_ermod_emax_exp_sel(
    data = data_er_cont,
    var_resp = "response",
    var_exp_candidates = c("exposure", "exposure2")
  )

ermod_emax_exp_sel
}

\donttest{
data_er_bin <- rstanemax::exposure.response.sample.binary

noise <- 1 + 0.5 * stats::rnorm(length(data_er_bin$conc))
data_er_bin$conc2 <- data_er_bin$conc * noise
data_er_bin$conc2[data_er_bin$conc2 < 0] <- 0

ermod_bin_emax_exp_sel <-
  dev_ermod_bin_emax_exp_sel(
    data = data_er_bin,
    var_resp = "y",
    var_exp_candidates = c("conc", "conc2")
  )
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/yyy.R
\docType{data}
\name{d_sim_lin}
\alias{d_sim_lin}
\title{Sample simulated data for exposure-response with continuous endpoint
using linear model.}
\format{
A data frame with columns:
\describe{
\item{ID}{Subject ID}
\item{AUCss}{Steady-state area under the curve}
\item{Cmaxss}{Steady-state maximum (peak) concentration}
\item{BAGE}{Baseline age in years}
\item{SEX}{M or F}
\item{response}{Response}
}
}
\usage{
d_sim_lin
}
\description{
Sample simulated data for exposure-response with continuous endpoint
using linear model.
}
\details{
True model is defined as \code{0.5 * AUCss + 0.5 * BAGE + 5 * SEX}, with
variability added with standard deviation of 10.
You can find the data generating code in the package source code,
under \code{data-raw/d_sim_lin.R}.
}
\examples{
d_sim_lin
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ermod-methods.R
\name{prior_summary}
\alias{prior_summary}
\alias{prior_summary.ermod}
\title{Summarize the priors used for linear or linear logistic regression models}
\usage{
prior_summary(object, ...)

\method{prior_summary}{ermod}(object, ...)
}
\arguments{
\item{object}{An object of class \code{ermod}}

\item{...}{Additional arguments passed to \code{rstanarm::prior_summary()}}
}
\value{
An object of class \code{prior_summary.stanreg}
}
\description{
See \code{\link[rstanarm:prior_summary.stanreg]{rstanarm::prior_summary()}} for details.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dev_ermod_lin.R
\name{dev_ermod_bin}
\alias{dev_ermod_bin}
\alias{dev_ermod_lin}
\title{Develop linear ER model for binary or continuous endpoint}
\usage{
dev_ermod_bin(
  data,
  var_resp,
  var_exposure,
  var_cov = NULL,
  prior = rstanarm::default_prior_coef(stats::binomial()),
  prior_intercept = rstanarm::default_prior_intercept(stats::binomial()),
  verbosity_level = 1,
  chains = 4,
  iter = 2000
)

dev_ermod_lin(
  data,
  var_resp,
  var_exposure,
  var_cov = NULL,
  prior = rstanarm::default_prior_coef(stats::binomial()),
  prior_intercept = rstanarm::default_prior_intercept(stats::binomial()),
  prior_aux = rstanarm::exponential(autoscale = TRUE),
  verbosity_level = 1,
  chains = 4,
  iter = 2000
)
}
\arguments{
\item{data}{Input data for E-R analysis}

\item{var_resp}{Response variable name in character}

\item{var_exposure}{Exposure variable names in character}

\item{var_cov}{Covariate variable names in character vector}

\item{prior, prior_intercept, prior_aux}{See \code{\link[rstanarm:stan_glm]{rstanarm::stan_glm()}}}

\item{verbosity_level}{Verbosity level. 0: No output, 1: Display steps,
2: Display progress in each step, 3: Display MCMC sampling.}

\item{chains}{Number of chains for Stan.}

\item{iter}{Number of iterations for Stan.}
}
\value{
An object of class \code{ermod_bin} or \code{ermod_lin}.
}
\description{
These functions are used to develop an linear ER model with binary
(\code{\link[=dev_ermod_bin]{dev_ermod_bin()}}) or continuous (\code{\link[=dev_ermod_lin]{dev_ermod_lin()}}) endpoint.
You can also specify covariates to be included in the model.
}
\examples{
\donttest{
data(d_sim_binom_cov_hgly2)

ermod_bin <- dev_ermod_bin(
  data = d_sim_binom_cov_hgly2,
  var_resp = "AEFLAG",
  var_exposure = "AUCss_1000",
  var_cov = "BHBA1C_5",
)

ermod_bin
}

\donttest{
data(d_sim_lin)

ermod_lin <- dev_ermod_lin(
  data = d_sim_lin,
  var_resp = "response",
  var_exposure = "AUCss",
  var_cov = c("SEX", "BAGE")
)

ermod_lin
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/s3_generics.R
\name{extract_method}
\alias{extract_method}
\alias{extract_data}
\alias{extract_mod}
\alias{extract_var_resp}
\alias{extract_var_exposure}
\alias{extract_var_cov}
\alias{extract_exp_sel_list_model}
\alias{extract_exp_sel_comp}
\alias{extract_var_selected}
\title{Extract elements from S3 objects}
\usage{
extract_data(x)

extract_mod(x)

extract_var_resp(x)

extract_var_exposure(x)

extract_var_cov(x)

extract_exp_sel_list_model(x)

extract_exp_sel_comp(x)

extract_var_selected(x)
}
\arguments{
\item{x}{An object to extract elements from}
}
\value{
\itemize{
\item \code{\link[=extract_data]{extract_data()}} extracts data used for the model fit.
\item \code{\link[=extract_mod]{extract_mod()}} extracts the model fit object.
\item \code{\link[=extract_var_resp]{extract_var_resp()}} extracts the response variable name
\item \code{\link[=extract_var_exposure]{extract_var_exposure()}} extracts the exposure metric name
\item \code{\link[=extract_var_cov]{extract_var_cov()}} extracts the covariates name
\item \code{\link[=extract_exp_sel_list_model]{extract_exp_sel_list_model()}} extracts the list of fitted models for
each exposure metrics.
\item \code{\link[=extract_exp_sel_comp]{extract_exp_sel_comp()}} extracts the comparison results of the exposure
metrics.
\item \code{\link[=extract_var_selected]{extract_var_selected()}} extracts the selected variables (both exposure
and covariates)in the final model after covariate selection.
}
}
\description{
S3 methods are defined for \verb{ermod_*} (see \link{extract_ermod}) and
\verb{ersim_*} (see \link{extract_ersim}) classes.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dev_ermod_emax.R
\name{dev_ermod_emax}
\alias{dev_ermod_emax}
\alias{dev_ermod_bin_emax}
\title{Develop Emax model for continuous and binary endpoint}
\usage{
dev_ermod_emax(
  data,
  var_resp,
  var_exposure,
  l_var_cov = NULL,
  gamma_fix = 1,
  e0_fix = NULL,
  emax_fix = NULL,
  priors = NULL,
  verbosity_level = 1,
  chains = 4,
  iter = 2000,
  seed = sample.int(.Machine$integer.max, 1)
)

dev_ermod_bin_emax(
  data,
  var_resp,
  var_exposure,
  l_var_cov = NULL,
  gamma_fix = 1,
  e0_fix = NULL,
  emax_fix = NULL,
  priors = NULL,
  verbosity_level = 1,
  chains = 4,
  iter = 2000,
  seed = sample.int(.Machine$integer.max, 1)
)
}
\arguments{
\item{data}{Input data for E-R analysis}

\item{var_resp}{Response variable name in character}

\item{var_exposure}{Exposure variable names in character}

\item{l_var_cov}{a names list of categorical covariate variables in
character vector. See details in the \code{param.cov} argument of
\code{\link[rstanemax:stan_emax]{rstanemax::stan_emax()}} or \code{\link[rstanemax:stan_emax_binary]{rstanemax::stan_emax_binary()}}}

\item{gamma_fix}{Hill coefficient, default fixed to 1.
See details in \code{\link[rstanemax:stan_emax]{rstanemax::stan_emax()}} or \code{\link[rstanemax:stan_emax_binary]{rstanemax::stan_emax_binary()}}}

\item{e0_fix}{See details in \code{\link[rstanemax:stan_emax]{rstanemax::stan_emax()}} or
\code{\link[rstanemax:stan_emax_binary]{rstanemax::stan_emax_binary()}}}

\item{emax_fix}{See details in \code{\link[rstanemax:stan_emax]{rstanemax::stan_emax()}} or
\code{\link[rstanemax:stan_emax_binary]{rstanemax::stan_emax_binary()}}}

\item{priors}{See details in \code{\link[rstanemax:stan_emax]{rstanemax::stan_emax()}} or
\code{\link[rstanemax:stan_emax_binary]{rstanemax::stan_emax_binary()}}}

\item{verbosity_level}{Verbosity level. 0: No output, 1: Display steps,
2: Display progress in each step, 3: Display MCMC sampling.}

\item{chains}{Number of chains for Stan.}

\item{iter}{Number of iterations for Stan.}

\item{seed}{Random seed for Stan model execution, see details in
\code{\link[rstan:stanmodel-method-sampling]{rstan::sampling()}} which is used in \code{\link[rstanemax:stan_emax]{rstanemax::stan_emax()}} or
\code{\link[rstanemax:stan_emax_binary]{rstanemax::stan_emax_binary()}}}
}
\value{
An object of class \code{ermod_emax}.or \code{ermod_bin_emax}.
}
\description{
These functions are used to develop an Emax model with continuous or binary
endpoint.
You can also specify covariates to be included in the model; note that only
categorical covariates are allowed.
}
\examples{
\dontshow{if (BayesERtools:::.if_run_ex_plot_er()) withAutoprint(\{ # examplesIf}
\donttest{
data_er_cont <- rstanemax::exposure.response.sample

ermod_emax <-
  dev_ermod_emax(
    data = data_er_cont,
    var_exposure = "exposure",
    var_resp = "response"
  )

plot_er(ermod_emax, show_orig_data = TRUE)

data_er_cont_cov <- rstanemax::exposure.response.sample.with.cov

ermod_emax_w_cov <-
  dev_ermod_emax(
    data = data_er_cont_cov,
    var_exposure = "conc",
    var_resp = "resp",
    l_var_cov = list(emax = "cov2", ec50 = "cov3", e0 = "cov1")
  )
}
\dontshow{\}) # examplesIf}
\dontshow{if (BayesERtools:::.if_run_ex_plot_er()) withAutoprint(\{ # examplesIf}
\donttest{
data_er_bin <- rstanemax::exposure.response.sample.binary

ermod_bin_emax <-
  dev_ermod_bin_emax(
    data = data_er_bin,
    var_exposure = "conc",
    var_resp = "y"
  )

plot_er(ermod_bin_emax, show_orig_data = TRUE)

ermod_bin_emax_w_cov <-
  dev_ermod_bin_emax(
    data = data_er_bin,
    var_exposure = "conc",
    var_resp = "y_cov",
    l_var_cov = list(emax = "sex")
  )
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/yyy.R
\docType{data}
\name{d_sim_binom_cov}
\alias{d_sim_binom_cov}
\alias{d_sim_binom_cov_hgly2}
\title{Sample simulated data for exposure-response with binary endpoint.}
\format{
A data frame with columns:
\describe{
\item{ID}{Subject ID}
\item{AETYPE}{Adverse event type: hgly2 (Gr2+ hyperglycemia),
dr2 (Gr2+ Diarrhea), ae_covsel_test (hypothetical AE for covariate
selection function test)}
\item{AEFLAG}{Adverse event flag: 0 - no event, 1 - event}
\item{Dose_mg}{Dose in mg: 200, 400}
\item{AUCss}{Steady-state area under the curve}
\item{Cmaxss}{Steady-state maximum (peak) concentration}
\item{Cminss}{Steady-state minimum (trough) concentration}
\item{BAGE}{Baseline age in years}
\item{BWT}{Baseline weight in kg}
\item{BGLUC}{Baseline glucose in mmol/L}
\item{BHBA1C}{Baseline HbA1c in percentage}
\item{RACE}{Race: White, Black, Asian}
\item{VISC}{Visceral disease: No, Yes}
\item{AUCss_1000}{AUCss/1000}
\item{BAGE_10}{BAGE/10}
\item{BWT_10}{BWT/10}
\item{BHBA1C_5}{BHBA1C/5}
}

An object of class \code{tbl_df} (inherits from \code{tbl}, \code{data.frame}) with 500 rows and 17 columns.
}
\usage{
d_sim_binom_cov

d_sim_binom_cov_hgly2
}
\description{
Sample simulated data for exposure-response with binary endpoint.
}
\details{
This simulated dataset is very loosely inspired by ER analysis of
ipatasertib by Kotani (2022) at:

https://doi.org/10.1007/s00280-022-04488-2

You can find the data generating code in the package source code,
under \code{data-raw/d_sim_binom_cov.R}.

d_sim_binom_cov_hgly2 is a subset of this dataset with only hgly2 AE type
and some columns added for testing.
}
\examples{
d_sim_binom_cov
d_sim_binom_cov_hgly2
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_ermod_exp_cov_sel.R
\name{plot_er_exp_sel}
\alias{plot_er_exp_sel}
\title{Plot exposure metric selection comparison}
\usage{
plot_er_exp_sel(x, n_draws_sim = NULL)
}
\arguments{
\item{x}{An object of class \code{ermod_bin_exp_sel}}

\item{n_draws_sim}{Number of draws to simulate response for each exposure
value. Default is NULL (use all draws in the model object)}
}
\value{
No return value, called for plotting side effect.
}
\description{
Plot ER curve for each exposure metric and compare them.
}
\examples{
\dontshow{if (BayesERtools:::.if_run_ex_plot_er()) withAutoprint(\{ # examplesIf}
\donttest{
data(d_sim_binom_cov_hgly2)

ermod_bin_exp_sel <-
  dev_ermod_bin_exp_sel(
    data = d_sim_binom_cov_hgly2,
    var_resp = "AEFLAG",
    var_exp_candidates = c("AUCss_1000", "Cmaxss", "Cminss")
  )

plot_er_exp_sel(ermod_bin_exp_sel) +
  xgxr::xgx_scale_x_log10(guide = ggplot2::guide_axis(minor.ticks = TRUE))
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coveff.R
\name{print_coveff}
\alias{print_coveff}
\title{Format the covariate effect simulation results for printing}
\usage{
print_coveff(
  coveffsim,
  n_sigfig = 3,
  use_seps = TRUE,
  drop_trailing_dec_mark = TRUE
)
}
\arguments{
\item{coveffsim}{an object of class \code{coveffsim}}

\item{n_sigfig}{Number of significant figures to form value_label of
continuous variables. See \code{\link[gt:vec_fmt_number]{gt::vec_fmt_number()}} for details.}

\item{use_seps}{Whether to use separators for thousands in printing numbers.
See \code{\link[gt:vec_fmt_number]{gt::vec_fmt_number()}} for details.}

\item{drop_trailing_dec_mark}{Whether to drop the trailing decimal mark
(".") in value_label of continuous variables. See \code{\link[gt:vec_fmt_number]{gt::vec_fmt_number()}}
for details.}
}
\value{
A data frame with the formatted covariate effect simulation results
with the following columns:
\itemize{
\item \code{var_label}: the label of the covariate
\item \code{value_label}: the label of the covariate value
\item \code{value_annot}: the annotation of the covariate value
\item \verb{Odds ratio} or \verb{Response difference}: the odds ratio (for binary models) or
response difference (for linear models) of the covariate effect
\item \verb{95\% CI}: the 95\% credible interval of the covariate effect
}
}
\description{
Format the covariate effect simulation results for printing
}
\details{
Note that \code{n_sigfig}, \code{use_seps}, and \code{drop_trailing_dec_mark} are only
applied to the response difference/odds ratio and 95\% CI columns; value_label column was
already generated in an earlier step in \code{\link[=build_spec_coveff]{build_spec_coveff()}} or
\code{\link[=sim_coveff]{sim_coveff()}}.
}
\examples{
\dontshow{if (BayesERtools:::.if_run_ex_coveff()) withAutoprint(\{ # examplesIf}
\donttest{
data(d_sim_binom_cov_hgly2)

ermod_bin <- dev_ermod_bin(
  data = d_sim_binom_cov_hgly2,
  var_resp = "AEFLAG",
  var_exposure = "AUCss_1000",
  var_cov = "BHBA1C_5",
)

print_coveff(sim_coveff(ermod_bin))

# Linear regression model example
data(d_sim_lin)

ermod_lin <- dev_ermod_lin(
  data = d_sim_lin,
  var_resp = "response",
  var_exposure = "AUCss",
  var_cov = c("SEX", "BAGE"),
)

print_coveff(sim_coveff(ermod_lin))
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ermod-methods.R
\name{extract_coef_exp_ci}
\alias{extract_coef_exp_ci}
\title{Extract credible interval of the exposure coefficient}
\usage{
extract_coef_exp_ci(x, ci_width = 0.95, exp_candidates = FALSE)
}
\arguments{
\item{x}{An object of class \code{ermod_bin} or \code{ermod_lin}}

\item{ci_width}{Width of the credible interval}

\item{exp_candidates}{Logical, whether to extract the credible interval for
all exposure candidates. Default is \code{FALSE}. Only supported for models with
exposure selection, created with \code{\link[=dev_ermod_bin_exp_sel]{dev_ermod_bin_exp_sel()}} or
\code{\link[=dev_ermod_lin_exp_sel]{dev_ermod_lin_exp_sel()}} functions.}
}
\value{
A named vector of length 2 with the lower and upper bounds of the
credible interval (.lower, .upper). If \code{exp_candidates = TRUE}, a matrix
with the same structure is returned, with each row corresponding to an
exposure candidate.
}
\description{
Extract credible interval of the exposure coefficient
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dev_ermod_lin.R
\name{dev_ermod_bin_cov_functions}
\alias{dev_ermod_bin_cov_functions}
\alias{.dev_ermod_refmodel}
\alias{.select_cov_projpred}
\title{Internal functions for developing an ER model with covariates
for binary endpoint}
\usage{
.dev_ermod_refmodel(
  data,
  var_resp,
  var_exposure,
  var_cov_candidates,
  verbosity_level = 1,
  chains = 4,
  iter = 2000,
  fun_family = quote(stats::binomial()),
  prior = rstanarm::default_prior_coef(stats::binomial()),
  prior_intercept = rstanarm::default_prior_intercept(stats::binomial()),
  prior_aux = rstanarm::exponential(autoscale = TRUE)
)

.select_cov_projpred(
  refm_obj,
  var_exposure,
  var_cov_candidates,
  nterms_max = NULL,
  cv_method = c("LOO", "kfold"),
  k = 5,
  .reduce_obj_size = TRUE,
  validate_search = FALSE,
  verbosity_level = 1
)
}
\arguments{
\item{data}{Input data for E-R analysis}

\item{var_resp}{Response variable name in character}

\item{var_exposure}{Exposure variable names in character}

\item{var_cov_candidates}{Candidate covariate names in character vector}

\item{verbosity_level}{Verbosity level. 0: No output, 1: Display steps,
2: Display progress in each step, 3: Display MCMC sampling.}

\item{chains}{Number of chains for Stan.}

\item{iter}{Number of iterations for Stan.}

\item{fun_family}{Family function for the model. Default is binomial.}

\item{prior, prior_intercept, prior_aux}{See \code{\link[rstanarm:stan_glm]{rstanarm::stan_glm()}}}

\item{refm_obj}{Reference model object used for variable selection}

\item{nterms_max}{Maximum number of terms to consider in the model.
Default is NULL (all terms are considered).}

\item{cv_method}{Cross-validation method. Default is "LOO" (recommended).
Use "kfold" if you see warnings on Pareto k estimates.}

\item{k}{Number of folds for kfold CV.
Only used if cv_method is "kfold".}

\item{.reduce_obj_size}{Whether to reduce object size by removing some
elements from projpred outputs that are not necessary for the functionality
of this package.}

\item{validate_search}{Whether to validate the search. Default is FALSE.
Recommend to set to TRUE for kfold CV. Do not use for LOO (run time would
become too long).}
}
\value{
\code{\link[=.dev_ermod_refmodel]{.dev_ermod_refmodel()}}: The reference model object that can be used
for variable selection.

\code{\link[=.select_cov_projpred]{.select_cov_projpred()}}: The selected variables
}
\description{
These functions are not intended to be used directly by users.
}
\details{
\code{\link[=.dev_ermod_refmodel]{.dev_ermod_refmodel()}} is used to fit the refmodel (full reference
model) necessary for projpred

\code{\link[=.select_cov_projpred]{.select_cov_projpred()}} is used to select variables with \code{projpred} package
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build_spec_coveff.R
\name{build_spec_coveff}
\alias{build_spec_coveff}
\title{Build specifications for covariate effect simulation/visualization}
\usage{
build_spec_coveff(
  ermod,
  data = NULL,
  qi_width_cov = 0.9,
  n_sigfig = 3,
  use_seps = TRUE,
  drop_trailing_dec_mark = TRUE
)
}
\arguments{
\item{ermod}{an object of class \code{ermod} (supports \code{ermod_bin} and \code{ermod_lin})}

\item{data}{an optional data frame to derive the covariate values for
forest plots. If NULL (default), the data used to fit the model is used.}

\item{qi_width_cov}{the width of the quantile interval for continuous
covariates in the forest plot. Default is 0.9 (i.e. visualize effect of
covariate effect at their 5th and 95th percentile values).}

\item{n_sigfig}{Number of significant figures to form value_label of
continuous variables. See \code{\link[gt:vec_fmt_number]{gt::vec_fmt_number()}} for details.}

\item{use_seps}{Whether to use separators for thousands in printing numbers.
See \code{\link[gt:vec_fmt_number]{gt::vec_fmt_number()}} for details.}

\item{drop_trailing_dec_mark}{Whether to drop the trailing decimal mark
(".") in value_label of continuous variables. See \code{\link[gt:vec_fmt_number]{gt::vec_fmt_number()}}
for details.}
}
\value{
\code{spec_coveff} (return object) is a data frame for the specification
of the covariate effects to be visualized. This is internally generated by
\code{\link[=build_spec_coveff]{build_spec_coveff()}} if you run \code{\link[=sim_coveff]{sim_coveff()}} or \code{\link[=plot_coveff]{plot_coveff()}}
directly. Alternatively, you can develop your own or modify the one
generated by \code{\link[=build_spec_coveff]{build_spec_coveff()}} and supply it to \code{\link[=sim_coveff]{sim_coveff()}} or
\code{\link[=plot_coveff]{plot_coveff()}}. The data frame should have the following columns (but
it's probably easier to try \code{\link[=build_spec_coveff]{build_spec_coveff()}} and see the structure):
\itemize{
\item \code{var_order}: The order of the covariate in the forest plot. The exposure
variable is always the first one and the covariates are ordered by the
order they are supplied in the \code{var_cov} argument of the \verb{dev_ermod_*}
function. If you used a model from \code{\link[=dev_ermod_bin_cov_sel]{dev_ermod_bin_cov_sel()}}, then the
order is determined by the variable selection process.
\item \code{var_name}: The name of the variable.
\item \code{var_label}: The label of the variable to be used for plot.
This is the same as \code{var_name} by default.
\item \code{value_order}: The order of the value of the variable to be evaluated.
\item \code{value_annot}: The annotation of the value of the variable to be
evaluated. This appears on the right hand side of the forest plot.
\item \code{value_label}: The label of the value of the variable to be evaluated.
\item \code{value_cont}: The value for continuous variables.
\item \code{value_cat}: The value for categorical variables.
\item \code{is_ref_value}: Whether the value is the reference value.
\item \code{show_ref_value}: Whether to show the reference value in the plot and
table. This is TRUE by default for is_ref_value == TRUE, otherwise NA
(and ignored).
\item \code{is_covariate}: Whether the variable is a covariate (TRUE) or exposure
variable (FALSE).
}
}
\description{
Build specifications for covariate effect simulation/visualization
}
\examples{
\dontshow{if (BayesERtools:::.if_run_ex_coveff()) withAutoprint(\{ # examplesIf}
\donttest{
data(d_sim_binom_cov_hgly2)

ermod_bin <- dev_ermod_bin(
  data = d_sim_binom_cov_hgly2,
  var_resp = "AEFLAG",
  var_exposure = "AUCss_1000",
  var_cov = c("BHBA1C_5", "RACE"),
)

spec_coveff <- build_spec_coveff(ermod_bin)
plot_coveff(ermod_bin, spec_coveff = spec_coveff)
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loo_kfold.R
\name{loo}
\alias{loo}
\alias{loo.ermod}
\alias{loo.ermod_emax}
\alias{loo.ermod_bin_emax}
\title{Efficient approximate leave-one-out cross-validation (LOO)}
\usage{
loo(x, ...)

\method{loo}{ermod}(x, ...)

\method{loo}{ermod_emax}(x, ...)

\method{loo}{ermod_bin_emax}(x, ...)
}
\arguments{
\item{x}{An object of class \code{ermod}}

\item{...}{Additional arguments passed to \code{loo::loo()}}
}
\value{
An object of class \code{loo}
}
\description{
See \code{\link[loo:loo]{loo::loo()}} for details.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ersim-methods.R
\name{extract_ersim}
\alias{extract_ersim}
\alias{extract_data.ersim}
\alias{extract_data.ersim_med_qi}
\alias{extract_var_resp.ersim}
\alias{extract_var_resp.ersim_med_qi}
\alias{extract_var_exposure.ersim}
\alias{extract_var_exposure.ersim_med_qi}
\alias{extract_var_cov.ersim}
\alias{extract_var_cov.ersim_med_qi}
\title{Extract elements from objects of the classes \verb{ersim_*} and `ersim_med_qi_*``}
\usage{
\method{extract_data}{ersim}(x)

\method{extract_data}{ersim_med_qi}(x)

\method{extract_var_resp}{ersim}(x)

\method{extract_var_resp}{ersim_med_qi}(x)

\method{extract_var_exposure}{ersim}(x)

\method{extract_var_exposure}{ersim_med_qi}(x)

\method{extract_var_cov}{ersim}(x)

\method{extract_var_cov}{ersim_med_qi}(x)
}
\arguments{
\item{x}{An object of class \code{ersim_*}}
}
\value{
\itemize{
\item \code{\link[=extract_data]{extract_data()}} extracts data used for the model fit.
\item \code{\link[=extract_mod]{extract_mod()}} extracts the model fit object.
\item \code{\link[=extract_var_resp]{extract_var_resp()}} extracts the response variable name
\item \code{\link[=extract_var_exposure]{extract_var_exposure()}} extracts the exposure metric name
\item \code{\link[=extract_var_cov]{extract_var_cov()}} extracts the covariates name
\item \code{\link[=extract_exp_sel_list_model]{extract_exp_sel_list_model()}} extracts the list of fitted models for
each exposure metrics.
\item \code{\link[=extract_exp_sel_comp]{extract_exp_sel_comp()}} extracts the comparison results of the exposure
metrics.
\item \code{\link[=extract_var_selected]{extract_var_selected()}} extracts the selected variables (both exposure
and covariates)in the final model after covariate selection.
}
}
\description{
Extract elements from objects of the classes \verb{ersim_*} and `ersim_med_qi_*``
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_ermod.R
\name{combine_er_components}
\alias{combine_er_components}
\title{Combine ER plot components}
\usage{
combine_er_components(
  components,
  heights = NULL,
  add_caption = !is.null(components$caption),
  ...
)
}
\arguments{
\item{components}{An object of class \code{er_plot_components} returned by
\code{plot_er()} or \code{plot_er_gof()} with \code{return_components = TRUE} or
\code{options_orig_data = list(return_components = TRUE)}.}

\item{heights}{Numeric vector of length 2 specifying the relative heights
of the main plot and boxplot. If \code{NULL} (default), uses the height from
the metadata (based on \code{boxplot_height} parameter).}

\item{add_caption}{Logical, whether to add the caption to the combined plot.
Default is \code{TRUE} if caption is available.}

\item{...}{Additional arguments passed to \code{patchwork::wrap_plots()}.}
}
\value{
A combined ggplot object.
}
\description{
Combine ER plot components
}
\examples{
\dontrun{
# Get components
comps <- plot_er_gof(ermod_bin, return_components = TRUE)

# Modify components
comps$main <- comps$main + labs(title = "Custom Title")

# Recombine
combine_er_components(comps)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build_spec_coveff.R
\name{edit_spec_coveff}
\alias{edit_spec_coveff}
\alias{build_spec_coveff_one_variable}
\alias{replace_spec_coveff}
\title{Customize specifications for covariate effect simulations/visualizations}
\usage{
build_spec_coveff_one_variable(
  var_name,
  values_vec,
  var_label = NULL,
  qi_width_cov = 0.9,
  n_sigfig = 3,
  use_seps = TRUE,
  drop_trailing_dec_mark = TRUE,
  show_ref_value = TRUE
)

replace_spec_coveff(spec_orig, spec_new, replace_ref_value = FALSE)
}
\arguments{
\item{var_name}{The name of the variable for which a new spec is to be
created.}

\item{values_vec}{The vector of the values for creating a new spec.}

\item{var_label}{The label of the variable to be used for plot. If NULL
(default), it is set to \code{var_name}.}

\item{qi_width_cov}{the width of the quantile interval for continuous
covariates in the forest plot. Default is 0.9 (i.e. visualize effect of
covariate effect at their 5th and 95th percentile values).}

\item{n_sigfig}{Number of significant figures to form value_label of
continuous variables. See \code{\link[gt:vec_fmt_number]{gt::vec_fmt_number()}} for details.}

\item{use_seps}{Whether to use separators for thousands in printing numbers.
See \code{\link[gt:vec_fmt_number]{gt::vec_fmt_number()}} for details.}

\item{drop_trailing_dec_mark}{Whether to drop the trailing decimal mark
(".") in value_label of continuous variables. See \code{\link[gt:vec_fmt_number]{gt::vec_fmt_number()}}
for details.}

\item{show_ref_value}{Whether to show the reference value in the plot and
table. Setting this results in the \code{show_ref_value} column in the
specification data frame.}

\item{spec_orig}{Original specification data frame.}

\item{spec_new}{New specification data frame. It can be generated by
\code{\link[=build_spec_coveff_one_variable]{build_spec_coveff_one_variable()}} or manually crafting with the
following variables: \code{var_name}, \code{var_label}, \code{value_order}, \code{value_annot},
\code{value_label}, \code{value_cont} or \code{value_cat}, \code{is_ref_value},
\code{show_ref_value}. You can have multiple variables stacked together.}

\item{replace_ref_value}{Whether to replace the reference values from the
original specification data frame. Default is FALSE; in this case,
show_ref_value is set to FALSE as it can be confusing.
If you set replace_ref_value to TRUE,
the reference calculation for the forest plot is also done with the one
in spec_new.}
}
\value{
See \code{\link[=build_spec_coveff]{build_spec_coveff()}} for the structure of the return object.
\code{\link[=build_spec_coveff_one_variable]{build_spec_coveff_one_variable()}} returns a data frame corresponding to
the specification for a single variable, which can be used as an input to
\code{\link[=replace_spec_coveff]{replace_spec_coveff()}}.
}
\description{
\itemize{
\item \code{\link[=build_spec_coveff_one_variable]{build_spec_coveff_one_variable()}} is a helper function to create a new
specification for a single variable. This is useful when you want to
customize the specification for a single variable.
\item \code{\link[=replace_spec_coveff]{replace_spec_coveff()}} is used to replace the specification for some (or
all) variables in the original specification data frame. If you want to
replace multiple variables, you can just stack the specifications
together.
}
}
\examples{
\dontshow{if (BayesERtools:::.if_run_ex_coveff()) withAutoprint(\{ # examplesIf}
\donttest{
set.seed(1234)
data(d_sim_binom_cov_hgly2)

ermod_bin <- suppressWarnings(dev_ermod_bin(
  data = d_sim_binom_cov_hgly2, var_resp = "AEFLAG",
  var_exposure = "AUCss_1000", var_cov = c("BGLUC", "RACE"),
  verbosity_level = 0,
  # Below option to make the example run fast
  chains = 2, iter = 1000
))

spec_coveff <- build_spec_coveff(ermod_bin)
spec_new_bgluc <- build_spec_coveff_one_variable(
  "BGLUC", seq(4, 8, by = 0.1),
  var_label = "Baseline Glucose (mmol/L)",
  qi_width_cov = 0.8, show_ref_value = FALSE
)
spec_coveff_new <- replace_spec_coveff(spec_coveff, spec_new_bgluc)
plot_coveff(ermod_bin, spec_coveff = spec_coveff_new)
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_ermod_exp_cov_sel.R
\name{plot_cov_sel}
\alias{plot_cov_sel}
\alias{plot_submod_performance}
\alias{plot_var_ranking}
\title{Plot variable selection performance}
\usage{
plot_submod_performance(x)

plot_var_ranking(x)
}
\arguments{
\item{x}{An object of class \code{ermod_bin_cov_sel}}
}
\value{
No return value, called for plotting side effect.
}
\description{
Plot variable selection performance
}
\details{
\code{\link[=plot_submod_performance]{plot_submod_performance()}} plots the performance of submodels
evaluated during variable selection.

\code{\link[=plot_var_ranking]{plot_var_ranking()}} plots the variable ranking
evaluated during variable selection.
}
\examples{
\dontshow{if (BayesERtools:::.if_run_ex_covsel()) withAutoprint(\{ # examplesIf}
\donttest{
data(d_sim_binom_cov_hgly2)

er_binary_cov_model_kfold <- dev_ermod_bin_cov_sel(
  data = d_sim_binom_cov_hgly2,
  var_resp = "AEFLAG",
  var_exposure = "AUCss_1000",
  var_cov_candidate = c(
    "BAGE_10", "BWT_10", "BGLUC",
    "BHBA1C_5", "RACE", "VISC"
  ),
  cv_method = "kfold",
  k = 3, # Choose 3 to make the example go fast
  validate_search = TRUE,
)

plot_submod_performance(er_binary_cov_model_kfold)
plot_var_ranking(er_binary_cov_model_kfold)
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/yyy.R
\docType{data}
\name{d_sim_emax}
\alias{d_sim_emax}
\title{Sample simulated data for Emax exposure-response models with covariates.}
\format{
A data frame with columns:
\describe{
\item{dose}{Nominal dose, units not specified}
\item{exposure}{Exposure value, units and metric not specified}
\item{response_1}{Continuous response value (units not specified)}
\item{response_2}{Binary response value (group labels not specified)}
\item{cnt_a}{Continuous valued covariate}
\item{cnt_b}{Continuous valued covariate}
\item{cnt_c}{Continuous valued covariate}
\item{bin_d}{Binary valued covariate}
\item{bin_e}{Binary valued covariate}
}
}
\usage{
d_sim_emax
}
\description{
Sample simulated data for Emax exposure-response models with covariates.
}
\details{
This simulated dataset is entirely synthetic. It is a generic data set that can be used
to illustrate Emax modeling. It contains variables corresponding to dose and exposure,
and includes both a continuous response variable and a binary response variable. Three
continuous valued covariates are included, along with two binary covariates.

You can find the data generating code in the package source code,
under \code{data-raw/d_sim_emax.R}.
}
\examples{
d_sim_emax
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loo_kfold.R
\name{kfold.ermod}
\alias{kfold.ermod}
\alias{kfold}
\alias{extract_kfold_loo}
\title{Run k-fold cross-validation}
\usage{
\method{kfold}{ermod}(x, k = 5, newdata = NULL, seed = NULL, ...)

kfold(x, ...)

extract_kfold_loo(kfold_ermod)
}
\arguments{
\item{x}{An \code{ermod} object containing the model and data.}

\item{k}{The number of folds for cross-validation. Default is 5.}

\item{newdata}{Optional new dataset to use instead of the original data.
Default is NULL.}

\item{seed}{Random seed for reproducibility. Default is NULL.}

\item{...}{Currently not used.}

\item{kfold_ermod}{An object of class \code{kfold_ermod} from \code{\link[=kfold]{kfold()}}}
}
\value{
\code{\link[=kfold]{kfold()}} returns \code{kfold_ermod} class object containing the fitted
models and holdout predictions for each fold.

\code{\link[=extract_kfold_loo]{extract_kfold_loo()}} returns  \code{c("kfold", "loo")} class object
that works well with \code{loo} ecosystem
}
\description{
This function performs k-fold cross-validation using the appropriate model
development function based on the class of the \code{ermod} object.
It is internally used by \code{\link[=eval_ermod]{eval_ermod()}}. The output is compatible with
\code{loo} ecosystem, e.g. it can be used for \code{\link[loo:loo_compare]{loo::loo_compare()}} function.
See \code{\link[loo:kfold-generic]{loo::kfold()}} for details.
}
\examples{
\dontshow{if (BayesERtools:::.if_run_ex_eval_mod()) withAutoprint(\{ # examplesIf}
\donttest{
data(d_sim_binom_cov_hgly2)

ermod_bin <- dev_ermod_bin(
  data = d_sim_binom_cov_hgly2,
  var_resp = "AEFLAG",
  var_exposure = "AUCss_1000",
  var_cov = "BHBA1C_5",
  # Settings to make the example run faster
  chains = 2,
  iter = 1000
)

cv_results <- kfold(ermod_bin, k = 3, seed = 123)

print(cv_results)
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ermod-methods.R
\name{as_draws}
\alias{as_draws}
\alias{as_draws_list}
\alias{as_draws_array}
\alias{as_draws_df}
\alias{as_draws_matrix}
\alias{as_draws_rvars}
\alias{as_draws.ermod}
\alias{as_draws_list.ermod}
\alias{as_draws_array.ermod}
\alias{as_draws_df.ermod}
\alias{as_draws_matrix.ermod}
\alias{as_draws_rvars.ermod}
\title{Transform to \code{draws} objects}
\usage{
as_draws(x, ...)

as_draws_list(x, ...)

as_draws_array(x, ...)

as_draws_df(x, ...)

as_draws_matrix(x, ...)

as_draws_rvars(x, ...)

\method{as_draws}{ermod}(x, ...)

\method{as_draws_list}{ermod}(x, ...)

\method{as_draws_array}{ermod}(x, ...)

\method{as_draws_df}{ermod}(x, ...)

\method{as_draws_matrix}{ermod}(x, ...)

\method{as_draws_rvars}{ermod}(x, ...)
}
\arguments{
\item{x}{An object of class \code{ermod}}

\item{...}{Arguments passed to individual methods (if applicable).}
}
\value{
A draws object from the \code{posterior} package.
}
\description{
See \code{\link[posterior:draws]{posterior::as_draws()}} for details.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ermod-methods.R
\name{ermod_cov_sel_method}
\alias{ermod_cov_sel_method}
\alias{print.ermod_cov_sel}
\alias{plot.ermod_cov_sel}
\title{S3 methods for the classes \code{ermod_bin_cov_sel}}
\usage{
\method{print}{ermod_cov_sel}(x, digits = 2, ...)

\method{plot}{ermod_cov_sel}(x, ...)
}
\arguments{
\item{x}{An object of class \code{ermod_bin_cov_sel}}

\item{digits}{Number of digits to print}

\item{...}{Additional arguments passed to functions}
}
\value{
No return value, called for print or plot side effects
}
\description{
S3 methods for the classes \code{ermod_bin_cov_sel}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eval_ermod.R
\name{eval_ermod}
\alias{eval_ermod}
\title{Evaluate exposure-response model prediction performance}
\usage{
eval_ermod(
  ermod,
  eval_type = c("training", "kfold", "test"),
  newdata = NULL,
  summary_method = c("median", "mean"),
  k = 5,
  seed_kfold = NULL
)
}
\arguments{
\item{ermod}{An object of class \code{ermod}.}

\item{eval_type}{A character string specifying the evaluation dataset.
Options are:
\itemize{
\item \code{training}: Use the training dataset.
\item \code{test}: Use a new dataset for evaluation.
\item \code{kfold}: Perform k-fold cross-validation (uses \code{newdata} if provided,
otherwise uses the training dataset).
}}

\item{newdata}{A data frame containing new data for evaluation when
\code{eval_type} is set to \code{test} or \code{kfold}.}

\item{summary_method}{A character string specifying how to summarize the
simulation draws. Default is \code{median}.}

\item{k}{The number of folds for cross-validation. Default is 5.}

\item{seed_kfold}{Random seed for k-fold cross-validation.}
}
\value{
A tibble with calculated performance metrics, such as AUROC or
RMSE, depending on the model type.
}
\description{
This function evaluates the performance of an exposure-response model
using various metrics.
}
\examples{
\dontshow{if (BayesERtools:::.if_run_ex_eval_mod()) withAutoprint(\{ # examplesIf}
\donttest{
data(d_sim_binom_cov_hgly2)
d_split <- rsample::initial_split(d_sim_binom_cov_hgly2)
d_train <- rsample::training(d_split)
d_test <- rsample::testing(d_split)

ermod_bin <- dev_ermod_bin(
  data = d_train,
  var_resp = "AEFLAG",
  var_exposure = "AUCss_1000",
  var_cov = "BHBA1C_5",
  # Settings to make the example run faster
  chains = 2,
  iter = 1000
)

metrics_training <- eval_ermod(ermod_bin, eval_type = "training")
metrics_test <- eval_ermod(ermod_bin, eval_type = "test", newdata = d_test)
metrics_kfold <- eval_ermod(ermod_bin, eval_type = "kfold", k = 3)

print(metrics_training)
print(metrics_test)
print(metrics_kfold)
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_ermod.R
\name{sim_er_new_exp}
\alias{sim_er_new_exp}
\alias{sim_er_curve}
\title{Simulate from ER model at specified exposure values}
\usage{
sim_er_new_exp(
  ermod,
  exposure_to_sim_vec = NULL,
  data_cov = NULL,
  n_draws_sim = NULL,
  seed_sample_draws = NULL,
  output_type = c("draws", "median_qi"),
  qi_width = 0.95
)

sim_er_curve(
  ermod,
  exposure_range = NULL,
  num_exposures = 51,
  data_cov = NULL,
  n_draws_sim = NULL,
  seed_sample_draws = NULL,
  output_type = c("draws", "median_qi"),
  qi_width = 0.95
)
}
\arguments{
\item{ermod}{An object of class \code{ermod}}

\item{exposure_to_sim_vec}{Vector of exposure values to simulate.}

\item{data_cov}{Data frame containing covariates to use for simulation,
see details below.}

\item{n_draws_sim}{Number of draws for simulation. If NULL (default),
all draws in the model object are used.}

\item{seed_sample_draws}{Seed for sampling draws. Default is NULL.}

\item{output_type}{Type of output. "draws" returns the raw draws from the
simulation, and "median_qi" returns the median and quantile interval.}

\item{qi_width}{Width of the quantile interval. Default is 0.95. Only
used when \code{output_type = "median_qi"}.}

\item{exposure_range}{Range of exposure values to simulate. If NULL
(default), it is set to the range of the exposure variable in the original
data for model development.}

\item{num_exposures}{Number of exposure values to simulate.}
}
\value{
\code{ersim} object, which is a tibble with the simulated responses
with some additional information in object attributes.
It has three types of predictions - \code{.linpred}, \code{.epred}, \code{.prediction}.
\code{.linpred} and \code{.epred} are similar in a way that they both represent
"expected response", i.e. without residual variability. They are the same
for models with continuous endpoits (Emax model). For models with binary
endpoints, \code{.linpred} is the linear predictor (i.e. on the logit scale) and
\code{.epred} is on the probability scale. \code{.prediction} is the predicted
response with residual variability (or in case of binary endpoint,
the predicted yes (1) or no (0) for event occurrence).
See \code{\link[rstantools:posterior_epred]{rstantools::posterior_epred()}} for more details.

In case of \code{output_type = "median_qi"}, it returns \code{ersim_med_qi} object.
}
\description{
Simulate from ER model at specified exposure values
}
\details{
Simulation dataset will be all combinations of covariates in \code{data_cov}
and exposure values in \code{exposure_to_sim_vec},
so the run time can become very long if \code{data_cov} has many rows.

\code{data_cov} has to be supplied if \code{ermod} is a model with covariates.
It is recommended that \code{data_cov} contains subject identifiers such as
\code{ID} for post-processing.

Exposure values in \code{data_cov} will be ignored.

\code{\link[=sim_er_curve]{sim_er_curve()}} is a wrapper function for \code{\link[=sim_er_new_exp]{sim_er_new_exp()}}
that use a range of exposure values to simulate the expected responses.
Particularly useful for plotting the exposure-response curve.
}
\examples{
\donttest{
data(d_sim_binom_cov_hgly2)

ermod_bin <- dev_ermod_bin(
  data = d_sim_binom_cov_hgly2,
  var_resp = "AEFLAG",
  var_exposure = "AUCss_1000",
  var_cov = "BHBA1C_5",
)

ersim_new_exp_med_qi <- sim_er_new_exp(
  ermod_bin,
  exposure_to_sim_vec = seq(2, 6, by = 0.2),
  data_cov = dplyr::tibble(BHBA1C_5 = 4:10),
  n_draws_sim = 500, # This is set to make the example run faster
  output_type = "median_qi"
)

ersim_new_exp_med_qi
}

}
\seealso{
\code{\link[=calc_ersim_med_qi]{calc_ersim_med_qi()}} for calculating median and quantile interval
from \code{ersim} object (generated with \code{output_type = "draws"}).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ersim-methods.R
\name{ersim_method}
\alias{ersim_method}
\alias{plot.ersim}
\alias{plot.ersim_med_qi}
\title{S3 methods for the classes \verb{ersim_*} and \verb{ersim_med_qi_*}}
\usage{
\method{plot}{ersim}(x, show_orig_data = FALSE, ...)

\method{plot}{ersim_med_qi}(x, show_orig_data = FALSE, ...)
}
\arguments{
\item{x}{An object of the classes \verb{ersim_*} or \verb{ersim_med_qi_*}}

\item{show_orig_data}{logical, whether to show the data points in the
model development dataset. Default is \code{FALSE}. Only support plotting
with data that was used in the model development. If you want to use
other data, consider adding geom_point() to the plot manually.}

\item{...}{Additional arguments passed to functions}
}
\value{
No return value, called for print or plot side effects
}
\description{
S3 methods for the classes \verb{ersim_*} and \verb{ersim_med_qi_*}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_ermod.R
\name{sim_er_new_exp_marg}
\alias{sim_er_new_exp_marg}
\alias{sim_er_curve_marg}
\title{Calculate marginal expected response for specified exposure values}
\usage{
sim_er_new_exp_marg(
  ermod,
  exposure_to_sim_vec = NULL,
  data_cov = extract_data(ermod),
  n_subj_sim = 100,
  n_draws_sim = 500,
  seed_sample_draws = NULL,
  output_type = c("draws", "median_qi"),
  qi_width = 0.95
)

sim_er_curve_marg(
  ermod,
  exposure_range = NULL,
  num_exposures = 51,
  data_cov = extract_data(ermod),
  n_subj_sim = 100,
  n_draws_sim = 500,
  seed_sample_draws = NULL,
  output_type = c("draws", "median_qi"),
  qi_width = 0.95
)
}
\arguments{
\item{ermod}{An object of class \code{ermod}}

\item{exposure_to_sim_vec}{Vector of exposure values to simulate.}

\item{data_cov}{Data frame containing covariates to use for simulation.
Different from \code{\link[=sim_er_new_exp]{sim_er_new_exp()}}, \code{data_cov} can be large as long as
\code{n_subj_sim} is set to a reasonable number. Default is set to
\code{extract_data(ermod)} which is the full data used to fit the model.}

\item{n_subj_sim}{Maximum number of subjects to simulate. Default of 100
should be sufficient in many cases, as it's only used for marginal
response calculation.
Set to NULL to use all subjects in \code{data_cov} without resampling;
in this case, be mindful of the computation time.}

\item{n_draws_sim}{Number of draws for simulation. Default is set to 500
to reduce computation time for marginal response calculation.}

\item{seed_sample_draws}{Seed for sampling draws. Default is NULL.}

\item{output_type}{Type of output. "draws" returns the raw draws from the
simulation, and "median_qi" returns the median and quantile interval.}

\item{qi_width}{Width of the quantile interval. Default is 0.95. Only
used when \code{output_type = "median_qi"}.}

\item{exposure_range}{Range of exposure values to simulate. If NULL
(default), it is set to the range of the exposure variable in the original
data for model development.}

\item{num_exposures}{Number of exposure values to simulate.}
}
\value{
\code{ersim_marg} object, which is a tibble with the simulated marginal
expected response with some additional information in object attributes.
In case of \code{output_type = "median_qi"}, it returns \code{ersim_marg_med_qi}
object.
}
\description{
Responses at specified exposure values are calculated for \code{n_subj_sim}
subjects with different covariates (sampled from \code{newdata}),
and the predicted responses are "marginalized" (averaged),
resulting in marginal expected response on the
population of interest.
}
\details{
\code{\link[=sim_er_new_exp_marg]{sim_er_new_exp_marg()}} returns a tibble with the marginal expected
response for each exposure value in \code{exposure_to_sim_vec}.

\code{\link[=sim_er_curve_marg]{sim_er_curve_marg()}} is a wrapper function for \code{\link[=sim_er_new_exp_marg]{sim_er_new_exp_marg()}}
that use a range of exposure values to simulate the marginal expected
responses. Particularly useful for plotting the exposure-response curve.
}
\examples{
\donttest{
data(d_sim_binom_cov_hgly2)

ermod_bin <- dev_ermod_bin(
  data = d_sim_binom_cov_hgly2,
  var_resp = "AEFLAG",
  var_exposure = "AUCss_1000",
  var_cov = "BHBA1C_5",
)

ersim_new_exp_marg_med_qi <- sim_er_new_exp_marg(
  ermod_bin,
  exposure_to_sim_vec = seq(2, 6, by = 0.2),
  data_cov = dplyr::tibble(BHBA1C_5 = 4:10),
  n_subj_sim = NULL,
  n_draws_sim = 500, # This is set to make the example run faster
  output_type = "median_qi"
)

ersim_new_exp_marg_med_qi
}

}
\seealso{
\code{\link[=calc_ersim_med_qi]{calc_ersim_med_qi()}} for calculating median and quantile interval
from \code{ersim_marg} object (generated with \code{output_type = "draws"}).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_ermod.R
\name{plot_er_gof}
\alias{plot_er_gof}
\title{Default GOF plot for ER model}
\usage{
plot_er_gof(
  x,
  add_boxplot = !is.null(var_group),
  boxplot_height = 0.15,
  show_boxplot_y_title = FALSE,
  var_group = NULL,
  n_bins = 4,
  bin_breaks = NULL,
  qi_width_obs = 0.95,
  show_coef_exp = FALSE,
  coef_pos_x = NULL,
  coef_pos_y = NULL,
  coef_size = 4,
  qi_width_coef = 0.95,
  qi_width_sim = 0.95,
  show_caption = TRUE,
  return_components = FALSE
)
}
\arguments{
\item{x}{an object of class \code{ermod}, \code{ersim},\code{ersim_med_qi},
or their subclasses}

\item{add_boxplot}{Logical, whether to add a boxplot of exposure values.
Default is \code{TRUE} if \code{var_group} is specified, otherwise \code{FALSE}.}

\item{boxplot_height}{Height of the boxplot relative to the main plot.
Default is \code{0.15}.}

\item{show_boxplot_y_title}{Logical, whether to show the y-axis title
for the boxplot. Default is \code{FALSE}.}

\item{var_group}{The column to use for grouping data for plotting.
If specified, observed data points and boxplot will be grouped
and colored by this column. Default is \code{NULL}.}

\item{n_bins}{Number of bins to use for observed probability
summary. Only relevant for binary models. Default is \code{4}.}

\item{bin_breaks}{Manually specify bin breaks for binary models. If specified
this overrides \code{n_bins}.}

\item{qi_width_obs}{Confidence level for the observed probability
summary. Default is \code{0.95}.}

\item{show_coef_exp}{Logical, whether to show the credible interval
of the exposure coefficient. Default is \code{FALSE}. This is only available
for linear and linear logistic regression models.}

\item{coef_pos_x}{x-coordinate of the text label. If \code{NULL} (default), it is
set to the minimum value for the exposure variable.}

\item{coef_pos_y}{y-coordinate of the text label. If \code{NULL} (default), it is
set to 0.9 for logistic regression models and the maximum value of the
response variable in the original data for linear regression models.}

\item{coef_size}{Size of the text label. Default is \code{4}.}

\item{qi_width_coef}{Width of the credible interval for the exposure
coefficient. Default is \code{0.95}.}

\item{qi_width_sim}{Width of the quantile interval to summarize simulated
draws. Default is \code{0.95}.}

\item{show_caption}{Logical, whether to show the caption note for the
plot. Default is \code{TRUE}.}

\item{return_components}{Logical, whether to return plot components as a
list instead of a combined plot. When \code{TRUE}, returns a list with
\verb{$main} (main plot), \verb{$boxplot} (boxplot if applicable), \verb{$caption}
(caption text if applicable), and \verb{$metadata}. This allows users to
customize individual plot components before combining them. Default is
\code{FALSE}.}
}
\value{
A ggplot object
}
\description{
This is a wrapper function for \code{\link[=plot_er]{plot_er()}} with default options for
goodness-of-fit (GOF) plots for ER models.
}
\details{
The following code will generate the same plot:
\preformatted{
plot_er(
  x,
  show_orig_data = TRUE,
  show_coef_exp = show_coef_exp,
  show_caption = show_caption,
  options_orig_data = list(
    add_boxplot = add_boxplot, boxplot_height = boxplot_height,
    show_boxplot_y_title = show_boxplot_y_title,
    var_group = var_group,
    n_bins = n_bins, qi_width = qi_width_obs
  ),
  options_coef_exp = list(
    qi_width = qi_width_coef, pos_x = coef_pos_x, pos_y = coef_pos_y,
    size = coef_size
  ),
  options_caption = list(
    orig_data_summary = TRUE, coef_exp = show_coef_exp
  ),
  qi_width_sim = qi_width_sim
)
}

To customize plot elements (titles, shapes, themes), use
\code{return_components = TRUE} to get individual plot components, modify them,
then recombine with \code{combine_er_components()}.
}
\examples{
\dontshow{if (BayesERtools:::.if_run_ex_plot_er()) withAutoprint(\{ # examplesIf}
\donttest{
data(d_sim_binom_cov_hgly2)

ermod_bin <- dev_ermod_bin(
  data = d_sim_binom_cov_hgly2,
  var_resp = "AEFLAG",
  var_exposure = "AUCss_1000"
)

plot_er_gof(ermod_bin, var_group = "Dose_mg", show_coef_exp = TRUE) *
  # Use log10 scale for exposure, need to use `*` instead of `+`
  xgxr::xgx_scale_x_log10(guide = ggplot2::guide_axis(minor.ticks = TRUE))

# Customize plot components
comps <- plot_er_gof(ermod_bin, var_group = "Dose_mg", return_components = TRUE)
comps$main <- comps$main +
  ggplot2::labs(title = "Exposure-Response Analysis", x = "AUC (ng*h/mL)") +
  ggplot2::theme_bw()
combine_er_components(comps)
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_ermod.R
\name{plot_er}
\alias{plot_er}
\alias{plot_er.ersim_med_qi}
\alias{plot_er.ersim}
\alias{plot_er.ermod}
\title{Plot ER model simulations}
\usage{
plot_er(x, ...)

\method{plot_er}{ersim_med_qi}(
  x,
  show_orig_data = FALSE,
  show_coef_exp = FALSE,
  show_caption = FALSE,
  options_orig_data = list(),
  options_coef_exp = list(),
  options_caption = list(),
  ...
)

\method{plot_er}{ersim}(
  x,
  show_orig_data = FALSE,
  show_coef_exp = FALSE,
  show_caption = FALSE,
  options_orig_data = list(),
  options_coef_exp = list(),
  options_caption = list(),
  qi_width_sim = 0.95,
  ...
)

\method{plot_er}{ermod}(
  x,
  show_orig_data = FALSE,
  show_coef_exp = FALSE,
  show_caption = FALSE,
  options_orig_data = list(),
  options_coef_exp = list(),
  options_caption = list(),
  n_draws_sim = if (marginal) 200 else NULL,
  seed_sample_draws = NULL,
  marginal = FALSE,
  exposure_range = NULL,
  num_exposures = 51,
  qi_width_sim = 0.95,
  ...
)
}
\arguments{
\item{x}{an object of class \code{ermod}, \code{ersim},\code{ersim_med_qi},
or their subclasses}

\item{...}{currently not used}

\item{show_orig_data}{logical, whether to show the data points in the
model development dataset. Default is \code{FALSE}. Only support plotting
with data that was used in the model development. If you want to use
other data, consider adding geom_point() to the plot manually.}

\item{show_coef_exp}{logical, whether to show the credible interval
of the exposure coefficient. Default is \code{FALSE}. This is only available
for linear and linear logistic regression models.}

\item{show_caption}{logical, whether to show the caption note for the plot.
Default is \code{FALSE}.}

\item{options_orig_data}{List of options for configuring how original data
is displayed. Possible options include:
\itemize{
\item \code{add_boxplot}: Logical, whether to add a boxplot of exposure values.
Default is \code{FALSE}.
\item \code{boxplot_height}: Height of the boxplot relative to the main plot.
Default is \code{0.15}.
\item \code{show_boxplot_y_title}: Logical, whether to show the y-axis title
for the boxplot. Default is \code{TRUE}.
\item \code{var_group}: The column to use for grouping data for plotting.
If specified, observed data points and boxplot will be grouped
and colored by this column. Default is \code{NULL}.
\item \code{n_bins}: Number of bins to use for observed probability
summary. Only relevant for binary models. Default is \code{4}.
\item \code{qi_width}: Width of the quantile interval (confidence interval) for
the observed probability summary. Only relevant for binary models.
Default is \code{0.95}.
\item \code{return_components}: Logical, whether to return plot components as a
list instead of a combined plot. When \code{TRUE}, returns a list with
\verb{$main} (main plot), \verb{$boxplot} (boxplot if applicable), \verb{$caption}
(caption text if applicable), and \verb{$metadata}. Default is \code{FALSE}.
}}

\item{options_coef_exp}{List of options for configuring how the exposure
coefficient credible interval is displayed. Possible options include:
\itemize{
\item \code{qi_width}: Width of the quantile interval (credible interval) for
the exposure coefficient. Default is \code{0.95}.
\item \code{n_sigfig}: Number of significant figures to display. Default is \code{3}.
\item \code{pos_x}: x-coordinate of the text label. If \code{NULL} (default), it is
set to the minimum value for the exposure variable.
\item \code{pos_y}: y-coordinate of the text label. If \code{NULL} (default), it is
set to 0.9 for logistic regression models and the maximum value of the
response variable in the original data for linear regression models.
\item \code{size}: Size of the text label. Default is \code{4}.
}}

\item{options_caption}{List of options for configuring the caption note.
Possible options include:
\itemize{
\item \code{orig_data}: Logical, whether to show the caption note for the
observed data. Default is \code{FALSE}.
\item \code{orig_data_summary}: Logical, whether to show the caption note for the
observed data summary. Default is \code{FALSE}.
Only relevant for binary models.
\item \code{coef_exp}: Logical, whether to show the caption note for the
exposure coefficient credible interval. Default is \code{FALSE}.
}}

\item{qi_width_sim}{Width of the quantile interval to summarize simulated
draws.}

\item{n_draws_sim}{Number of draws to simulate response for each exposure
value. Set to NULL to use all draws in the model object. Default is NULL
unless marginal is set to TRUE (in that case 200 by default to reduce
computation time).}

\item{seed_sample_draws}{Seed for sampling draws. Default is NULL.}

\item{marginal}{logical, whether to use marginal ER simulation. Default
to \code{FALSE}. Need to set to \code{TRUE} if the model has covariates for the
plot to work.}

\item{exposure_range}{Only relevant when the input x is an \code{ermod} object.
Range of exposure values to simulate. If NULL
(default), it is set to the range of the exposure variable in the original
data for model development.}

\item{num_exposures}{Only relevant as with \code{exposure_range}.
Number of exposure values to simulate.}
}
\value{
A ggplot object
}
\description{
Plot ER model simulations
}
\details{
Plotting with \code{ermod} is done with some default values. If they are not
suitable, you can always perform the simulation manually and use
\code{plot_er()} on the simulated data.
}
\examples{
\dontshow{if (BayesERtools:::.if_run_ex_plot_er()) withAutoprint(\{ # examplesIf}
\donttest{
data(d_sim_binom_cov_hgly2)

ermod_bin <- dev_ermod_bin(
  data = d_sim_binom_cov_hgly2,
  var_resp = "AEFLAG",
  var_exposure = "AUCss_1000"
)

ersim_med_qi <- sim_er_curve(
  ermod_bin,
  output_type = "median_qi"
)

plot_er(ersim_med_qi, show_orig_data = TRUE) +
  # Use log10 scale for exposure
  xgxr::xgx_scale_x_log10(guide = ggplot2::guide_axis(minor.ticks = TRUE))
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ersim-methods.R
\name{calc_ersim_med_qi}
\alias{calc_ersim_med_qi}
\title{Calculate median and quantile intervals from ersim object}
\usage{
calc_ersim_med_qi(x, qi_width = 0.95)
}
\arguments{
\item{x}{An object of class \code{ersim} or \code{ersim_marg}}

\item{qi_width}{Width of the quantile interval}
}
\value{
An object of class \code{ersim_med_qi} or \code{ersim_marg_med_qi}
}
\description{
This is useful when you performed simulation with output_type = "draws" and
want to calculate median and quantile intervals without re-simulating.
}
